@@ -11,5 +11,5 @@
 #import "PSMYosemiteTabStyle.h"
 
 @interface PSMDarkTabStyle : PSMYosemiteTabStyle<PSMTabStyle>
-+ (NSColor *)tabBarColor;
++ (NSColor *)tabBarColorWhenMainAndActive:(BOOL)keyMainAndActive;
 @end
