@@ -6,10 +6,15 @@
 //
 //
 
-#import <Foundation/Foundation.h>
 #include <assert.h>
 
+#ifndef __OBJC__
+#include "iTermCLogging.h"
+#else
+// Rest of the file is Obj-C code path
+#import <Foundation/Foundation.h>
 extern BOOL gDebugLogging;
+#include "iTermCLogging.h"
 
 #define USE_STOPWATCH 0
 
@@ -48,7 +53,7 @@ extern BOOL gDebugLogging;
 
 
 // I use a variadic macro here because of an apparent compiler bug in XCode 4.2 that thinks a
-// variadaic objc call as an argument is not a single value.
+// variadic objc call as an argument is not a single value.
 #define DebugLog(args...) DebugLogImpl(__FILE__, __LINE__, __FUNCTION__, args)
 
 //#define GENERAL_VERBOSE_LOGGING
@@ -63,18 +68,24 @@ extern BOOL gDebugLogging;
         } \
     } while (0)
 
+// Write to crash log immediately and unconditionally. Use this when you expect to crash right away.
+#define CrashLog(args...) \
+    do { \
+        LogForNextCrash(__FILE__, __LINE__, __FUNCTION__, [NSString stringWithFormat:args], YES); \
+    } while (0)
+
 // Info log: no private info. Low-volume. Logged to crash reports.
 #define ILog(args...) \
     do { \
         DLog(args); \
-        LogForNextCrash(__FILE__, __LINE__, __FUNCTION__, [NSString stringWithFormat:args]); \
+        LogForNextCrash(__FILE__, __LINE__, __FUNCTION__, [NSString stringWithFormat:args], NO); \
     } while (0)
 
 // Error log: no private info. Low-volume. Logged to crash reports.
 #define ELog(args...) \
     do { \
         DLog(args); \
-        LogForNextCrash(__FILE__, __LINE__, __FUNCTION__, [NSString stringWithFormat:args]); \
+        LogForNextCrash(__FILE__, __LINE__, __FUNCTION__, [NSString stringWithFormat:args], NO); \
         NSLog(args); \
     } while (0)
 
@@ -97,6 +108,12 @@ extern BOOL gDebugLogging;
     } \
   } while (0)
 
+#if __has_feature(objc_arc)
+#define ITCriticalErrorCreateAlert [[NSAlert alloc] init]
+#else
+#define ITCriticalErrorCreateAlert [[[NSAlert alloc] init] autorelease]
+#endif
+
 #define ITCriticalError(condition, args...) \
   do { \
     if (!(condition)) { \
@@ -112,7 +129,7 @@ extern BOOL gDebugLogging;
       ELog(args); \
       if (TurnOffDebugLoggingSilently()) { \
         dispatch_async(dispatch_get_main_queue(), ^{ \
-          NSAlert *alert = [[[NSAlert alloc] init] autorelease]; \
+          NSAlert *alert = ITCriticalErrorCreateAlert; \
           alert.messageText = @"Critical Error"; \
           alert.informativeText =  @"A critical error occurred and a debug log was created. Please send /tmp/debuglog.txt to the developers."; \
           [alert addButtonWithTitle:@"OK"]; \
@@ -124,6 +141,25 @@ extern BOOL gDebugLogging;
 
 #define IT_STRINGIFY(x) #x
 
+#if BETA
+#define ITBetaAssertSampled(percentage, condition, args...) \
+  do { \
+    if (!(condition) && arc4random_uniform(100) < percentage) { \
+      DLog(@"Crashing because %s from:\n%@", #condition, [NSThread callStackSymbols]); \
+      ELog(args); \
+      assert(NO); \
+    } \
+  } while (0)
+#else  // BETA
+#define ITBetaAssertSampled(percentage, condition, args...) \
+  do { \
+    if (!(condition) && arc4random_uniform(100) < percentage) { \
+      ELog(@"BETA ASSERT: Failed beta assert because %s from:\n%@", #condition, [NSThread callStackSymbols]); \
+      ELog(args); \
+    } \
+  } while (0)
+#endif
+
 #if BETA
 #define ITBetaAssert(condition, args...) \
   do { \
@@ -143,11 +179,41 @@ extern BOOL gDebugLogging;
   } while (0)
 #endif
 
+#if BETA
+#define ITConservativeBetaAssert(condition, args...) \
+  do { \
+    if (!(condition)) { \
+      DLog(@"Crashing because %s from:\n%@", #condition, [NSThread callStackSymbols]); \
+      ELog(args); \
+      assert(NO); \
+    } \
+  } while (0)
+#else  // BETA
+#define ITConservativeBetaAssert(condition, args...)
+#endif
+
+#define ITAssertWithMessage(condition, args...) \
+    do { \
+        if (!(condition)) { \
+            DLog(@"Crashing because %s from:\n%@", #condition, [NSThread callStackSymbols]); \
+            ELog(args); \
+            __assert_rtn(__func__, __FILE__, __LINE__, [[NSString stringWithFormat:@#condition ": " args] UTF8String]); \
+        } \
+    } while (0)
+
 void ToggleDebugLogging(void);
 int DebugLogImpl(const char *file, int line, const char *function, NSString* value);
-void LogForNextCrash(const char *file, int line, const char *function, NSString* value);
+void LogForNextCrash(const char *file, int line, const char *function, NSString* value, BOOL force);
 void TurnOnDebugLoggingSilently(void);
 BOOL TurnOffDebugLoggingSilently(void);
 
 void SetPinnedDebugLogMessage(NSString *key, NSString *value, ...);
 void AppendPinnedDebugLogMessage(NSString *key, NSString *value, ...);
+
+@interface NSException(iTerm)
+@property (nonatomic, readonly) NSArray<NSString *> *it_originalCallStackSymbols;
+@property (nonatomic, readonly) NSString *it_compressedDescription;
+- (NSException *)it_rethrowWithMessage:(NSString *)format, ...;
+@end
+
+#endif  // __OBJC__
