@@ -29,8 +29,14 @@
 
 #import "DebugLogging.h"
 #import "iTermDynamicProfileManager.h"
-#import "iTermKeyBindingMgr.h"
+#import "iTermExpressionEvaluator.h"
+#import "iTermHotKeyController.h"
+#import "iTermHotKeyMigrationHelper.h"
+#import "iTermHotKeyProfileBindingController.h"
+#import "iTermKeyMappings.h"
+#import "iTermMigrationHelper.h"
 #import "iTermPreferences.h"
+#import "iTermProfilesMenuController.h"
 #import "iTermProfilePreferences.h"
 #import "PreferencePanel.h"
 #import "ProfileModel.h"
@@ -39,7 +45,82 @@
 #import "NSFont+iTerm.h"
 #include <arpa/inet.h>
 
+NSString *const iTermUnicodeVersionDidChangeNotification = @"iTermUnicodeVersionDidChangeNotification";
+
+NSString *const kProfilePreferenceCommandTypeCustomValue = @"Yes";
+NSString *const kProfilePreferenceCommandTypeLoginShellValue = @"No";
+NSString *const kProfilePreferenceCommandTypeCustomShellValue = @"Custom Shell";
+
 const NSTimeInterval kMinimumAntiIdlePeriod = 1.0;
+const NSInteger iTermMaxInitialSessionSize = 1250;
+
+static NSMutableArray<NSNotification *> *sDelayedNotifications;
+
+iTermWindowType iTermWindowDefaultType(void) {
+    return iTermThemedWindowType(WINDOW_TYPE_NORMAL);
+}
+
+iTermWindowType iTermThemedWindowType(iTermWindowType windowType) {
+    switch (windowType) {
+        case WINDOW_TYPE_COMPACT:
+        case WINDOW_TYPE_NORMAL:
+            if (@available(macOS 10.14, *)) {} else {
+                // 10.13 and earlier do not support compact
+                return WINDOW_TYPE_NORMAL;
+            }
+            switch ((iTermPreferencesTabStyle)[iTermPreferences intForKey:kPreferenceKeyTabStyle]) {
+                case TAB_STYLE_COMPACT:
+                case TAB_STYLE_MINIMAL:
+                    return WINDOW_TYPE_COMPACT;
+
+                case TAB_STYLE_AUTOMATIC:
+                case TAB_STYLE_LIGHT:
+                case TAB_STYLE_DARK:
+                case TAB_STYLE_LIGHT_HIGH_CONTRAST:
+                case TAB_STYLE_DARK_HIGH_CONTRAST:
+                    return WINDOW_TYPE_NORMAL;
+            }
+            assert(false);
+            return windowType;
+
+        case WINDOW_TYPE_COMPACT_MAXIMIZED:
+        case WINDOW_TYPE_MAXIMIZED:
+            if (@available(macOS 10.14, *)) {} else {
+                // 10.13 and earlier do not support compact
+                return WINDOW_TYPE_MAXIMIZED;
+            }
+            switch ((iTermPreferencesTabStyle)[iTermPreferences intForKey:kPreferenceKeyTabStyle]) {
+                case TAB_STYLE_COMPACT:
+                case TAB_STYLE_MINIMAL:
+                    return WINDOW_TYPE_COMPACT_MAXIMIZED;
+
+                case TAB_STYLE_AUTOMATIC:
+                case TAB_STYLE_LIGHT:
+                case TAB_STYLE_DARK:
+                case TAB_STYLE_LIGHT_HIGH_CONTRAST:
+                case TAB_STYLE_DARK_HIGH_CONTRAST:
+                    return WINDOW_TYPE_MAXIMIZED;
+            }
+            assert(false);
+            return windowType;
+
+        case WINDOW_TYPE_TOP:
+        case WINDOW_TYPE_LEFT:
+        case WINDOW_TYPE_RIGHT:
+        case WINDOW_TYPE_BOTTOM:
+        case WINDOW_TYPE_ACCESSORY:
+        case WINDOW_TYPE_TRADITIONAL_FULL_SCREEN:
+        case WINDOW_TYPE_LION_FULL_SCREEN:
+        case WINDOW_TYPE_TOP_PARTIAL:
+        case WINDOW_TYPE_LEFT_PARTIAL:
+        case WINDOW_TYPE_BOTTOM_PARTIAL:
+        case WINDOW_TYPE_RIGHT_PARTIAL:
+        case WINDOW_TYPE_NO_TITLE_BAR:
+            return windowType;
+    }
+    ITAssertWithMessage(NO, @"Unknown window type %@", @(windowType));
+    return WINDOW_TYPE_NORMAL;
+}
 
 @implementation ITAddressBookMgr {
     NSNetServiceBrowser *sshBonjourBrowser;
@@ -69,15 +150,17 @@ const NSTimeInterval kMinimumAntiIdlePeriod = 1.0;
             ![prefs objectForKey:KEY_NEW_BOOKMARKS]) {
             // Have only old-style bookmarks. Load them and convert them to new-style
             // bookmarks.
-            [self recursiveMigrateBookmarks:[prefs objectForKey:KEY_DEPRECATED_BOOKMARKS] path:@[]];
+            [iTermMigrationHelper recursiveMigrateBookmarks:[prefs objectForKey:KEY_DEPRECATED_BOOKMARKS] path:@[]];
             [prefs removeObjectForKey:KEY_DEPRECATED_BOOKMARKS];
             [prefs setObject:[[ProfileModel sharedInstance] rawData] forKey:KEY_NEW_BOOKMARKS];
             [[ProfileModel sharedInstance] removeAllBookmarks];
         }
 
+        iTermProfilesMenuController *menuController = [[iTermProfilesMenuController alloc] init];
+        [[ProfileModel sharedInstance] setMenuController:menuController];
         // Load new-style bookmarks.
         id newBookmarks = [prefs objectForKey:KEY_NEW_BOOKMARKS];
-        NSString *originalDefaultGuid = [[[prefs objectForKey:KEY_DEFAULT_GUID] copy] autorelease];
+        NSString *originalDefaultGuid = [[prefs objectForKey:KEY_DEFAULT_GUID] copy];
         if ([newBookmarks isKindOfClass:[NSArray class]]) {
             [self setBookmarks:newBookmarks
                    defaultGuid:[prefs objectForKey:KEY_DEFAULT_GUID]];
@@ -105,10 +188,10 @@ const NSTimeInterval kMinimumAntiIdlePeriod = 1.0;
 
         // Make sure there is at least one bookmark.
         if ([[ProfileModel sharedInstance] numberOfBookmarks] == 0) {
-            NSMutableDictionary* aDict = [[NSMutableDictionary alloc] init];
+            NSMutableDictionary *aDict = [[NSMutableDictionary alloc] init];
             [ITAddressBookMgr setDefaultsInBookmark:aDict];
             [[ProfileModel sharedInstance] addBookmark:aDict];
-            [aDict release];
+            [[ProfileModel sharedInstance] flush];
         }
 
         if ([iTermPreferences boolForKey:kPreferenceKeyAddBonjourHostsToProfiles]) {
@@ -133,6 +216,9 @@ const NSTimeInterval kMinimumAntiIdlePeriod = 1.0;
             // One of the dynamic profiles has the default guid.
             [[ProfileModel sharedInstance] setDefaultByGuid:originalDefaultGuid];
         }
+
+        [[iTermHotKeyMigrationHelper sharedInstance] migrateSingleHotkeyToMulti];
+        [[iTermHotKeyProfileBindingController sharedInstance] refresh];
     }
 
     return self;
@@ -140,15 +226,10 @@ const NSTimeInterval kMinimumAntiIdlePeriod = 1.0;
 
 - (void)dealloc {
     [bonjourServices removeAllObjects];
-    [bonjourServices release];
 
     [sshBonjourBrowser stop];
     [ftpBonjourBrowser stop];
     [telnetBonjourBrowser stop];
-    [sshBonjourBrowser release];
-    [ftpBonjourBrowser release];
-    [telnetBonjourBrowser release];
-    [super dealloc];
 }
 
 - (void)removeBonjourProfiles {
@@ -185,18 +266,14 @@ const NSTimeInterval kMinimumAntiIdlePeriod = 1.0;
 
 - (void)stopLocatingBonjourServices {
     [sshBonjourBrowser stop];
-    [sshBonjourBrowser release];
     sshBonjourBrowser = nil;
 
     [ftpBonjourBrowser stop];
-    [ftpBonjourBrowser release];
     ftpBonjourBrowser = nil;
 
     [telnetBonjourBrowser stop];
-    [telnetBonjourBrowser release];
     telnetBonjourBrowser = nil;
 
-    [bonjourServices release];
     bonjourServices = nil;
 }
 
@@ -204,99 +281,15 @@ const NSTimeInterval kMinimumAntiIdlePeriod = 1.0;
     return [origColor dictionaryValue];
 }
 
-// This method always returns a color in the calibrated color space. If the
-// color space in the plist is not calibrated, it is converted (which preserves
-// the actual color values).
 + (NSColor *)decodeColor:(NSDictionary*)plist {
     return [plist colorValue];
 }
 
-- (void)copyProfileToBookmark:(NSMutableDictionary *)dict
-{
-    NSString* plistFile = [[NSBundle bundleForClass:[self class]] pathForResource:@"MigrationMap"
-                                                                           ofType:@"plist"];
-    NSDictionary* fileDict = [NSDictionary dictionaryWithContentsOfFile: plistFile];
-    NSUserDefaults* prefs = [NSUserDefaults standardUserDefaults];
-    NSDictionary* keybindingProfiles = [prefs objectForKey: @"KeyBindings"];
-    NSDictionary* displayProfiles =  [prefs objectForKey: @"Displays"];
-    NSDictionary* terminalProfiles = [prefs objectForKey: @"Terminals"];
-    NSArray* xforms = [fileDict objectForKey:@"Migration Map"];
-    for (int i = 0; i < [xforms count]; ++i) {
-        NSDictionary* xform = [xforms objectAtIndex:i];
-        NSString* destination = [xform objectForKey:@"Destination"];
-        if ([dict objectForKey:destination]) {
-            continue;
-        }
-        NSString* prefix = [xform objectForKey:@"Prefix"];
-        NSString* suffix = [xform objectForKey:@"Suffix"];
-        id defaultValue = [xform objectForKey:@"Default"];
-
-        NSDictionary* parent = nil;
-        if ([prefix isEqualToString:@"Terminal"]) {
-            parent = [terminalProfiles objectForKey:[dict objectForKey:KEY_TERMINAL_PROFILE]];
-        } else if ([prefix isEqualToString:@"Displays"]) {
-            parent = [displayProfiles objectForKey:[dict objectForKey:KEY_DISPLAY_PROFILE]];
-        } else if ([prefix isEqualToString:@"KeyBindings"]) {
-            parent = [keybindingProfiles objectForKey:[dict objectForKey:KEY_KEYBOARD_PROFILE]];
-        } else {
-            NSAssert(0, @"Bad prefix");
-        }
-        id value = nil;
-        if (parent) {
-            value = [parent objectForKey:suffix];
-        }
-        if (!value) {
-            value = defaultValue;
-        }
-        [dict setObject:value forKey:destination];
-    }
-}
-
-- (void)recursiveMigrateBookmarks:(NSDictionary*)node path:(NSArray*)path
-{
-    NSDictionary* data = [node objectForKey:@"Data"];
-
-    if ([data objectForKey:KEY_COMMAND_LINE]) {
-        // Not just a folder if it has a command.
-        NSMutableDictionary* temp = [NSMutableDictionary dictionaryWithDictionary:data];
-        [self copyProfileToBookmark:temp];
-        [temp setObject:[ProfileModel freshGuid] forKey:KEY_GUID];
-        [temp setObject:path forKey:KEY_TAGS];
-        [temp setObject:@"Yes" forKey:KEY_CUSTOM_COMMAND];
-        NSString* dir = [data objectForKey:KEY_WORKING_DIRECTORY];
-        if (dir && [dir length] > 0) {
-            [temp setObject:kProfilePreferenceInitialDirectoryCustomValue
-                     forKey:KEY_CUSTOM_DIRECTORY];
-        } else if (dir && [dir length] == 0) {
-            [temp setObject:kProfilePreferenceInitialDirectoryRecycleValue
-                     forKey:KEY_CUSTOM_DIRECTORY];
-        } else {
-            [temp setObject:kProfilePreferenceInitialDirectoryHomeValue
-                     forKey:KEY_CUSTOM_DIRECTORY];
-        }
-        [[ProfileModel sharedInstance] addBookmark:temp];
-    }
-
-    NSArray* entries = [node objectForKey:@"Entries"];
-    for (int i = 0; i < [entries count]; ++i) {
-        NSMutableArray* childPath = [NSMutableArray arrayWithArray:path];
-        NSDictionary* dataDict = [node objectForKey:@"Data"];
-        if (dataDict) {
-            NSString* name = [dataDict objectForKey:@"Name"];
-            if (name) {
-                [childPath addObject:name];
-            }
-        }
-        [self recursiveMigrateBookmarks:[entries objectAtIndex:i] path:childPath];
-    }
-}
-
 + (NSFont *)fontWithDesc:(NSString *)fontDesc {
     return [fontDesc fontValue];
 }
 
-- (void)setBookmarks:(NSArray*)newBookmarksArray defaultGuid:(NSString*)guid
-{
+- (void)setBookmarks:(NSArray *)newBookmarksArray defaultGuid:(NSString *)guid {
     [[ProfileModel sharedInstance] load:newBookmarksArray];
     if (guid) {
         if ([[ProfileModel sharedInstance] bookmarkWithGuid:guid]) {
@@ -342,7 +335,7 @@ const NSTimeInterval kMinimumAntiIdlePeriod = 1.0;
     }
 
     // remove host entry from this group
-    NSMutableArray* toRemove = [[[NSMutableArray alloc] init] autorelease];
+    NSMutableArray *toRemove = [[NSMutableArray alloc] init];
 #ifdef SUPPORT_SFTP
     NSString* sftpName = [NSString stringWithFormat:@"%@-sftp", [aNetService name]];
 #endif
@@ -383,7 +376,7 @@ const NSTimeInterval kMinimumAntiIdlePeriod = 1.0;
                                                [NSBundle bundleForClass: [self class]],
                                                @"Terminal Profiles");
     [aDict setObject:aName forKey: KEY_NAME];
-    [aDict setObject:@"No" forKey:KEY_CUSTOM_COMMAND];
+    [aDict setObject:kProfilePreferenceCommandTypeLoginShellValue forKey:KEY_CUSTOM_COMMAND];
     [aDict setObject:@"" forKey: KEY_COMMAND_LINE];
     [aDict setObject:aName forKey: KEY_DESCRIPTION];
     [aDict setObject:kProfilePreferenceInitialDirectoryHomeValue
@@ -391,10 +384,23 @@ const NSTimeInterval kMinimumAntiIdlePeriod = 1.0;
     [aDict setObject:NSHomeDirectory() forKey: KEY_WORKING_DIRECTORY];
 }
 
+- (BOOL)usernameIsSafe:(NSString *)username {
+    NSCharacterSet *unsafeSet = [[NSCharacterSet characterSetWithCharactersInString:@"abcdefghijklmnopqrstuvwxyz_1234567890"] invertedSet];
+    const NSRange range = [username rangeOfCharacterFromSet:unsafeSet
+                                                    options:NSCaseInsensitiveSearch];
+    return range.location == NSNotFound;
+}
+
 - (void)_addBonjourHostProfileWithName:(NSString *)serviceName
-                       ipAddressString:(NSString *)ipAddressString
+                       ipAddressString:(NSString *)address
                                   port:(int)port
-                           serviceType:(NSString *)serviceType {
+                           serviceType:(NSString *)serviceType
+                              userName:(NSString *)username {
+    NSArray<NSString *> *allowedServices = @[ @"ssh", @"scp", @"sftp" ];
+    if (![allowedServices containsObject:serviceType]) {
+        return;
+    }
+
     NSMutableDictionary *newBookmark;
     Profile* prototype = [[ProfileModel sharedInstance] defaultBookmark];
     if (prototype) {
@@ -411,13 +417,22 @@ const NSTimeInterval kMinimumAntiIdlePeriod = 1.0;
     if ([serviceType isEqualToString:@"ssh"] && port != 22) {
         optionalPortArg = [NSString stringWithFormat:@"-p %d ", port];
     }
-    [newBookmark setObject:[NSString stringWithFormat:@"%@ %@%@", serviceType, optionalPortArg, ipAddressString]
+    NSString *userNameArg = @"";
+    NSString *destination = address;
+    if (username.length > 0 && [self usernameIsSafe:username]) {
+        if ([serviceType isEqualToString:@"ssh"]) {
+            userNameArg = [NSString stringWithFormat:@"-l %@ ", username];
+        } else {
+            destination = [NSString stringWithFormat:@"%@@%@", username, address];
+        }
+    }
+    [newBookmark setObject:[NSString stringWithFormat:@"%@ %@%@%@", serviceType, userNameArg, optionalPortArg, destination]
                     forKey:KEY_COMMAND_LINE];
     [newBookmark setObject:@"" forKey:KEY_WORKING_DIRECTORY];
-    [newBookmark setObject:@"Yes" forKey:KEY_CUSTOM_COMMAND];
+    [newBookmark setObject:kProfilePreferenceCommandTypeCustomValue forKey:KEY_CUSTOM_COMMAND];
     [newBookmark setObject:kProfilePreferenceInitialDirectoryHomeValue
                     forKey:KEY_CUSTOM_DIRECTORY];
-    [newBookmark setObject:ipAddressString forKey:KEY_BONJOUR_SERVICE_ADDRESS];
+    [newBookmark setObject:destination forKey:KEY_BONJOUR_SERVICE_ADDRESS];
     [newBookmark setObject:[NSArray arrayWithObjects:@"bonjour",nil] forKey:KEY_TAGS];
     [newBookmark setObject:[ProfileModel freshGuid] forKey:KEY_GUID];
     [newBookmark setObject:@"No" forKey:KEY_DEFAULT_BOOKMARK];
@@ -456,6 +471,14 @@ const NSTimeInterval kMinimumAntiIdlePeriod = 1.0;
     }
 }
 
+- (NSString *)usernameFromTXTRecord:(NSData *)txtData {
+    NSDictionary<NSString *, NSData *> *txtFields = [NSNetService dictionaryFromTXTRecordData:txtData];
+    // https://kodi.wiki/view/Avahi_Zeroconf
+    NSData *usernameData = txtFields[@"u"] ?: txtFields[@"username"];
+    NSString *usernameString = [[NSString alloc] initWithData:usernameData encoding:NSUTF8StringEncoding];
+    return usernameString;
+}
+
 // NSNetService delegate
 - (void)netServiceDidResolveAddress:(NSNetService *)sender
 {
@@ -497,10 +520,13 @@ const NSTimeInterval kMinimumAntiIdlePeriod = 1.0;
         if ([self verbose]) {
             NSLog(@"netServiceDidResolveAddress add profile with address %s", strAddr);
         }
+        
+        NSString *username = [self usernameFromTXTRecord:sender.TXTRecordData];
         [self _addBonjourHostProfileWithName:serviceName
                              ipAddressString:[NSString stringWithFormat:@"%s", strAddr]
                                         port:[self portFromSockaddr:socketAddress]
-                                 serviceType:serviceType];
+                                 serviceType:serviceType
+                                    userName:username];
 
         // remove from array now that resolving is done
         if ([bonjourServices containsObject:sender]) {
@@ -551,43 +577,55 @@ const NSTimeInterval kMinimumAntiIdlePeriod = 1.0;
     }
 }
 
-+ (NSString *)shellLauncherCommand {
-    return [NSString stringWithFormat:@"/usr/bin/login -f%@pl %@ %@ --launch_shell",
-            [self hushlogin] ? @"q" : @"",
-            [NSUserName() stringWithEscapedShellCharacters],
-            [[[NSBundle mainBundle] executablePath] stringWithEscapedShellCharacters]];
++ (NSString *)sanitizedCustomShell:(NSString *)customShell {
+    NSArray<NSString *> *parts = [customShell componentsSeparatedByCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
+    if ([parts[0] length] == 0) {
+        return nil;
+    }
+    return parts[0];
 }
 
-+ (NSString*)loginShellCommandForBookmark:(Profile*)bookmark
++ (NSString *)shellLauncherCommandWithCustomShell:(NSString *)customShell {
+    NSString *sanitizedCustomShell = [self sanitizedCustomShell:customShell];
+    NSString *customShellArg = sanitizedCustomShell ? [@" SHELL=" stringByAppendingString:sanitizedCustomShell] : @"";
+    return [NSString stringWithFormat:@"/usr/bin/login -f%@pl %@ %@ --launch_shell%@",
+            [self hushlogin] ? @"q" : @"",
+            [NSUserName() stringWithBackslashEscapedShellCharactersIncludingNewlines:YES],
+            [[[NSBundle mainBundle] executablePath] stringWithBackslashEscapedShellCharactersIncludingNewlines:YES],
+            customShellArg];
+}
+
++ (NSString*)loginShellCommandForBookmark:(Profile*)profile
                             forObjectType:(iTermObjectType)objectType {
     NSString *customDirectoryString;
-    if ([[bookmark objectForKey:KEY_CUSTOM_DIRECTORY] isEqualToString:kProfilePreferenceInitialDirectoryAdvancedValue]) {
+    if ([profile[KEY_CUSTOM_DIRECTORY] isEqualToString:kProfilePreferenceInitialDirectoryAdvancedValue]) {
         switch (objectType) {
             case iTermWindowObject:
-                customDirectoryString = [bookmark objectForKey:KEY_AWDS_WIN_OPTION];
+                customDirectoryString = profile[KEY_AWDS_WIN_OPTION];
                 break;
             case iTermTabObject:
-                customDirectoryString = [bookmark objectForKey:KEY_AWDS_TAB_OPTION];
+                customDirectoryString = profile[KEY_AWDS_TAB_OPTION];
                 break;
             case iTermPaneObject:
-                customDirectoryString = [bookmark objectForKey:KEY_AWDS_PANE_OPTION];
+                customDirectoryString = profile[KEY_AWDS_PANE_OPTION];
                 break;
             default:
                 NSLog(@"Bogus object type %d", (int)objectType);
                 customDirectoryString = kProfilePreferenceInitialDirectoryHomeValue;
         }
     } else {
-        customDirectoryString = [bookmark objectForKey:KEY_CUSTOM_DIRECTORY];
+        customDirectoryString = profile[KEY_CUSTOM_DIRECTORY];
     }
 
-    if ([customDirectoryString isEqualToString:kProfilePreferenceInitialDirectoryHomeValue]) {
+    if ([customDirectoryString isEqualToString:kProfilePreferenceInitialDirectoryHomeValue] &&
+        [[self customShellForProfile:profile] length] == 0) {
         // Run login without -l argument: this is a login session and will use the home dir.
         return [self standardLoginCommand];
     } else {
-        // Not using the home directory. This requires some trickery.
+        // Not using the home directory/default shell. This requires some trickery.
         // Run iTerm2's executable with a special flag that makes it run the shell as a login shell
         // (with "-" inserted at the start of argv[0]). See shell_launcher.c for more details.
-        NSString *launchShellCommand = [self shellLauncherCommand];
+        NSString *launchShellCommand = [self shellLauncherCommandWithCustomShell:[self customShellForProfile:profile]];
         return launchShellCommand;
     }
 }
@@ -599,19 +637,74 @@ const NSTimeInterval kMinimumAntiIdlePeriod = 1.0;
 }
 
 + (NSString *)standardLoginCommand {
-    return [NSString stringWithFormat:@"login -f%@p \"%@\"", [self hushlogin] ? @"q" : @"", NSUserName()];
+    NSString *userName = NSUserName();
+    // Active directory users have backslash in their user name (issue 6999)
+    // Somehow, users can have spaces in their user name (issue 8360)
+    //
+    // Avoid using standard escaping which is wrong for a quoted string. I don't know why
+    // this is in quotes, but I'm afraid to change it because it's been that way for so
+    // long and the original commit message was lost.
+    //
+    // The returned value gets parsed into an argument array using -componentsInShellCommand
+    // by computeArgvForCommand:substitutions:completion:.
+    userName = [userName stringByReplacingOccurrencesOfString:@"\\" withString:@"\\\\"];
+    userName = [userName stringByReplacingOccurrencesOfString:@"\"" withString:@"\\\""];
+    return [NSString stringWithFormat:@"login -f%@p \"%@\"", [self hushlogin] ? @"q" : @"",
+            userName];
 }
 
-+ (NSString*)bookmarkCommand:(Profile*)bookmark
-               forObjectType:(iTermObjectType)objectType
-{
-    BOOL custom = [[bookmark objectForKey:KEY_CUSTOM_COMMAND] isEqualToString:@"Yes"];
-    if (custom) {
-        return [bookmark objectForKey:KEY_COMMAND_LINE];
-    } else {
-        return [ITAddressBookMgr loginShellCommandForBookmark:bookmark
-                                                forObjectType:objectType];
++ (void)computeCommandForProfile:(Profile *)profile
+                      objectType:(iTermObjectType)objectType
+                           scope:(iTermVariableScope *)scope
+                      completion:(void (^)(NSString *command))completion {
+    BOOL custom = [profile[KEY_CUSTOM_COMMAND] isEqualToString:kProfilePreferenceCommandTypeCustomValue];
+    NSString *swifty = [self bookmarkCommandSwiftyString:profile forObjectType:objectType];
+    if (!custom) {
+        DLog(@"Don't have a custom command. COmputed command is %@", swifty);
+        completion(swifty);
+        return;
     }
+
+    DLog(@"Must evaluate swifty string: %@", swifty);
+    iTermExpressionEvaluator *evaluator =
+    [[iTermExpressionEvaluator alloc] initWithStrictInterpolatedString:swifty
+                                                                 scope:scope];
+    [evaluator evaluateWithTimeout:5 completion:^(iTermExpressionEvaluator * _Nonnull evaluator) {
+        NSString *string = [NSString castFrom:evaluator.value];
+        DLog(@"Evaluation finished with value %@", string);
+        string = [string stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
+        if (!string.length) {
+            string = [ITAddressBookMgr loginShellCommandForBookmark:profile
+                                                      forObjectType:objectType];
+        }
+        DLog(@"Finish with %@", string);
+        completion(string);
+    }];
+}
+
++ (NSString *)bookmarkCommandSwiftyString:(Profile *)bookmark
+                            forObjectType:(iTermObjectType)objectType {
+    BOOL custom = [bookmark[KEY_CUSTOM_COMMAND] isEqualToString:kProfilePreferenceCommandTypeCustomValue];
+    if (custom) {
+        NSString *command = bookmark[KEY_COMMAND_LINE];
+        if ([[command stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet] ] length] > 0) {
+            return command;
+        }
+    }
+    return [ITAddressBookMgr loginShellCommandForBookmark:bookmark
+                                            forObjectType:objectType];
+}
+
++ (NSString *)customShellForProfile:(Profile *)profile {
+    if (![profile[KEY_CUSTOM_COMMAND] isEqualToString:kProfilePreferenceCommandTypeCustomShellValue]) {
+        return nil;
+    }
+    NSString *customShell = profile[KEY_COMMAND_LINE];
+    customShell = [customShell stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
+    if (customShell.length == 0) {
+        return nil;
+    }
+    return customShell;
 }
 
 + (NSString *)_advancedWorkingDirWithOption:(NSString *)option
@@ -627,34 +720,6 @@ const NSTimeInterval kMinimumAntiIdlePeriod = 1.0;
     }
 }
 
-+ (NSString*)bookmarkWorkingDirectory:(Profile*)bookmark forObjectType:(iTermObjectType)objectType
-{
-    NSString* custom = [bookmark objectForKey:KEY_CUSTOM_DIRECTORY];
-    if ([custom isEqualToString:kProfilePreferenceInitialDirectoryCustomValue]) {
-        return [bookmark objectForKey:KEY_WORKING_DIRECTORY];
-    } else if ([custom isEqualToString:kProfilePreferenceInitialDirectoryRecycleValue]) {
-        return @"";
-    } else if ([custom isEqualToString:kProfilePreferenceInitialDirectoryAdvancedValue]) {
-        switch (objectType) {
-          case iTermWindowObject:
-              return [ITAddressBookMgr _advancedWorkingDirWithOption:[bookmark objectForKey:KEY_AWDS_WIN_OPTION]
-                                                           directory:[bookmark objectForKey:KEY_AWDS_WIN_DIRECTORY]];
-          case iTermTabObject:
-              return [ITAddressBookMgr _advancedWorkingDirWithOption:[bookmark objectForKey:KEY_AWDS_TAB_OPTION]
-                                                           directory:[bookmark objectForKey:KEY_AWDS_TAB_DIRECTORY]];
-          case iTermPaneObject:
-              return [ITAddressBookMgr _advancedWorkingDirWithOption:[bookmark objectForKey:KEY_AWDS_PANE_OPTION]
-                                                           directory:[bookmark objectForKey:KEY_AWDS_PANE_DIRECTORY]];
-          default:
-              NSLog(@"Bogus object type %d", (int)objectType);
-              return NSHomeDirectory();  // Shouldn't happen
-        }
-    } else {
-        // Home dir, custom == "No"
-        return NSHomeDirectory();
-    }
-}
-
 + (BOOL)canRemoveProfile:(NSDictionary *)profile fromModel:(ProfileModel *)model {
     DLog(@"removeProfile called");
     if (!profile) {
@@ -676,12 +741,12 @@ const NSTimeInterval kMinimumAntiIdlePeriod = 1.0;
     return YES;
 }
 
-+ (void)removeProfile:(NSDictionary *)profile fromModel:(ProfileModel *)model {
++ (BOOL)removeProfile:(NSDictionary *)profile fromModel:(ProfileModel *)model {
     NSString *guid = profile[KEY_GUID];
     DLog(@"Remove profile with guid %@...", guid);
     if ([model numberOfBookmarks] == 1) {
         DLog(@"Refusing to remove only profile");
-        return;
+        return NO;
     }
 
     DLog(@"Removing key bindings that reference the guid being removed");
@@ -691,30 +756,76 @@ const NSTimeInterval kMinimumAntiIdlePeriod = 1.0;
 
     // Ensure all profile list views reload their data to avoid issue 4033.
     DLog(@"Posting profile was deleted notification");
-    [[NSNotificationCenter defaultCenter] postNotificationName:kProfileWasDeletedNotification
-                                                        object:nil];
+    [self postNotificationName:kProfileWasDeletedNotification object:nil userInfo:nil];
     [model flush];
+    return YES;
 }
 
 + (void)removeKeyMappingsReferringToGuid:(NSString *)badRef {
     for (NSString* guid in [[ProfileModel sharedInstance] guids]) {
         Profile *profile = [[ProfileModel sharedInstance] bookmarkWithGuid:guid];
-        profile = [iTermKeyBindingMgr removeMappingsReferencingGuid:badRef fromBookmark:profile];
+        profile = [iTermKeyMappings removeKeyMappingsReferencingGuid:badRef fromProfile:profile];
         if (profile) {
             [[ProfileModel sharedInstance] setBookmark:profile withGuid:guid];
         }
     }
     for (NSString* guid in [[ProfileModel sessionsInstance] guids]) {
         Profile* profile = [[ProfileModel sessionsInstance] bookmarkWithGuid:guid];
-        profile = [iTermKeyBindingMgr removeMappingsReferencingGuid:badRef fromBookmark:profile];
+        profile = [iTermKeyMappings removeKeyMappingsReferencingGuid:badRef fromProfile:profile];
         if (profile) {
             [[ProfileModel sessionsInstance] setBookmark:profile withGuid:guid];
         }
     }
-    [iTermKeyBindingMgr removeMappingsReferencingGuid:badRef fromBookmark:nil];
-    [[NSNotificationCenter defaultCenter] postNotificationName:kKeyBindingsChangedNotification
-                                                        object:nil
-                                                      userInfo:nil];
+    [iTermKeyMappings removeKeyMappingsReferencingGuid:badRef fromProfile:nil];
+    [self postNotificationName:kKeyBindingsChangedNotification object:nil userInfo:nil];
+}
+
++ (void)postNotificationName:(NSString *)name object:(id)object userInfo:(id)userInfo {
+    NSNotification *notification = [NSNotification notificationWithName:name object:object userInfo:userInfo];
+    [self postNotification:notification];
+}
+
++ (void)postNotification:(NSNotification *)notification {
+    if (sDelayedNotifications) {
+        for (NSNotification *existing in sDelayedNotifications) {
+            if ([existing.name isEqualToString:notification.name] &&
+                (existing.object == notification.object || [existing.object isEqual:notification.object]) &&
+                (existing.userInfo == notification.userInfo || [existing.userInfo isEqual:notification.userInfo])) {
+                // Already have a notification like this
+                return;
+            }
+        }
+        [sDelayedNotifications addObject:notification];
+    } else {
+        [[NSNotificationCenter defaultCenter] postNotification:notification];
+    }
+}
+
++ (void)performBlockWithCoalescedNotifications:(void (^)(void))block {
+    if (!sDelayedNotifications) {
+        sDelayedNotifications = [[NSMutableArray alloc] init];
+
+        block();
+
+        for (NSNotification *notification in sDelayedNotifications) {
+            [[NSNotificationCenter defaultCenter] postNotification:notification];
+        }
+        sDelayedNotifications = nil;
+    } else {
+        block();
+    }
+}
+
+// identifier is optional. Old shortcuts only have a title.
++ (BOOL)shortcutIdentifier:(NSString *)identifier title:(NSString *)title matchesItem:(NSMenuItem *)item {
+    if (item.identifier && [identifier isEqualToString:item.identifier]) {
+        return YES;
+    }
+    if (!identifier && [title isEqualToString:[item title]]) {
+        return YES;
+    }
+
+    return NO;
 }
 
 @end
