@@ -22,12 +22,12 @@ NS_CLASS_AVAILABLE(10_11, NA)
 @property (nonatomic, strong) id<MTLTexture> backgroundTexture;
 @property (nonatomic) iTermMetalUnderlineDescriptor asciiUnderlineDescriptor;
 @property (nonatomic) iTermMetalUnderlineDescriptor nonAsciiUnderlineDescriptor;
+@property (nonatomic) iTermMetalUnderlineDescriptor strikethroughUnderlineDescriptor;
 @property (nonatomic) vector_float4 defaultBackgroundColor;
 
-
-- (void)setGlyphKeysData:(iTermData *)glyphKeysData
+- (void)setGlyphKeysData:(iTermGlyphKeyData*)glyphKeysData
                    count:(int)count
-          attributesData:(iTermData *)attributesData
+          attributesData:(iTermAttributesData *)attributesData
                      row:(int)row
   backgroundColorRLEData:(iTermData *)backgroundColorData  // array of iTermMetalBackgroundColorRLE background colors.
        markedRangeOnLine:(NSRange)markedRangeOnLine
