@@ -7,6 +7,8 @@
 //
 
 #import "NSDictionary+iTerm.h"
+
+#import "iTermTuple.h"
 #import "NSColor+iTerm.h"
 #import "NSWorkspace+iTerm.h"
 
@@ -16,6 +18,7 @@ static NSString *const kGridCoordAbsYKey = @"absY";
 static NSString *const kGridCoordStartKey = @"start";
 static NSString *const kGridCoordEndKey = @"end";
 static NSString *const kGridCoordRange = @"Coord Range";
+static NSString *const kGridCoordAbsRange = @"Coord Abs Range";
 static NSString *const kGridRange = @"Range";
 static NSString *const kGridRangeLocation = @"Location";
 static NSString *const kGridRangeLength = @"Length";
@@ -121,6 +124,22 @@ static const NSEventModifierFlags iTermHotkeyModifierMask = (NSEventModifierFlag
     return range;
 }
 
++ (NSDictionary *)dictionaryWithGridAbsWindowedRange:(VT100GridAbsWindowedRange)absRange {
+    return @{ kGridCoordAbsRange: [NSDictionary dictionaryWithGridAbsCoordRange:absRange.coordRange],
+              kGridRange: [NSDictionary dictionaryWithGridRange:absRange.columnWindow] };
+}
+
+- (VT100GridAbsWindowedRange)gridAbsWindowedRange {
+    VT100GridAbsWindowedRange absRange;
+    absRange.coordRange = [self[kGridCoordAbsRange] gridAbsCoordRange];
+    absRange.columnWindow = [self[kGridRange] gridRange];
+    return absRange;
+}
+
+- (BOOL)hasGridAbsWindowedRange {
+    return self[kGridCoordAbsRange] != nil && self[kGridRange] != nil;
+}
+
 + (NSDictionary *)dictionaryWithGridRange:(VT100GridRange)range {
     return @{ kGridRangeLocation: @(range.location),
               kGridRangeLength: @(range.length) };
@@ -157,10 +176,19 @@ static const NSEventModifierFlags iTermHotkeyModifierMask = (NSEventModifierFlag
 }
 
 - (NSColor *)colorValue {
-    return [self colorValueWithDefaultAlpha:1.0];
+    return [self colorValueForKey:nil];
 }
 
-- (NSColor *)colorValueWithDefaultAlpha:(CGFloat)alpha {
++ (CGFloat)defaultAlphaForColorPresetKey:(NSString *)key {
+    if ([key isEqualToString:KEY_CURSOR_GUIDE_COLOR]) {
+        return 0.25;
+    } else {
+        return 1.0;
+    }
+}
+
+- (NSColor *)colorValueForKey:(NSString *)key {
+    CGFloat alpha = [NSDictionary defaultAlphaForColorPresetKey:key];
     if ([self count] < 3) {
         return [NSColor colorWithCalibratedRed:0.0 green:0.0 blue:0.0 alpha:1.0];
     }
@@ -196,17 +224,28 @@ static const NSEventModifierFlags iTermHotkeyModifierMask = (NSEventModifierFlag
 }
 
 - (NSDictionary *)dictionaryBySettingObject:(id)object forKey:(id)key {
-    NSMutableDictionary *temp = [[self mutableCopy] autorelease];
+    NSMutableDictionary *temp = [self mutableCopy];
     temp[key] = object;
     return temp;
 }
 
 - (NSDictionary *)dictionaryByRemovingObjectForKey:(id)key {
-    NSMutableDictionary *temp = [[self mutableCopy] autorelease];
+    NSMutableDictionary *temp = [self mutableCopy];
     [temp removeObjectForKey:key];
     return temp;
 }
 
+- (NSDictionary *)dictionaryKeepingOnlyKeys:(NSArray *)keys {
+    NSMutableDictionary *dictionary = [NSMutableDictionary dictionary];
+    for (id key in keys) {
+        id object = self[key];
+        if (object) {
+            dictionary[key] = object;
+        }
+    }
+    return dictionary;
+}
+
 - (NSData *)propertyListData {
     NSString *filename = [[NSWorkspace sharedWorkspace] temporaryFileNameWithPrefix:@"DictionaryPropertyList" suffix:@"iTerm2"];
     [self writeToFile:filename atomically:NO];
@@ -217,7 +256,7 @@ static const NSEventModifierFlags iTermHotkeyModifierMask = (NSEventModifierFlag
 
 - (NSString *)sizeInfo {
     NSMutableDictionary<NSString *, NSNumber *> *sizes = [NSMutableDictionary dictionary];
-    NSCountedSet<NSString *> *counts = [[[NSCountedSet alloc] init] autorelease];
+    NSCountedSet<NSString *> *counts = [[NSCountedSet alloc] init];
     sizes[@""] = @([self addSizeInfoToSizes:sizes counts:counts keypath:@""]);
     [counts addObject:@""];
 
@@ -228,6 +267,65 @@ static const NSEventModifierFlags iTermHotkeyModifierMask = (NSEventModifierFlag
     return result;
 }
 
+- (NSDictionary *)filteredWithBlock:(BOOL (^NS_NOESCAPE)(id key, id value))block {
+    NSMutableDictionary *result = [NSMutableDictionary dictionary];
+    [self enumerateKeysAndObjectsUsingBlock:^(id  _Nonnull key, id  _Nonnull obj, BOOL * _Nonnull stop) {
+        if (block(key, obj)) {
+            result[key] = obj;
+        }
+    }];
+    return result;
+}
+
+- (NSDictionary *)mapValuesWithBlock:(id (^)(id, id))block {
+    NSMutableDictionary *result = [NSMutableDictionary dictionary];
+    [self enumerateKeysAndObjectsUsingBlock:^(id  _Nonnull key, id  _Nonnull obj, BOOL * _Nonnull stop) {
+        id mapped = block(key, obj);
+        if (mapped) {
+            result[key] = mapped;
+        }
+    }];
+    return result;
+}
+
+- (NSDictionary *)mapKeysWithBlock:(id (^)(id, id))block {
+    NSMutableDictionary *result = [NSMutableDictionary dictionary];
+    [self enumerateKeysAndObjectsUsingBlock:^(id  _Nonnull key, id  _Nonnull obj, BOOL * _Nonnull stop) {
+        id mappedKey = block(key, obj);
+        if (mappedKey) {
+            result[mappedKey] = obj;
+        }
+    }];
+    return result;
+}
+
+- (NSDictionary *)mapWithBlock:(iTermTuple *(^)(id, id))block {
+    NSMutableDictionary *result = [NSMutableDictionary dictionary];
+    [self enumerateKeysAndObjectsUsingBlock:^(id  _Nonnull key, id  _Nonnull obj, BOOL * _Nonnull stop) {
+        iTermTuple *tuple = block(key, obj);
+        if (tuple) {
+            result[tuple.firstObject] = tuple.secondObject;
+        }
+    }];
+    return result;
+}
+
+- (NSDictionary<id, NSDictionary *> *)classifyWithBlock:(id (^NS_NOESCAPE)(id key, id object))block {
+    NSMutableDictionary<id, NSMutableDictionary *> *result = [NSMutableDictionary dictionary];
+    [self enumerateKeysAndObjectsUsingBlock:^(id  _Nonnull key, id  _Nonnull obj, BOOL * _Nonnull stop) {
+        id class = block(key, obj);
+        if (class) {
+            NSMutableDictionary *subdict = result[class];
+            if (!subdict) {
+                subdict = [NSMutableDictionary dictionary];
+                result[class] = subdict;
+            }
+            subdict[key] = obj;
+        }
+    }];
+    return result;
+}
+
 - (NSInteger)addSizeInfoToSizes:(NSMutableDictionary<NSString *, NSNumber *> *)sizes
                     counts:(NSCountedSet<NSString *> *)counts
                    keypath:(NSString *)keypath {
@@ -254,6 +352,47 @@ static const NSEventModifierFlags iTermHotkeyModifierMask = (NSEventModifierFlag
     return total;
 }
 
+- (BOOL)it_writeToXMLPropertyListAt:(NSString *)filename {
+    NSOutputStream *outputStream = [NSOutputStream outputStreamToFileAtPath:filename append:NO];
+    [outputStream scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
+    [outputStream open];
+    NSError *error = nil;
+    [NSPropertyListSerialization writePropertyList:self
+                                          toStream:outputStream
+                                            format:NSPropertyListXMLFormat_v1_0
+                                           options:0
+                                             error:&error];
+    [outputStream close];
+    return error == nil;
+}
+
+- (NSDictionary *)it_attributesDictionaryWithAppearance:(NSAppearance *)appearance {
+    NSMutableDictionary *result = [NSMutableDictionary dictionary];
+    [self enumerateKeysAndObjectsUsingBlock:^(id  _Nonnull key, id  _Nonnull obj, BOOL * _Nonnull stop) {
+        if ([obj isKindOfClass:[NSColor class]]) {
+            result[key] = [obj it_colorWithAppearance:appearance];
+        } else {
+            result[key] = obj;
+        }
+    }];
+    return result;
+}
+
+- (id)it_jsonSafeValue {
+    NSMutableDictionary *safe = [NSMutableDictionary dictionary];
+    for (id key in self) {
+        id value = self[key];
+        id safeKey = nil;
+        if ([key respondsToSelector:_cmd]) {
+            safeKey = [key it_jsonSafeValue];
+        }
+        if (safeKey && [value respondsToSelector:_cmd]) {
+            safe[safeKey] = [value it_jsonSafeValue];
+        }
+    }
+    return safe;
+}
+
 @end
 
 @implementation NSDictionary(HotKey)
@@ -298,7 +437,7 @@ static const NSEventModifierFlags iTermHotkeyModifierMask = (NSEventModifierFlag
 }
 
 - (NSDictionary *)dictionaryByMergingDictionary:(NSDictionary *)other {
-    NSMutableDictionary *temp = [[self mutableCopy] autorelease];
+    NSMutableDictionary *temp = [self mutableCopy];
     [other enumerateKeysAndObjectsUsingBlock:^(id  _Nonnull key, id  _Nonnull obj, BOOL * _Nonnull stop) {
         temp[key] = obj;
     }];
@@ -320,3 +459,35 @@ static const NSEventModifierFlags iTermHotkeyModifierMask = (NSEventModifierFlag
 }
 
 @end
+
+@implementation NSMutableDictionary (iTerm)
+
+- (NSInteger)removeObjectsPassingTest:(BOOL (^)(id, id))block {
+    NSMutableSet *keys = [NSMutableSet set];
+    [self enumerateKeysAndObjectsUsingBlock:^(id  _Nonnull key, id  _Nonnull obj, BOOL * _Nonnull stop) {
+        if (block(key, obj)) {
+            [keys addObject:key];
+        }
+    }];
+    [self removeObjectsForKeys:keys.allObjects];
+    return keys.count;
+}
+
+- (void)it_mergeFrom:(NSDictionary *)other {
+    assert(self != other);
+    
+    [other enumerateKeysAndObjectsUsingBlock:^(id  _Nonnull key, id  _Nonnull obj, BOOL * _Nonnull stop) {
+        self[key] = obj;
+    }];
+}
+
+- (void)it_addObject:(id)object toMutableArrayForKey:(id)key {
+    NSMutableArray *array = self[key];
+    if (!array) {
+        array = [NSMutableArray array];
+        self[key] = array;
+    }
+    [array addObject:object];
+}
+
+@end
