@@ -34,6 +34,11 @@
 
 #import <Cocoa/Cocoa.h>
 
+#import "iTermOrderedDictionary.h"
+#import "iTermTuple.h"
+
+@class iTermVariableScope;
+
 // This is the standard unicode replacement character for when input couldn't
 // be parsed properly but we need to render something there.
 #define UNICODE_REPLACEMENT_CHAR 0xfffd
@@ -47,12 +52,16 @@
 // negative: abs(this many) bytes are illegal, should be replaced by one
 //   single replacement symbol.
 // zero: Unfinished sequence, input needs to grow.
+#ifndef __cplusplus
 int decode_utf8_char(const unsigned char * restrict datap,
                      int datalen,
                      int * restrict result);
+#endif
 
 @interface NSString (iTerm)
 
+@property (nonatomic, readonly) NSString *jsonEncodedString;
+
 + (NSString *)stringWithInt:(int)num;
 + (BOOL)isDoubleWidthCharacter:(int)unicode
         ambiguousIsDoubleWidth:(BOOL)ambiguousIsDoubleWidth
@@ -67,15 +76,20 @@ int decode_utf8_char(const unsigned char * restrict datap,
 
 // Returns the number of lines in a string.
 - (NSUInteger)numberOfLines;
+// May use single quotes by user preference. Only safe to use with user's default shell.
 - (NSString *)stringWithEscapedShellCharactersIncludingNewlines:(BOOL)includingNewlines;
+
+// foo' -> $'foo\\x27'
+// Suitable for use as bash -c 'escaped string'
+- (NSString *)stringEscapedForBash;
+
+// Always uses backslash.
+- (NSString *)stringWithBackslashEscapedShellCharactersIncludingNewlines:(BOOL)includingNewlines;
 - (NSString *)stringWithEscapedShellCharactersExceptTabAndNewline;
 
 // Replaces tab with ^V + tab.
 - (NSString *)stringWithShellEscapedTabs;
 
-// Properly escapes chars for a string to stick in a URL query param.
-- (NSString*)stringWithPercentEscape;
-
 // Convert DOS-style and \n newlines to \r newlines.
 - (NSString*)stringWithLinefeedNewlines;
 
@@ -97,7 +111,7 @@ int decode_utf8_char(const unsigned char * restrict datap,
 // Convert a string of hex values (an even number of [0-9A-Fa-f]) into data.
 - (NSData *)dataFromHexValues;
 
-// Always returns a non-null vaule, but it may contain replacement chars for
+// Always returns a non-null value, but it may contain replacement chars for
 // malformed utf-8 sequences.
 - (NSString *)initWithUTF8DataIgnoringErrors:(NSData *)data;
 
@@ -141,11 +155,8 @@ int decode_utf8_char(const unsigned char * restrict datap,
 
 - (NSString *)stringByRemovingEnclosingBrackets;
 
-- (NSString *)stringByEscapingForURL;
 - (NSString *)stringByCapitalizingFirstLetter;
 
-- (NSArray<NSString *> *)helpfulSynonyms;
-
 // String starts with http:// or https://. Used to tell if a custom prefs
 // location is a path or URL.
 - (BOOL)stringIsUrlLike;
@@ -159,7 +170,7 @@ int decode_utf8_char(const unsigned char * restrict datap,
 
 // Compose/Decompose UTF8 string without normalization
 
-// This is better than -precomposedStringWithCanonicalMapping because it preserves compatability
+// This is better than -precomposedStringWithCanonicalMapping because it preserves compatibility
 // equivalence. It's most relevant when two canonically equivalent characters have different widths
 // (one is half-width while the other is ambiguous width). The difference is in the following
 // ranges: 2000-2FFF, F900-FAFF, 2F800-2FAFF. See issue 2872.
@@ -172,13 +183,12 @@ int decode_utf8_char(const unsigned char * restrict datap,
 // How tall is this string when rendered within a fixed width?
 - (CGFloat)heightWithAttributes:(NSDictionary *)attributes constrainedToWidth:(CGFloat)maxWidth;
 
-- (NSArray *)keyValuePair;
+- (iTermTuple *)keyValuePair;
+- (iTermTuple<NSString *, NSString *> *)it_stringBySplittingOnFirstSubstring:(NSString *)substring;
 
-- (NSString *)stringByReplacingVariableReferencesWithVariables:(NSDictionary *)vars;
-- (NSString *)stringByPerformingSubstitutions:(NSDictionary *)substituions;
+- (NSIndexSet *)indicesOfCharactersInSet:(NSCharacterSet *)characterSet;
 
-// Does self contain |substring|?
-- (BOOL)containsString:(NSString *)substring;
+- (NSString *)stringByPerformingSubstitutions:(NSDictionary *)substitutions;
 
 // Returns self repeated |n| times.
 - (NSString *)stringRepeatedTimes:(int)n;
@@ -189,14 +199,14 @@ int decode_utf8_char(const unsigned char * restrict datap,
 // Turns a string like fooBar into FooBar.
 - (NSString *)stringWithFirstLetterCapitalized;
 
-// Given a bitmask of modifiers like NSAlternateKeyMask, return a string indicating those modifiers.
+// Given a bitmask of modifiers like NSEventModifierFlagOption, return a string indicating those modifiers.
 + (NSString *)stringForModifiersWithMask:(NSUInteger)mask;
 
 // Returns a fresh UUID
 + (NSString *)uuid;
 
 // Characters in [0, 31] and 127 get replaced with ?
-- (NSString *)stringByReplacingControlCharsWithQuestionMark;
+- (NSString *)stringByReplacingControlCharactersWithCaretLetter;
 
 // Returns the set of $$VARIABLES$$ in the string.
 - (NSSet *)doubleDollarVariables;
@@ -214,9 +224,14 @@ int decode_utf8_char(const unsigned char * restrict datap,
                                               NSString *complexString,
                                               BOOL *stop))block;
 
+- (NSString *)firstComposedCharacter:(NSString **)rest;
+- (NSString *)lastComposedCharacter;
+- (NSInteger)numberOfComposedCharacters;
+- (NSString *)byTruncatingComposedCharactersInCenter:(NSInteger)count;
+
 // It is safe to modify, delete, or insert characters in `range` within `block`.
 - (void)reverseEnumerateSubstringsEqualTo:(NSString *)query
-                                    block:(void (^)(NSRange range))block;
+                                    block:(void (^ NS_NOESCAPE)(NSRange range))block;
 
 - (NSUInteger)iterm_unsignedIntegerValue;
 
@@ -228,11 +243,11 @@ int decode_utf8_char(const unsigned char * restrict datap,
 // 1.0010 -> 1.001
 - (NSString *)stringByCompactingFloatingPointString;
 
-// A fast, non-cryto-quality hash.
+// A fast, non-crypto-quality hash.
 - (NSUInteger)hashWithDJB2;
 
-- (NSUInteger)firstCharacter;
-// Is this a phrase enclosed in qutoation marks?
+- (UTF32Char)firstCharacter;
+// Is this a phrase enclosed in quotation marks?
 - (BOOL)isInQuotationMarks;
 
 // Stick punctuation (should be a comma or a period) at the end, placing it
@@ -255,6 +270,9 @@ int decode_utf8_char(const unsigned char * restrict datap,
 // Contains only digits?
 - (BOOL)isNumeric;
 
+// First character is a digit?
+- (BOOL)startsWithDigit;
+
 // Modify the range's endpoint to not sever a surrogate pair.
 - (NSRange)makeRangeSafe:(NSRange)range;
 
@@ -266,7 +284,52 @@ int decode_utf8_char(const unsigned char * restrict datap,
 - (void)it_drawInRect:(CGRect)rect attributes:(NSDictionary *)attributes;
 
 - (BOOL)startsWithEmoji;
++ (NSString *)it_formatBytes:(double)bytes;
++ (NSString *)it_formatBytesCompact:(double)bytes;
 
+// For a string like
+// lll\(eee(eee,eee,"eee","\\"","ee\(EE())"))ll
+// Invoke block for each literal and expression. In the above example there would be three calls:
+// lll                                      YES
+// eee(eee,eee,"eee","\\"","ee\(EE())")     NO
+// ll                                       YES
+- (void)enumerateSwiftySubstrings:(void (^)(NSUInteger index, NSString *substring, BOOL isLiteral, BOOL *stop))block;
+- (NSString *)it_stringByExpandingBackslashEscapedCharacters;
++ (NSString *)sparkWithHeight:(double)fraction;
+- (id)it_jsonSafeValue;
+- (NSInteger)it_numberOfLines;
+
+// Empty strings are prefixes of all strings.
+- (BOOL)it_hasPrefix:(NSString *)prefix;
+
+// If this is a 2+ part version number, return a 2 part version number. Otherwise, nil.
+- (NSString *)it_twoPartVersionNumber;
+- (NSString *)stringByEscapingForSandboxLiteral;
+- (NSString *)stringByDroppingLastCharacters:(NSInteger)count;
+- (NSString *)stringByKeepingLastCharacters:(NSInteger)count;
+- (NSString *)stringByTrimmingOrphanedSurrogates;
+
+- (NSString *)stringByAppendingVariablePathComponent:(NSString *)component;
+- (NSString *)stringByAppendingPathComponents:(NSArray<NSString *> *)pathComponents;
+- (NSArray<NSString *> *)it_normalizedTokens;
+- (double)it_localizedDoubleValue;
+- (NSString *)it_contentHash;
+- (NSString *)it_unescapedTmuxWindowName;
+- (NSString *)it_substringToIndex:(NSInteger)index;
+- (NSString *)it_escapedForRegex;
+- (NSString *)it_compressedString;
+
+// Use this in #!/usr/bin/env -S "%@"
+// Important! It assumes you put the value in double quotes. Amusingly, the man page for env
+// trolls you by explaining that single-quoted values only need to escape ' and \ but neglects to
+// mention that other characters simple won't work at all, escaped or otherwise.
+- (NSString *)it_escapedForEnv;
+
+// Perform substitutions in order.
+- (NSString *)stringByPerformingOrderedSubstitutions:(iTermOrderedDictionary<NSString *, NSString *> *)substitutions;
+- (NSString *)stringByReplacingCharactersAtIndices:(NSIndexSet *)indexSet
+                               withStringFromBlock:(NSString *(^ NS_NOESCAPE)(void))replacement;
+- (BOOL)caseInsensitiveHasPrefix:(NSString *)prefix;
 @end
 
 @interface NSMutableString (iTerm)
@@ -275,9 +338,14 @@ int decode_utf8_char(const unsigned char * restrict datap,
 
 // Puts backslashes before characters in shellEscapableCharacters.
 - (void)escapeShellCharactersIncludingNewlines:(BOOL)includingNewlines;
+- (void)escapeShellCharactersWithBackslashIncludingNewlines:(BOOL)includingNewlines;
 - (void)escapeShellCharactersExceptTabAndNewline;
 
+// foo' -> $'foo\\x27'
+- (void)escapeShellCharactersForBash;
+
 // Convenience method to append a single character.
 - (void)appendCharacter:(unichar)c;
+- (void)escapeCharacters:(NSString *)charsToEscape;
 
 @end
