@@ -3,26 +3,34 @@
 
 #import <Cocoa/Cocoa.h>
 #import "FutureMethods.h"
+#import "iTermEncoderAdapter.h"
 #import "PSMTabBarControl.h"
 #import "PTYSession.h"
 #import "PTYSplitView.h"
 #import "PTYTabDelegate.h"
 #import "WindowControllerInterface.h"
+#import "Api.pbobjc.h"
 
+@class FakeWindow;
+@protocol iTermTabScope;
+@class iTermVariables;
+@class iTermVariableScope;
 @class PTYSession;
 @class PTYTab;
-@class FakeWindow;
 @class SessionView;
-@class TmuxController;
 @class SolidColorView;
+@class TmuxController;
 
 extern NSString *const iTermTabDidChangeWindowNotification;
+extern NSString *const iTermSessionBecameKey;
+extern NSString *const PTYTabVariableTitleOverride;
 
 // This implements NSSplitViewDelegate but it was an informal protocol in 10.5. If 10.5 support
 // is eventually dropped, change this to make it official.
 @interface PTYTab : NSObject <
   NSCopying,
   NSSplitViewDelegate,
+  iTermUniquelyIdentifiable,
   PTYSessionDelegate,
   PTYSplitViewDelegate,
   PSMTabBarControlRepresentedObjectIdentifierProtocol>
@@ -30,19 +38,19 @@ extern NSString *const iTermTabDidChangeWindowNotification;
 @property(nonatomic, assign, getter=isBroadcasting) BOOL broadcasting;
 
 // Parent controller. Always set. Equals one of realParent or fakeParent.
-@property(nonatomic, assign) __unsafe_unretained id<WindowControllerInterface> parentWindow;
+@property(nonatomic, weak) id<WindowControllerInterface> parentWindow;
 
 // uniqueId lazily auto-assigns a unique id unless you assign it a value first. It is never 0.
 @property(nonatomic, assign) int uniqueId;
 @property(nonatomic, readonly) BOOL isMaximized;
 // Sessions ordered in a similar-to-reading-order fashion.
 @property(nonatomic, readonly) NSArray *orderedSessions;
-@property(nonatomic, assign) id<PTYTabDelegate> delegate;
+@property(nonatomic, weak) id<PTYTabDelegate> delegate;
 
 // While activeSession is not retained, it should only ever refer to a session that belongs to
 // this tab, and is thus retained through the view-to-session map.
-@property(nonatomic, assign) __unsafe_unretained PTYSession *activeSession;
-@property(nonatomic, retain) NSTabViewItem *tabViewItem;
+@property(nonatomic, weak) PTYSession *activeSession;
+@property(nonatomic, weak) NSTabViewItem *tabViewItem;
 
 // These values are observed by PSMTTabBarControl:
 // Tab number for display
@@ -55,32 +63,53 @@ extern NSString *const iTermTabDidChangeWindowNotification;
 @property(nonatomic, copy) NSString *tmuxWindowName;
 @property (readonly, getter=isTmuxTab) BOOL tmuxTab;
 @property (nonatomic, readonly) PTYTabState state;
+@property (nonatomic, readonly) iTermVariables *variables;
+@property (nonatomic, readonly) iTermVariables *userVariables;
+
+// NOTE: This isn't actually the root view for tmux integration settings.
+@property (nonatomic, readonly) NSView *rootView;
+@property (nonatomic, readonly) NSView *realRootView;
 
 // If non-nil, this session may not change size. This is useful when you want
 // to change a session's size. You can resize it, lock it, and then
 // adjustSubviews of the splitview (ordinarily done by a call to -[PTYTab
 // setSize:]).
-@property(nonatomic, assign) __unsafe_unretained PTYSession *lockedSession;
+@property(nonatomic, weak) PTYSession *lockedSession;
+
+// A string that overrides the default behavior of using the active session's title.
+// Set to nil to use the default behavior. This is a swifty string.
+@property (nonatomic, copy) NSString *titleOverride;
+
+@property (nonatomic, readonly) iTermVariableScope<iTermTabScope> *variablesScope;
+@property(nonatomic, readonly) iTermMetalUnavailableReason metalUnavailableReason;
+@property(nonatomic) BOOL deferFontChanges;
+
+// If true, report that the tab's ideal size is its currentSize.
+@property(nonatomic) BOOL reportIdeal;
+@property(nonatomic, readonly) NSArray<PTYSession *> *sessionsAtTop;
+@property(nonatomic, readonly) NSArray<PTYSession *> *sessionsAtLeft;
+@property(nonatomic, readonly) NSArray<PTYSession *> *sessionsAtBottom;
+
++ (NSSize)sizeForTmuxWindowWithAffinity:(NSString *)affinity
+                             controller:(TmuxController *)controller;
 
 // Save the contents of all sessions. Used during window restoration so that if
 // the sessions are later restored from a saved arrangement during startup
 // activities, their contents can be rescued.
 + (void)registerSessionsInArrangement:(NSDictionary *)arrangement;
++ (void)registerBuiltInFunctions;
 
 + (void)drawArrangementPreview:(NSDictionary*)arrangement frame:(NSRect)frame;
 
-+ (PTYTab *)openTabWithArrangement:(NSDictionary*)arrangement
-                        inTerminal:(NSWindowController<iTermWindowController> *)term
-                   hasFlexibleView:(BOOL)hasFlexible
-                           viewMap:(NSDictionary<NSNumber *, SessionView *> *)viewMap
-                        sessionMap:(NSDictionary<NSString *, PTYSession *> *)sessionMap;
-
 + (PTYTab *)tabWithArrangement:(NSDictionary*)arrangement
+                         named:(NSString *)arrangementName
                     inTerminal:(NSWindowController<iTermWindowController> *)term
                hasFlexibleView:(BOOL)hasFlexible
                        viewMap:(NSDictionary<NSNumber *, SessionView *> *)viewMap
                     sessionMap:(NSDictionary<NSString *, PTYSession *> *)sessionMap
-                tmuxController:(TmuxController *)tmuxController;
+                tmuxController:(TmuxController *)tmuxController
+            partialAttachments:(NSDictionary *)partialAttachments
+              reservedTabGUIDs:(NSSet<NSString *> *)reservedTabGUIDs;
 
 + (NSDictionary<NSString *, PTYSession *> *)sessionMapWithArrangement:(NSDictionary *)arrangement
                                                              sessions:(NSArray *)sessions;
@@ -94,13 +123,27 @@ extern NSString *const iTermTabDidChangeWindowNotification;
              replacingProfileWithGUID:(NSString *)badGuid
                           withProfile:(Profile *)goodProfile;
 
++ (NSDictionary *)repairedArrangement:(NSDictionary *)arrangement
+     replacingOldCWDOfSessionWithGUID:(NSString *)guid
+                           withOldCWD:(NSString *)replacementOldCWD;
+
++ (NSDictionary *)arrangementForSessionWithGUID:(NSString *)sessionGUID
+                                  inArrangement:(NSDictionary *)arrangement;
+
++ (void)openPartialAttachmentsForArrangement:(NSDictionary *)arrangement
+                                  completion:(void (^)(NSDictionary *))completion;
+
 // init/dealloc
-- (instancetype)initWithSession:(PTYSession*)session;
+- (instancetype)initWithSession:(PTYSession *)session
+                   parentWindow:(NSWindowController<iTermWindowController> *)parentWindow;
 - (instancetype)initWithRoot:(NSSplitView *)root
                     sessions:(NSMapTable<SessionView *, PTYSession *> *)sessions;
 
 - (void)setRoot:(NSSplitView *)newRoot;
 
+// This is safe to call when there may be a maximized pane.
+- (void)setActiveSessionPreservingMaximization:(PTYSession *)session;
+
 - (NSRect)absoluteFrame;
 - (int)indexOfSessionView:(SessionView*)sessionView;
 
@@ -159,7 +202,8 @@ extern NSString *const iTermTabDidChangeWindowNotification;
 // arrangement with matching session unique IDs will be assigned those
 // SessionView's.
 - (void)updateFlexibleViewColors;
-- (NSDictionary*)arrangement;
+- (NSDictionary *)arrangement;
+- (NSDictionary *)arrangementWithNewGUID;
 
 - (void)notifyWindowChanged;
 - (void)maximize;
@@ -180,7 +224,7 @@ extern NSString *const iTermTabDidChangeWindowNotification;
        tmuxController:(TmuxController *)tmuxController
                zoomed:(NSNumber *)zoomed;
 // Returns true if the tmux layout is too large for the window to accommodate.
-- (BOOL)layoutIsTooLarge;
+- (BOOL)updatedTmuxLayoutRequiresAdjustment;
 - (TmuxController *)tmuxController;
 
 - (void)setTmuxFont:(NSFont *)font
@@ -199,8 +243,10 @@ extern NSString *const iTermTabDidChangeWindowNotification;
 - (void)replaceWithContentsOfTab:(PTYTab *)tabToGut;
 
 - (NSDictionary*)arrangementWithContents:(BOOL)contents;
+- (BOOL)encodeWithContents:(BOOL)contents
+                   encoder:(id<iTermEncoderAdapter>)encoder;
 
-// Update the tab's title from the active session's name. Needed for initialzing the tab's title
+// Update the tab's title from the active session's name. Needed for initializing the tab's title
 // after setting up tmux tabs.
 - (void)loadTitleFromSession;
 
@@ -211,5 +257,10 @@ extern NSString *const iTermTabDidChangeWindowNotification;
 - (void)setTmuxWindowName:(NSString *)tmuxWindowName;
 
 - (void)updateUseMetal NS_AVAILABLE_MAC(10_11);
+- (ITMSplitTreeNode *)rootSplitTreeNode;
+
+- (void)setSizesFromSplitTreeNode:(ITMSplitTreeNode *)node;
+- (void)arrangeSplitPanesEvenly;
+- (void)bounceMetal;
 
 @end
