@@ -20,12 +20,14 @@
 #include <sys/time.h>
 
 @class CRunStorage;
+@class iTermAction;
 @class iTermExpect;
 @class iTermFindCursorView;
 @class iTermFindOnPageHelper;
 @class iTermQuickLookController;
 @class iTermSelection;
 @protocol iTermSemanticHistoryControllerDelegate;
+@protocol iTermSwipeHandler;
 @class iTermURLActionHelper;
 @class iTermVariableScope;
 @class MovingAverage;
@@ -55,6 +57,7 @@ typedef NS_ENUM(NSInteger, PTYCharType) {
 
 - (BOOL)xtermMouseReporting;
 - (BOOL)xtermMouseReportingAllowMouseWheel;
+- (BOOL)xtermMouseReportingAllowClicksAndDrags;
 - (BOOL)isPasting;
 - (void)queueKeyDown:(NSEvent *)event;
 - (void)keyDown:(NSEvent *)event;
@@ -76,6 +79,7 @@ typedef NS_ENUM(NSInteger, PTYCharType) {
 - (void)sendHexCode:(NSString *)codes;
 - (void)sendText:(NSString *)text;
 - (void)sendTextSlowly:(NSString *)text;
+- (void)textViewSelectionDidChangeToTruncatedString:(NSString *)maybeSelection;
 - (void)launchCoprocessWithCommand:(NSString *)command;
 - (void)insertText:(NSString *)string;
 - (PTYTask *)shell;
@@ -165,13 +169,14 @@ typedef NS_ENUM(NSInteger, PTYCharType) {
 - (void)textViewThinksUserIsTryingToSendArrowKeysWithScrollWheel:(BOOL)trying;
 
 // Update the text view's frame needed.
-- (void)textViewResizeFrameIfNeeded;
+- (BOOL)textViewResizeFrameIfNeeded;
 
 - (NSInteger)textViewUnicodeVersion;
 - (void)textViewDidRefresh;
 
 // The background color in the color map changed.
 - (void)textViewBackgroundColorDidChange;
+- (void)textViewProcessedBackgroundColorDidChange;
 
 // Describes the current user, host, and path.
 - (NSURL *)textViewCurrentLocation;
@@ -202,7 +207,18 @@ typedef NS_ENUM(NSInteger, PTYCharType) {
 - (void)textViewDidUpdateDropTargetVisibility;
 - (void)textViewDidDetectMouseReportingFrustration;
 - (BOOL)textViewCanBury;
-
+- (void)textViewFindOnPageLocationsDidChange;
+- (void)textViewFindOnPageSelectedResultDidChange;
+- (CGFloat)textViewBlend;
+- (NSEdgeInsets)textViewExtraMargins;
+- (id<iTermSwipeHandler>)textViewSwipeHandler;
+- (void)textViewAddContextMenuItems:(NSMenu *)menu;
+- (NSString *)textViewShell;
+- (void)textViewContextMenuInvocation:(NSString *)invocation
+                      failedWithError:(NSError *)error
+                          forMenuItem:(NSString *)title;
+- (void)textViewApplyAction:(iTermAction *)action;
+- (void)textViewAddTrigger:(NSString *)text;
 @end
 
 @interface iTermHighlightedRow : NSObject
@@ -240,8 +256,8 @@ typedef NS_ENUM(NSInteger, PTYCharType) {
 @property(nonatomic, copy) NSArray *smartSelectionRules;
 
 // Intercell spacing as a proportion of cell size.
-@property(nonatomic, assign) double horizontalSpacing;
-@property(nonatomic, assign) double verticalSpacing;
+@property(nonatomic, assign) CGFloat horizontalSpacing;
+@property(nonatomic, assign) CGFloat verticalSpacing;
 
 // Use a different font for bold, if available?
 @property(nonatomic, assign) BOOL useBoldFont;
@@ -330,9 +346,6 @@ typedef NS_ENUM(NSInteger, PTYCharType) {
 // Is this view in the key window?
 @property(nonatomic, readonly) BOOL isInKeyWindow;
 
-// Blending level for background color over background image
-@property(nonatomic, assign) float blend;
-
 // Used by tests to modify drawing helper. Called within -drawRect:.
 typedef void (^PTYTextViewDrawingHookBlock)(iTermTextDrawingHelper *);
 @property(nonatomic, copy) PTYTextViewDrawingHookBlock drawingHook;
@@ -380,21 +393,19 @@ typedef void (^PTYTextViewDrawingHookBlock)(iTermTextDrawingHelper *);
 @property (nonatomic, readonly) iTermURLActionHelper *urlActionHelper;
 
 @property (nonatomic, readonly) VT100GridCoord cursorCoord;
+@property (nonatomic, readonly) iTermFindOnPageHelper *findOnPageHelper;
 
 // Returns the size of a cell for a given font. hspace and vspace are multipliers and the width
 // and height.
 + (NSSize)charSizeForFont:(NSFont*)aFont
-        horizontalSpacing:(double)hspace
-          verticalSpacing:(double)vspace;
+        horizontalSpacing:(CGFloat)hspace
+          verticalSpacing:(CGFloat)vspace;
 
 // This is the designated initializer. The color map should have the
 // basic colors plus the 8-bit ansi colors set shortly after this is
 // called.
 - (instancetype)initWithFrame:(NSRect)frame colorMap:(iTermColorMap *)colorMap;
 
-// Sets the "changed since last Exposé" flag to NO and returns its original value.
-- (BOOL)getAndResetChangedSinceLastExpose;
-
 // Changes the document cursor, if needed. The event is used to get modifier flags.
 - (void)updateCursor:(NSEvent *)event;
 
@@ -445,8 +456,8 @@ typedef void (^PTYTextViewDrawingHookBlock)(iTermTextDrawingHelper *);
 // Various accessors (TODO: convert as many as possible into properties)
 - (void)setFont:(NSFont*)aFont
     nonAsciiFont:(NSFont *)nonAsciiFont
-    horizontalSpacing:(double)horizontalSpacing
-    verticalSpacing:(double)verticalSpacing;
+    horizontalSpacing:(CGFloat)horizontalSpacing
+    verticalSpacing:(CGFloat)verticalSpacing;
 - (NSRect)scrollViewContentSize;
 - (void)setAntiAlias:(BOOL)asciiAA nonAscii:(BOOL)nonAsciiAA;
 
@@ -541,6 +552,7 @@ scrollToFirstResult:(BOOL)scrollToFirstResult;
 // Open a semantic history path.
 - (void)openSemanticHistoryPath:(NSString *)path
                   orRawFilename:(NSString *)rawFileName
+                       fragment:(NSString *)fragment
                workingDirectory:(NSString *)workingDirectory
                      lineNumber:(NSString *)lineNumber
                    columnNumber:(NSString *)columnNumber
@@ -592,6 +604,9 @@ scrollToFirstResult:(BOOL)scrollToFirstResult;
                            by:(PTYTextViewSelectionExtensionUnit)unit
                   cursorCoord:(VT100GridCoord)cursorCoord;
 
+- (void)selectCoordRange:(VT100GridCoordRange)range;
+- (NSRect)frameForCoord:(VT100GridCoord)coord;
+
 - (iTermLogicalMovementHelper *)logicalMovementHelperForCursorCoordinate:(VT100GridCoord)cursorCoord;
 
 // Undoes -refuseFirstResponderAtCurrentMouseLocation.
@@ -604,8 +619,6 @@ scrollToFirstResult:(BOOL)scrollToFirstResult;
 
 - (void)showFireworks;
 
-- (BOOL)imageIsVisible:(iTermImageInfo *)image;
-
 // Turns on the flicker fixer (if enabled) while drawing.
 - (void)performBlockWithFlickerFixerGrid:(void (NS_NOESCAPE ^)(void))block;
 
@@ -614,9 +627,15 @@ scrollToFirstResult:(BOOL)scrollToFirstResult;
 
 #pragma mark - Testing only
 
-- (id)selectedTextAttributed:(BOOL)attributed
-                cappedAtSize:(int)maxBytes
-           minimumLineNumber:(int)minimumLineNumber;
+typedef NS_ENUM(NSUInteger, iTermCopyTextStyle) {
+    iTermCopyTextStylePlainText,
+    iTermCopyTextStyleAttributed,
+    iTermCopyTextStyleWithControlSequences
+};
+
+- (id)selectedTextWithStyle:(iTermCopyTextStyle)style
+               cappedAtSize:(int)maxBytes
+          minimumLineNumber:(int)minimumLineNumber;
 
 @end
 
