@@ -25,8 +25,12 @@
 #import "ProfileListView.h"
 
 #import "DebugLogging.h"
+#import "iTermSplitViewAnimation.h"
 #import "ITAddressBookMgr.h"
+#import "NSArray+iTerm.h"
 #import "NSMutableAttributedString+iTerm.h"
+#import "NSObject+iTerm.h"
+#import "NSTextField+iTerm.h"
 #import "PTYSession.h"
 #import "ProfileModel.h"
 #import "ProfileModelWrapper.h"
@@ -35,9 +39,74 @@
 #import "ProfileTagsView.h"
 #import "iTermSearchField.h"
 #import "NSView+RecursiveDescription.h"
+#import "NSWindow+iTerm.h"
 
 #define kProfileTableViewDataType @"iTerm2ProfileGuid"
 
+// NSAttributedString attribute keys used as source values by
+// iTermProfileListViewTextField. One of these colors will be used as the
+// NSForegroundColorAttributeName's value when the background style changes.
+static NSString *const iTermSelectedActiveForegroundColor = @"iTermSelectedActiveForegroundColor";
+static NSString *const iTermRegularForegroundColor = @"iTermRegularForegroundColor";
+static NSString *const iTermProfileListViewRestorableStateTagsVisible = @"iTermProfileListViewRestorableStateTagsVisible";
+static NSString *const iTermProfileListViewRestorableStateTagsFraction = @"iTermProfileListViewRestorableStateTagsFraction";
+
+// This is a text field that updates its text colors depending on the current background style.
+@interface iTermProfileListViewTextField : NSTextField
+@end
+
+@implementation iTermProfileListViewTextField
+
+- (void)setBackgroundStyle:(NSBackgroundStyle)backgroundStyle {
+    if (@available(macOS 10.14, *)) {
+        switch (backgroundStyle) {
+            case NSBackgroundStyleNormal:
+                self.textColor = [NSColor labelColor];
+                [self setAttributedTextColorsForKey:iTermRegularForegroundColor];
+                break;
+            case NSBackgroundStyleEmphasized:
+                [self setAttributedTextColorsForKey:iTermSelectedActiveForegroundColor];
+                self.textColor = [NSColor labelColor];
+                break;
+                
+            case NSBackgroundStyleRaised:
+            case NSBackgroundStyleLowered:
+                DLog(@"Unexpected background style %@", @(backgroundStyle));
+                break;
+        }
+        return;
+    }
+    switch (backgroundStyle) {
+        case NSBackgroundStyleLight:
+            self.textColor = [NSColor blackColor];
+            [self setAttributedTextColorsForKey:iTermRegularForegroundColor];
+            break;
+        case NSBackgroundStyleDark:
+            [self setAttributedTextColorsForKey:iTermSelectedActiveForegroundColor];
+            self.textColor = [NSColor whiteColor];
+            break;
+
+        case NSBackgroundStyleRaised:
+        case NSBackgroundStyleLowered:
+            DLog(@"Unexpected background style %@", @(backgroundStyle));
+            break;
+    }
+}
+
+- (void)setAttributedTextColorsForKey:(NSString *)key {
+    NSMutableAttributedString *m = [self.attributedStringValue.mutableCopy autorelease];
+    [self.attributedStringValue enumerateAttributesInRange:NSMakeRange(0, self.attributedStringValue.string.length)
+                                                   options:0
+                                                usingBlock:^(NSDictionary<NSAttributedStringKey,id> * _Nonnull attrs, NSRange range, BOOL * _Nonnull stop) {
+                                                    NSMutableDictionary *newAttrs = [[attrs mutableCopy] autorelease];
+                                                    newAttrs[NSForegroundColorAttributeName] = attrs[key];
+                                                    [m setAttributes:newAttrs range:range];
+                                                }];
+    self.attributedStringValue = m;
+}
+
+@end
+
 NSString *const kProfileWasDeletedNotification = @"kProfileWasDeletedNotification";
 
 const int kSearchWidgetHeight = 22;
@@ -71,6 +140,8 @@ const CGFloat kDefaultTagsWidth = 80;
     BOOL _haveHeights;
     CGFloat _heightWithTags;
     CGFloat _heightWithoutTags;
+    NSFont *_font;
+    NSInteger _restoringSelection;
 }
 
 - (instancetype)initWithFrame:(NSRect)frameRect {
@@ -134,10 +205,15 @@ const CGFloat kDefaultTagsWidth = 80;
                           horizontalScrollerClass:nil
                             verticalScrollerClass:[scrollView_.verticalScroller class]
                                        borderType:scrollView_.borderType
-                                      controlSize:NSRegularControlSize
+                                      controlSize:NSControlSizeRegular
                                     scrollerStyle:scrollView_.verticalScroller.scrollerStyle];
 
         tableView_ = [[ProfileTableView alloc] initWithFrame:tableViewFrame];
+#ifdef MAC_OS_X_VERSION_10_16
+        if (@available(macOS 10.16, *)) {
+            tableView_.style = NSTableViewStyleInset;
+        }
+#endif
         [tableView_ setMenuHandler:self];
         [tableView_ registerForDraggedTypes:[NSArray arrayWithObject:kProfileTableViewDataType]];
         [tableView_ setSelectionHighlightStyle:NSTableViewSelectionHighlightStyleRegular];
@@ -147,7 +223,6 @@ const CGFloat kDefaultTagsWidth = 80;
         [tableView_ setAllowsEmptySelection:YES];
         [tableView_ setAllowsMultipleSelection:NO];
         [tableView_ setAllowsTypeSelect:NO];
-        [tableView_ setBackgroundColor:[NSColor whiteColor]];
 
         tableColumn_ =
             [[NSTableColumn alloc] initWithIdentifier:@"name"];
@@ -155,15 +230,17 @@ const CGFloat kDefaultTagsWidth = 80;
         [tableView_ addTableColumn:tableColumn_];
 
         [scrollView_ setDocumentView:tableView_];
-        [scrollView_ setBorderType:NSBezelBorder];
+        if (@available(macOS 10.16, *)) {
+            scrollView_.borderType = NSLineBorder;
+        } else {
+            [scrollView_ setBorderType:NSBezelBorder];
+        }
 
         selectedGuids_ = [[NSMutableSet alloc] init];
 
         [tableView_ setDoubleAction:@selector(onDoubleClick:)];
 
-        NSTableHeaderView* header = [[[NSTableHeaderView alloc] init] autorelease];
-        [tableView_ setHeaderView:header];
-        [[tableColumn_ headerCell] setStringValue:@"Profile Name"];
+        tableColumn_.title = @"Profile Name";
 
         [tableView_ sizeLastColumnToFit];
 
@@ -209,9 +286,22 @@ const CGFloat kDefaultTagsWidth = 80;
     if (tableView_.dataSource == self) {
         tableView_.dataSource = nil;
     }
+    [_font release];
     [super dealloc];
 }
 
+- (BOOL)performKeyEquivalent:(NSEvent *)event {
+    DLog(@"ProfileListView: performKeyEquivalent: %@", event);
+    BOOL result = [super performKeyEquivalent:event];
+    DLog(@"ProfileListView: performKeyEquivalent: returns %@", @(result));
+    return result;
+}
+
+- (void)forceOverlayScroller {
+    scrollView_.scrollerStyle = NSScrollerStyleOverlay;
+    tagsView_.scrollView.scrollerStyle = NSScrollerStyleOverlay;
+}
+
 - (void)focusSearchField
 {
     [[self window] makeFirstResponder:searchField_];
@@ -236,7 +326,9 @@ const CGFloat kDefaultTagsWidth = 80;
         rowIndex = [rowIndexes indexGreaterThanIndex:rowIndex];
     }
 
-    NSData *data = [NSKeyedArchiver archivedDataWithRootObject:guids];
+    NSData *data = [NSKeyedArchiver archivedDataWithRootObject:guids
+                                         requiringSecureCoding:NO
+                                                         error:nil];
     [pboard declareTypes:[NSArray arrayWithObject:kProfileTableViewDataType] owner:self];
     [pboard setData:data forType:kProfileTableViewDataType];
     return YES;
@@ -272,7 +364,16 @@ const CGFloat kDefaultTagsWidth = 80;
                                         object:[self rowOrder]];
     NSPasteboard* pboard = [info draggingPasteboard];
     NSData* rowData = [pboard dataForType:kProfileTableViewDataType];
-    NSSet* guids = [NSKeyedUnarchiver unarchiveObjectWithData:rowData];
+
+    NSError *error = nil;
+    NSKeyedUnarchiver *unarchiver = [[[NSKeyedUnarchiver alloc] initForReadingFromData:rowData error:&error] autorelease];
+    if (error) {
+        return NO;
+    }
+    NSSet<NSString *> *guids = [unarchiver decodeObjectOfClass:[NSSet class] forKey:NSKeyedArchiveRootObjectKey];
+    if (!guids) {
+        return NO;
+    }
     NSMutableDictionary* map = [[[NSMutableDictionary alloc] init] autorelease];
 
     for (NSString* guid in guids) {
@@ -311,14 +412,18 @@ const CGFloat kDefaultTagsWidth = 80;
     [[dataSource_ underlyingModel] rebuildMenus];
     [[dataSource_ underlyingModel] postChangeNotification];
 
+    [self selectGuids:guids];
+
+    [self reloadData];
+}
+
+- (void)selectGuids:(NSArray *)guids {
     NSMutableIndexSet* newIndexes = [[[NSMutableIndexSet alloc] init] autorelease];
     for (NSString* guid in guids) {
         int row = [dataSource_ indexOfProfileWithGuid:guid];
         [newIndexes addIndex:row];
     }
     [tableView_ selectRowIndexes:newIndexes byExtendingSelection:NO];
-
-    [self reloadData];
 }
 
 // End Drag drop -------------------------------
@@ -489,12 +594,12 @@ const CGFloat kDefaultTagsWidth = 80;
 
 - (CGFloat)heightOfRowWithTags:(BOOL)hasTags {
     if (!_haveHeights) {
-        _heightWithTags = [[self attributedStringForName:@"M"
-                                                    tags:@[ @"M" ]
+        _heightWithTags = [[self attributedStringForName:@"Mj"
+                                                    tags:@[ @"Mj" ]
                                                 selected:NO
                                                isDefault:YES
                                                   filter:nil] heightForWidth:100] + [self extraHeightWithTags:YES];
-        _heightWithoutTags = [[self attributedStringForName:@"M"
+        _heightWithoutTags = [[self attributedStringForName:@"Mj"
                                                        tags:nil
                                                    selected:NO
                                                   isDefault:YES
@@ -513,19 +618,15 @@ const CGFloat kDefaultTagsWidth = 80;
 }
 
 - (CGFloat)extraHeightWithTags:(BOOL)hasTags {
-    if (self.mainFont.pointSize <= [NSFont smallSystemFontSize]) {
-        if (hasTags) {
-            return 2;
-        } else {
-            return 1;
-        }
+    if (hasTags) {
+        return 4;
     } else {
         return 2;
     }
 }
 
 - (NSFont *)mainFont {
-    return [[tableColumn_ dataCell] font];
+    return _font ?: [NSFont systemFontOfSize:[NSFont systemFontSize]];
 }
 
 - (NSFont *)tagFont {
@@ -538,41 +639,89 @@ const CGFloat kDefaultTagsWidth = 80;
     return [NSFont systemFontOfSize:self.mainFont.pointSize - reduction];
 }
 
+- (BOOL)lightTheme {
+    return ![self.window.appearance.name isEqual:NSAppearanceNameVibrantDark];
+}
+
+- (NSColor *)regularTextColor {
+    if (@available(macOS 10.14, *)) {
+        return [NSColor labelColor];
+    }
+    if ([self lightTheme]) {
+        return [NSColor blackColor];
+    } else {
+        return [NSColor whiteColor];
+    }
+}
+
+- (NSColor *)textColorWhenInactiveAndSelected {
+    if (@available(macOS 10.14, *)) {
+        return [NSColor unemphasizedSelectedTextColor];
+    }
+    if ([self lightTheme]) {
+        return [NSColor blackColor];
+    } else {
+        return [NSColor whiteColor];
+    }
+}
+
+- (NSColor *)selectedActiveTagColor {
+    if (@available(macOS 10.14, *)) {
+        return [NSColor selectedMenuItemTextColor];
+    }
+    if ([self lightTheme]) {
+        return [NSColor whiteColor];
+    } else {
+        return [NSColor blackColor];
+    }
+}
+
+- (NSColor *)selectedActiveTextColor {
+    if (@available(macOS 10.14, *)) {
+        return [NSColor selectedMenuItemTextColor];
+    }
+    if ([self lightTheme]) {
+        return [NSColor whiteColor];
+    } else {
+        return [NSColor blackColor];
+    }
+}
+
+- (NSColor *)regularTagColor {
+    return [NSColor colorWithCalibratedWhite:0.5 alpha:1];
+}
+
 - (NSAttributedString *)attributedStringForName:(NSString *)name
                                            tags:(NSArray *)tags
                                        selected:(BOOL)selected
                                       isDefault:(BOOL)isDefault
                                          filter:(NSString *)filter {
-    NSColor *textColor;
-    NSColor *tagColor;
-    NSColor *highlightedBackgroundColor;
-    if (selected) {
-        if ([NSApp isActive] && self.window.isKeyWindow) {
-            textColor = [NSColor whiteColor];
-        } else {
-            textColor = [NSColor blackColor];
-        }
-        tagColor = [NSColor whiteColor];
-        highlightedBackgroundColor = [NSColor colorWithCalibratedRed:1 green:1 blue:0 alpha:0.4];
-    } else {
-        textColor = [NSColor blackColor];
-        tagColor = [NSColor colorWithCalibratedWhite:0.5 alpha:1];
-        highlightedBackgroundColor = [NSColor colorWithCalibratedRed:1 green:1 blue:0 alpha:0.4];
-    }
+    NSColor *highlightedBackgroundColor = [NSColor colorWithCalibratedRed:1 green:1 blue:0 alpha:0.4];
+
+    NSColor *textColor = [self regularTextColor];
+    NSColor *highlightedTextColor = [NSColor blackColor];
+    NSColor *tagColor = [self regularTagColor];
+    NSColor *selectedActiveTextColor = [self selectedActiveTextColor];
+    NSColor *selectedActiveTagColor = [self selectedActiveTagColor];
+    
     NSMutableParagraphStyle *paragraphStyle = [[[NSMutableParagraphStyle alloc] init] autorelease];
     paragraphStyle.lineBreakMode = NSLineBreakByTruncatingTail;
 
-    NSDictionary* plainAttributes = @{ NSForegroundColorAttributeName: textColor,
+    NSDictionary* plainAttributes = @{ iTermSelectedActiveForegroundColor: selectedActiveTextColor,
+                                       iTermRegularForegroundColor: textColor,
                                        NSParagraphStyleAttributeName: paragraphStyle,
                                        NSFontAttributeName: self.mainFont };
-    NSDictionary* highlightedNameAttributes = @{ NSForegroundColorAttributeName: textColor,
+    NSDictionary* highlightedNameAttributes = @{ iTermSelectedActiveForegroundColor: selectedActiveTextColor,
+                                                 iTermRegularForegroundColor: highlightedTextColor,
                                                  NSParagraphStyleAttributeName: paragraphStyle,
                                                  NSBackgroundColorAttributeName: highlightedBackgroundColor,
                                                  NSFontAttributeName: self.mainFont };
-    NSDictionary* smallAttributes = @{ NSForegroundColorAttributeName: tagColor,
+    NSDictionary* smallAttributes = @{ iTermSelectedActiveForegroundColor: selectedActiveTagColor,
+                                       iTermRegularForegroundColor: tagColor,
                                        NSParagraphStyleAttributeName: paragraphStyle,
                                        NSFontAttributeName: self.tagFont };
-    NSDictionary* highlightedSmallAttributes = @{ NSForegroundColorAttributeName: tagColor,
+    NSDictionary* highlightedSmallAttributes = @{ iTermSelectedActiveForegroundColor: selectedActiveTagColor,
+                                                  iTermRegularForegroundColor: highlightedTextColor,
                                                   NSParagraphStyleAttributeName: paragraphStyle,
                                                   NSBackgroundColorAttributeName: highlightedBackgroundColor,
                                                   NSFontAttributeName: self.tagFont };
@@ -613,21 +762,70 @@ const CGFloat kDefaultTagsWidth = 80;
 - (NSAttributedString *)attributedStringForString:(NSString *)string selected:(BOOL)selected {
     NSMutableParagraphStyle *paragraphStyle = [[[NSMutableParagraphStyle alloc] init] autorelease];
     paragraphStyle.lineBreakMode = NSLineBreakByTruncatingTail;
+
+    NSColor *textColor = [self regularTextColor];
+    NSColor *selectedActiveTextColor = [self selectedActiveTextColor];
+
     NSDictionary *attributes = @{ NSFontAttributeName: [NSFont systemFontOfSize:[NSFont systemFontSize]],
                                   NSParagraphStyleAttributeName: paragraphStyle,
-                                  NSForegroundColorAttributeName: (selected && [NSApp isActive] && self.window.isKeyWindow) ? [NSColor whiteColor] : [NSColor blackColor] };
-    return [[[NSAttributedString alloc] initWithString:string attributes:attributes] autorelease];
+                                  iTermSelectedActiveForegroundColor: selectedActiveTextColor,
+                                  iTermRegularForegroundColor: textColor };
+    return [[[NSAttributedString alloc] initWithString:string ?: @"" attributes:attributes] autorelease];
 }
 
-- (id)tableView:(NSTableView *)aTableView
-    objectValueForTableColumn:(NSTableColumn *)aTableColumn
-                          row:(NSInteger)rowIndex {
+- (NSView *)tableView:(NSTableView *)tableView viewForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row {
+    static NSString *const identifier = @"ProfileListViewIdentifier";
+    NSTableCellView *result = [tableView makeViewWithIdentifier:identifier owner:self];
+    if (result == nil) {
+        // Apple couldn't be bothered to document it but this method is kind of
+        // supposed to return an NSTableCellView. Its backgroundStyle gets
+        // changed when a row is selected/deselected. It's supposed to pass that
+        // on to its textField, which of course you have to manually create like
+        // an animal unless you use IB for this (which is silly).
+        result = [[[NSTableCellView alloc] initWithFrame:NSMakeRect(0, 0, 100, 100)] autorelease];
+        result.textField = [iTermProfileListViewTextField it_textFieldForTableViewWithIdentifier:identifier];
+        result.textField.frame = result.bounds;
+        result.textField.autoresizingMask = (NSViewWidthSizable |
+                                             NSViewHeightSizable);
+        result.autoresizesSubviews = YES;
+        [result addSubview:result.textField];
+    }
+    BOOL multiline = NO;
+    id value = [self stringOrAttributedStringForColumn:tableColumn row:row multiline:&multiline];
+    // Single line mode breaks vertical alignment
+    result.textField.usesSingleLineMode = NO;
+    if ([value isKindOfClass:[NSAttributedString class]]) {
+        result.textField.attributedStringValue = value;
+        result.textField.toolTip = [value string];
+    } else {
+        result.textField.stringValue = value;
+        result.textField.toolTip = value;
+    }
+    iTermProfileListViewTextField *textField = (id)result.textField;
+    NSTableRowView *rowView = [tableView rowViewAtRow:row makeIfNecessary:NO];
+    if (rowView) {
+        // An explicit call to rowViewAtRow:makeIfNecessary:YES as in
+        // -selectRowIndex: is needed for this to be initialized correctly
+        // if the background style is not default. If there's no rowView at
+        // the time this is called, the backgroundStyle never gets initialized.
+        // I think I'm going to become a farmer.
+        result.backgroundStyle = [rowView interiorBackgroundStyle];
+        textField.backgroundStyle = [rowView interiorBackgroundStyle];
+    }
+    return result;
+}
+
+- (id)stringOrAttributedStringForColumn:(NSTableColumn *)aTableColumn
+                                    row:(NSInteger)rowIndex
+                              multiline:(BOOL *)multilinePtr {
     Profile* bookmark = [dataSource_ profileAtIndex:rowIndex];
 
+    *multilinePtr = 0;
     if (aTableColumn == tableColumn_) {
         DLog(@"Getting name of profile at row %d. The dictionary's address is %p. Its name is %@",
              (int)rowIndex, bookmark, bookmark[KEY_NAME]);
         Profile *defaultProfile = [[ProfileModel sharedInstance] defaultBookmark];
+        *multilinePtr = [bookmark[KEY_TAGS] count] > 0;
         return [self attributedStringForName:bookmark[KEY_NAME] ?: @""
                                         tags:bookmark[KEY_TAGS]
                                     selected:[[tableView_ selectedRowIndexes] containsIndex:rowIndex]
@@ -635,10 +833,12 @@ const CGFloat kDefaultTagsWidth = 80;
                                       filter:[searchField_ stringValue]];
     } else if (aTableColumn == commandColumn_) {
         NSString *theString = nil;
-        if (![[bookmark objectForKey:KEY_CUSTOM_COMMAND] isEqualToString:@"Yes"]) {
-            theString = @"Login shell";
-        } else {
+        NSString *customCommand = bookmark[KEY_CUSTOM_COMMAND];
+        if ([customCommand isEqualToString:kProfilePreferenceCommandTypeCustomValue] ||
+            [customCommand isEqualToString:kProfilePreferenceCommandTypeCustomShellValue]) {
             theString = [bookmark objectForKey:KEY_COMMAND_LINE];
+        } else if ([customCommand isEqualToString:kProfilePreferenceCommandTypeLoginShellValue]) {
+            theString = @"Login shell";
         }
         return [self attributedStringForString:theString
                                       selected:[[tableView_ selectedRowIndexes] containsIndex:rowIndex]];
@@ -720,6 +920,11 @@ const CGFloat kDefaultTagsWidth = 80;
 
 - (void)tableViewSelectionDidChange:(NSNotification *)aNotification
 {
+    if (_restoringSelection) {
+        // After reloadData, setting selection back to what it was.
+        return;
+    }
+
     // There was a click on a row
     if (self.delegate && [self.delegate respondsToSelector:@selector(profileTableSelectionDidChange:)]) {
         [self.delegate profileTableSelectionDidChange:self];
@@ -742,19 +947,38 @@ const CGFloat kDefaultTagsWidth = 80;
     [dataSource_ sync];
     DLog(@"calling reloadData on the profile tableview");
     [tableView_ reloadData];
-    if (self.delegate && ![selectedGuids_ isEqualToSet:[self selectedGuids]]) {
-        [selectedGuids_ release];
-        selectedGuids_ = [self selectedGuids];
-        [selectedGuids_ retain];
-        if ([self.delegate respondsToSelector:@selector(profileTableSelectionDidChange:)]) {
-            [self.delegate profileTableSelectionDidChange:self];
+    NSSet *newSelectedGuids = [NSSet setWithArray:[selectedGuids_.allObjects filteredArrayUsingBlock:^BOOL(id guid) {
+        return ([dataSource_ indexOfProfileWithGuid:guid] != -1);
+    }]];
+    if (self.delegate && [selectedGuids_ isEqualToSet:newSelectedGuids]) {
+        // No change to selection. Don't tell the delegate.
+        _restoringSelection++;
+        [self selectGuids:newSelectedGuids.allObjects];
+        _restoringSelection--;
+    } else {
+        if (self.delegate && ![selectedGuids_ isEqualToSet:newSelectedGuids]) {
+            // Selection is changing.
+            [selectedGuids_ release];
+            selectedGuids_ = newSelectedGuids;
+            [selectedGuids_ retain];
+            if ([self.delegate respondsToSelector:@selector(profileTableSelectionDidChange:)]) {
+                [self.delegate profileTableSelectionDidChange:self];
+            }
         }
-    }
+
+        // Selection changed or there is no delegate.
+        [self selectGuids:newSelectedGuids.allObjects];
+     }
 }
 
-- (void)selectRowIndex:(int)theRow
-{
+- (void)selectRowIndex:(int)theRow {
+    if (theRow >= (int)tableView_.numberOfRows) {
+        return;
+    }
     NSIndexSet* indexes = [NSIndexSet indexSetWithIndex:theRow];
+    // Make sure the rowview exists so its background style can be known when
+    // the NSTableCellView is created.
+    [tableView_ rowViewAtRow:theRow makeIfNecessary:YES];
     [tableView_ selectRowIndexes:indexes byExtendingSelection:NO];
     [tableView_ scrollRowToVisible:theRow];
 }
@@ -841,9 +1065,17 @@ const CGFloat kDefaultTagsWidth = 80;
         dataSource_.lockedGuid = nil;
         [self updateResultsForSearch];
         return YES;
-    } else {
-        return NO;
     }
+    if (commandSelector == @selector(insertNewline:) &&
+        (self.numberOfRows == 1 || tableView_.selectedRow != -1) &&
+        [self.delegate respondsToSelector:@selector(profileTableRowSelected:)]) {
+        if (tableView_.selectedRow == -1) {
+            [tableView_ selectRowIndexes:[NSIndexSet indexSetWithIndex:0] byExtendingSelection:NO];
+        }
+        [self.delegate profileTableRowSelected:self];
+        return YES;
+    }
+    return NO;
 }
 
 
@@ -882,8 +1114,8 @@ const CGFloat kDefaultTagsWidth = 80;
 
     [tableColumn_ setWidth:250];
 
-    [[shortcutColumn_ headerCell] setStringValue:@"Shortcut"];
-    [[commandColumn_ headerCell] setStringValue:@"Command"];
+    shortcutColumn_.title = @"Shortcut";
+    commandColumn_.title = @"Command";
     [tableView_ sizeLastColumnToFit];
 }
 
@@ -953,13 +1185,11 @@ const CGFloat kDefaultTagsWidth = 80;
 - (void)setFont:(NSFont *)theFont
 {
     _haveHeights = NO;
-    for (NSTableColumn *col in [tableView_ tableColumns]) {
-        [[col dataCell] setFont:theFont];
-    }
+    _font = theFont;
 
     if ([theFont pointSize] < 13) {
         [[searchField_ cell] setFont:theFont];
-        [[searchField_ cell] setControlSize:NSSmallControlSize];
+        [[searchField_ cell] setControlSize:NSControlSizeSmall];
         [searchField_ sizeToFit];
 
         margin_ = 5;
@@ -982,8 +1212,6 @@ const CGFloat kDefaultTagsWidth = 80;
     if (open == self.tagsVisible) {
         return;
     }
-    NSRect newTableFrame = tableView_.frame;
-    NSRect newTagsFrame = tagsView_.frame;
     CGFloat newTagsWidth;
     if (open) {
         newTagsWidth = lastTagsWidth_;
@@ -991,14 +1219,16 @@ const CGFloat kDefaultTagsWidth = 80;
         lastTagsWidth_ = tagsView_.frame.size.width;
         newTagsWidth = 0;
     }
-    newTableFrame.size.width =  self.frame.size.width - newTagsWidth;
-    newTagsFrame.size.width = newTagsWidth;
+    const CGFloat oldDividerPosition = NSWidth(tagsView_.frame);
     if (animated) {
-        [tagsView_.animator setFrame:newTagsFrame];
-        [tableView_.animator setFrame:newTableFrame];
+        [[[[iTermSplitViewAnimation alloc] initWithSplitView:splitView_
+                                              dividerAtIndex:0
+                                                        from:oldDividerPosition
+                                                          to:newTagsWidth
+                                                    duration:0.125
+                                                  completion:nil] autorelease] startAnimation];
     } else {
-        tagsView_.frame = newTagsFrame;
-        tableView_.frame = newTableFrame;
+        [splitView_.animator setPosition:newTagsWidth ofDividerAtIndex:0];
     }
 }
 
@@ -1006,10 +1236,46 @@ const CGFloat kDefaultTagsWidth = 80;
     return tagsView_.frame.size.width > 0;
 }
 
+- (CGFloat)tagsFraction {
+    return tagsView_.frame.size.width / splitView_.frame.size.width;
+}
+
+- (void)setTagsFraction:(CGFloat)tagsFraction {
+    NSRect rect = tagsView_.frame;
+    rect.size.width = tagsFraction * splitView_.frame.size.width;
+    tagsView_.frame = rect;
+
+    rect = scrollView_.frame;
+    rect.origin.x = NSMaxX(tagsView_.frame) + splitView_.dividerThickness;
+    rect.size.width = NSWidth(splitView_.frame) - NSMinX(rect);
+    scrollView_.frame = rect;
+    
+    [splitView_ adjustSubviews];
+}
+
+- (NSDictionary *)restorableState {
+    return @{ iTermProfileListViewRestorableStateTagsVisible: @(self.tagsVisible),
+              iTermProfileListViewRestorableStateTagsFraction: @(self.tagsFraction) };
+}
+
+- (void)restoreFromState:(NSDictionary *)state {
+    if (!state) {
+        return;
+    }
+    const CGFloat fraction = [NSNumber castFrom:state[iTermProfileListViewRestorableStateTagsFraction]].doubleValue;
+    if ([state[iTermProfileListViewRestorableStateTagsVisible] boolValue] && fraction > 0 & fraction <= 1) {
+        [self setTagsOpen:YES animated:NO];
+        self.tagsFraction = fraction;
+    }
+}
+
 #pragma mark - ProfileTagsViewDelegate
 
 - (void)profileTagsViewSelectionDidChange:(ProfileTagsView *)profileTagsView {
-    searchField_.stringValue = [profileTagsView.selectedTags componentsJoinedByString:@" "];
+    searchField_.stringValue =
+        [[profileTagsView.selectedTags mapWithBlock:^id(NSString *tag) {
+            return [@"tag:" stringByAppendingString:tag];
+        }] componentsJoinedByString:@" "];
     [self updateResultsForSearch];
 }
 
@@ -1021,5 +1287,7 @@ const CGFloat kDefaultTagsWidth = 80;
         [self.delegate profileTableTagsVisibilityDidChange:self];
     }
     tagsViewIsCollapsed_ = (tagsView_.frame.size.width == 0);
+    [self.window invalidateRestorableState];
 }
+
 @end
