@@ -7,9 +7,12 @@
 //
 
 #import "ProfilesGeneralPreferencesViewController.h"
+
 #import "AdvancedWorkingDirectoryWindowController.h"
+#import "DebugLogging.h"
 #import "ITAddressBookMgr.h"
 #import "iTermAPIHelper.h"
+#import "iTermAdvancedSettingsModel.h"
 #import "iTermBadgeConfigurationWindowController.h"
 #import "iTermFunctionCallTextFieldDelegate.h"
 #import "iTermImageWell.h"
@@ -80,6 +83,8 @@ static NSString *const iTermProfilePreferencesUpdateSessionName = @"iTermProfile
     IBOutlet NSTextField *_badgeLabel;
     IBOutlet NSTextField *_badgeTextForEditCurrentSession;
     IBOutlet NSButton *_editBadgeButton;
+    iTermFunctionCallTextFieldDelegate *_commandDelegate;
+    iTermFunctionCallTextFieldDelegate *_sendTextAtStartDelegate;
     iTermFunctionCallTextFieldDelegate *_profileNameFieldDelegate;
     iTermFunctionCallTextFieldDelegate *_profileNameFieldForEditCurrentSessionDelegate;
     iTermFunctionCallTextFieldDelegate *_badgeTextFieldDelegate;
@@ -95,7 +100,8 @@ static NSString *const iTermProfilePreferencesUpdateSessionName = @"iTermProfile
     IBOutlet NSTextField *_tabTitle;
     IBOutlet NSTextField *_windowTitle;
     IBOutlet NSButton *_allowTitleSetting;
-
+    IBOutlet NSButton *_locked;
+    
     // Controls for Edit Info
     IBOutlet ProfileListView *_profiles;
     IBOutlet iTermShortcutInputView *_sessionHotkeyInputView;
@@ -225,6 +231,15 @@ static NSString *const iTermProfilePreferencesUpdateSessionName = @"iTermProfile
 
     _customCommand.cell.usesSingleLineMode = YES;
     _customCommand.hidden = YES;
+
+    _commandDelegate =
+    [[iTermFunctionCallTextFieldDelegate alloc] initWithPathSource:[iTermVariableHistory pathSourceForContext:iTermVariablesSuggestionContextSession]
+                                                       passthrough:_customCommand.delegate
+                                                     functionsOnly:NO];
+    if ([[self objectForKey:KEY_CUSTOM_COMMAND] isEqual:kProfilePreferenceCommandTypeCustomValue]) {
+        _customCommand.delegate = _commandDelegate;
+    }
+
     info = [self defineControl:_customCommand
                            key:KEY_COMMAND_LINE
                    displayName:@"Profile custom ommand"
@@ -242,6 +257,13 @@ static NSString *const iTermProfilePreferencesUpdateSessionName = @"iTermProfile
             relatedView:_sendTextAtStartLabel
                    type:kPreferenceInfoTypeStringTextField];
 
+    _sendTextAtStartDelegate =
+    [[iTermFunctionCallTextFieldDelegate alloc] initWithPathSource:[iTermVariableHistory pathSourceForContext:iTermVariablesSuggestionContextSession]
+                                                       passthrough:_sendTextAtStart.delegate
+                                                     functionsOnly:NO];
+    _sendTextAtStart.delegate = _sendTextAtStartDelegate;
+
+
     [self defineControl:_initialDirectoryType
                     key:KEY_CUSTOM_DIRECTORY
             displayName:@"Profile initial working directory"
@@ -255,6 +277,7 @@ static NSString *const iTermProfilePreferencesUpdateSessionName = @"iTermProfile
                                                            iTermVariableKeySessionHostname,
                                                            iTermVariableKeySessionName,
                                                            iTermVariableKeySessionJob,
+                                                           iTermVariableKeySessionProcessTitle,
                                                            iTermVariableKeySessionPath,
                                                            iTermVariableKeySessionJobPid] ];
     _customDirectoryTextFieldDelegate =
@@ -369,7 +392,11 @@ static NSString *const iTermProfilePreferencesUpdateSessionName = @"iTermProfile
 }
 
 - (void)sessionNameDidEndEditing {
-    [self setString:_profileNameFieldForEditCurrentSession.stringValue forKey:KEY_TMUX_PANE_TITLE];
+    if ([self stringForKey:KEY_TMUX_PANE_TITLE]) {
+        [self setString:_profileNameFieldForEditCurrentSession.stringValue forKey:KEY_TMUX_PANE_TITLE];
+    } else {
+        [self setString:_profileNameFieldForEditCurrentSession.stringValue forKey:KEY_NAME];
+    }
     [_rateLimit force];
     [_profileDelegate profilesGeneralPreferencesNameDidEndEditing];
 }
@@ -400,9 +427,10 @@ static NSString *const iTermProfilePreferencesUpdateSessionName = @"iTermProfile
     [panel setAllowedFileTypes:[NSImage imageTypes]];
 
     void (^completion)(NSInteger) = ^(NSInteger result) {
-        if (result == NSFileHandlingPanelOKButton) {
+        if (result == NSModalResponseOK) {
             NSURL *url = [[panel URLs] objectAtIndex:0];
             if (![self loadIconWithFilename:url.path]) {
+                DLog(@"Beep: Failed to load icon at %@", url);
                 NSBeep();
             }
         }
@@ -449,6 +477,33 @@ static NSString *const iTermProfilePreferencesUpdateSessionName = @"iTermProfile
 
 - (void)layoutSubviewsForEditCurrentSessionMode {
     self.view = _editCurrentSessionView;
+
+    PreferenceInfo *info = [self infoForControl:_allowTitleSetting];
+    [self setControl:_locked inPreference:info];
+    __weak __typeof(self) weakSelf = self;
+    info.observer = ^{
+        [weakSelf updateLockImage];
+    };
+    info.customSettingChangedHandler = ^(id sender) {
+        [weakSelf toggleLock];
+    };
+    [self updateLockImage];
+}
+
+- (void)toggleLock {
+    [self setBool:![self boolForKey:KEY_ALLOW_TITLE_SETTING] forKey:KEY_ALLOW_TITLE_SETTING];
+}
+
+- (void)updateLockImage {
+    static NSImage *unlockedImage;
+    static NSImage *lockedImage;
+    static dispatch_once_t onceToken;
+    dispatch_once(&onceToken, ^{
+        unlockedImage = [NSImage imageNamed:NSImageNameLockUnlockedTemplate];
+        lockedImage = [NSImage imageNamed:NSImageNameLockLockedTemplate];
+    });
+    const BOOL unlocked = [self boolForKey:KEY_ALLOW_TITLE_SETTING];
+    _locked.image = unlocked ? unlockedImage : lockedImage;
 }
 
 - (void)updateTmuxTabTitle {
@@ -618,7 +673,7 @@ static NSString *const iTermProfilePreferencesUpdateSessionName = @"iTermProfile
 
 - (BOOL)validateMenuItem:(NSMenuItem *)menuItem {
     if (menuItem.menu == _urlSchemes.menu) {
-        menuItem.state = [self profileHandlesScheme:menuItem.title] ? NSOnState : NSOffState;
+        menuItem.state = [self profileHandlesScheme:menuItem.title] ? NSControlStateValueOn : NSControlStateValueOff;
     }
     return YES;
 }
@@ -730,12 +785,15 @@ static NSString *const iTermProfilePreferencesUpdateSessionName = @"iTermProfile
     switch (tag) {
         case iTermGeneralProfilePreferenceCustomCommandTagCustom:
             value = kProfilePreferenceCommandTypeCustomValue;
+            _customCommand.delegate = _commandDelegate;
             break;
         case iTermGeneralProfilePreferenceCustomCommandTagLoginShell:
             value = kProfilePreferenceCommandTypeLoginShellValue;
+            _customCommand.delegate = _commandDelegate.passthrough;
             break;
         case iTermGeneralProfilePreferenceCustomCommandTagCustomShell:
             value = kProfilePreferenceCommandTypeCustomShellValue;
+            _customCommand.delegate = _commandDelegate.passthrough;
             break;
     }
     [self setString:value forKey:KEY_CUSTOM_COMMAND];
@@ -946,7 +1004,7 @@ static NSString *const iTermProfilePreferencesUpdateSessionName = @"iTermProfile
         } else if (item.tag == -1) {
             selected = NO;
         }
-        item.state = selected ? NSOnState : NSOffState;
+        item.state = selected ? NSControlStateValueOn : NSControlStateValueOff;
         if (selected) {
             [parts addObject:item.title];
         }
@@ -1032,6 +1090,10 @@ static NSString *const iTermProfilePreferencesUpdateSessionName = @"iTermProfile
     id control = [aNotification object];
     if (control == _profileNameFieldForEditCurrentSession) {
         [self sessionNameDidChange];
+        if ([self boolForKey:KEY_ALLOW_TITLE_SETTING] &&
+            [iTermAdvancedSettingsModel autoLockSessionNameOnEdit]) {
+            [self toggleLock];
+        }
         return;
     }
     if (control == _tabTitle) {
