@@ -42,6 +42,7 @@ static NSString *const kDeleteKeyString = @"0x7f-0x0";
     IBOutlet NSButton *_useLibTickit;
     IBOutlet NSView *_hotKeyContainerView;
     IBOutlet iTermKeyMappingViewController *_keyMappingViewController;
+    IBOutlet NSButton *_allowModifyOtherKeys;
     iTermHotkeyPreferencesWindowController *_hotkeyPanel;
 }
 
@@ -89,6 +90,10 @@ static NSString *const kDeleteKeyString = @"0x7f-0x0";
                     key:KEY_APPLICATION_KEYPAD_ALLOWED
             relatedView:nil
                    type:kPreferenceInfoTypeCheckbox];
+    [self defineControl:_allowModifyOtherKeys
+                    key:KEY_ALLOW_MODIFY_OTHER_KEYS
+            relatedView:nil
+                   type:kPreferenceInfoTypeCheckbox];
 
     PreferenceInfo *info = [self defineControl:_useLibTickit
                                            key:KEY_USE_LIBTICKIT_PROTOCOL
