@@ -11,15 +11,46 @@
 #import "PseudoTerminalRestorer.h"
 #import "DebugLogging.h"
 #import "iTermAdvancedSettingsModel.h"
+#import "iTermApplication.h"
+#import "iTermApplicationDelegate.h"
 #import "iTermController.h"
 #import "iTermOrphanServerAdopter.h"
 #import "iTermPreferences.h"
+#import "iTermUserDefaults.h"
 #import "NSApplication+iTerm.h"
+#import "NSObject+iTerm.h"
 #import "PseudoTerminal.h"
+#import "PseudoTerminal+Private.h"
+#import "PseudoTerminal+WindowStyle.h"
 
 static NSMutableArray *queuedBlocks;
 typedef void (^VoidBlock)(void);
 static BOOL gWaitingForFullScreen;
+static void (^gPostRestorationCompletionBlock)(void);
+static BOOL gRanQueuedBlocks;
+static BOOL gShouldIgnoreOpenUntitledFile;
+static BOOL gExternalRestorationDidComplete;
+
+@implementation PseudoTerminalState
+
+- (instancetype)initWithCoder:(NSCoder *)coder {
+    return [self initWithDictionary:[NSDictionary castFrom:[coder decodeObjectForKey:kTerminalWindowStateRestorationWindowArrangementKey]]];
+}
+
+- (instancetype)initWithDictionary:(NSDictionary *)arrangement {
+    self = [super init];
+    if (self) {
+        _arrangement = [arrangement retain];
+    }
+    return self;
+}
+
+- (void)dealloc {
+    [_arrangement release];
+    [super dealloc];
+}
+
+@end
 
 @implementation PseudoTerminalRestorer
 
@@ -27,36 +58,7 @@ static BOOL gWaitingForFullScreen;
     return queuedBlocks.count > 0;
 }
 
-+ (BOOL)useElCapitanFullScreenLogic {
-    return [NSWindow instancesRespondToSelector:@selector(maxFullScreenContentSize)];
-}
-
 + (void)runQueuedBlocks {
-    if ([self useElCapitanFullScreenLogic]) {
-        [self runQueuedBlocks_10_11_andLater];
-    } else {
-        [self runQueuedBlocks_10_10_andEarlier];
-    }
-}
-
-// The windows must be open one iteration of mainloop after the application
-// finishes launching. Otherwise, on OS 10.7, non-lion-style fullscreen windows
-// open but the menu bar stays up.
-+ (void)runQueuedBlocks_10_10_andEarlier {
-    DLog(@"runQueuedBlocks (<=10.10) starting");
-    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, 0),
-                   dispatch_get_main_queue(),
-                   ^{
-                       for (VoidBlock block in queuedBlocks) {
-                           block();
-                       }
-                       [queuedBlocks release];
-                       queuedBlocks = nil;
-                   });
-}
-
-// 10.11 wants this to happen right away.
-+ (void)runQueuedBlocks_10_11_andLater {
     DLog(@"runQueuedBlocks (10.11+) starting");
     while (queuedBlocks.count) {
         if (gWaitingForFullScreen) {
@@ -72,11 +74,75 @@ static BOOL gWaitingForFullScreen;
     DLog(@"Ran all queued blocks");
     [queuedBlocks release];
     queuedBlocks = nil;
+    gRanQueuedBlocks = YES;
+    [self runPostRestorationBlockIfNeeded];
+}
+
++ (void)runPostRestorationBlockIfNeeded {
+    if (gPostRestorationCompletionBlock && gExternalRestorationDidComplete) {
+        DLog(@"run post-restoration block %p", gPostRestorationCompletionBlock);
+        gPostRestorationCompletionBlock();
+        [gPostRestorationCompletionBlock release];
+        gPostRestorationCompletionBlock = nil;
+    }
+}
+
++ (void)externalRestorationDidComplete {
+    DLog(@"external restoration completed");
+    gExternalRestorationDidComplete = YES;
+    [self runPostRestorationBlockIfNeeded];
+}
+
++ (void)setPostRestorationCompletionBlock:(void (^)(void))postRestorationCompletionBlock {
+    DLog(@"set post-restoration completion block");
+    if (gRanQueuedBlocks && gExternalRestorationDidComplete) {
+        postRestorationCompletionBlock();
+    } else {
+        if (gPostRestorationCompletionBlock) {
+            void (^oldBlock)(void) = [[gPostRestorationCompletionBlock retain] autorelease];
+            gPostRestorationCompletionBlock = [^{
+                DLog(@"call older postrestoration block");
+                oldBlock();
+                [oldBlock release];
+                postRestorationCompletionBlock();
+            } copy];
+            DLog(@"replace postretoration block %p with new one %p", oldBlock, gPostRestorationCompletionBlock);
+        } else {
+            gPostRestorationCompletionBlock = [postRestorationCompletionBlock copy];
+            DLog(@"postrestoration block is now %p", gPostRestorationCompletionBlock);
+        }
+    }
+}
+
++ (void (^)(void))postRestorationCompletionBlock {
+    return gPostRestorationCompletionBlock;
 }
 
 + (void)restoreWindowWithIdentifier:(NSString *)identifier
                               state:(NSCoder *)state
                   completionHandler:(void (^)(NSWindow *, NSError *))completionHandler {
+    [self restoreWindowWithIdentifier:identifier
+                  pseudoTerminalState:[[[PseudoTerminalState alloc] initWithCoder:state] autorelease]
+                               system:YES
+                    completionHandler:completionHandler];
+}
+
++ (BOOL)shouldIgnoreOpenUntitledFile {
+    return gShouldIgnoreOpenUntitledFile;
+}
+
++ (void)restoreWindowWithIdentifier:(NSString *)identifier
+                pseudoTerminalState:(PseudoTerminalState *)state
+                             system:(BOOL)system
+                  completionHandler:(void (^)(NSWindow *, NSError *))completionHandler {
+    if (system && [iTermUserDefaults ignoreSystemWindowRestoration]) {
+        DLog(@"Ignore system window restoration because we're using our own restorable state controller.");
+        gShouldIgnoreOpenUntitledFile = YES;
+        completionHandler(nil, nil);
+        gShouldIgnoreOpenUntitledFile = NO;
+        DLog(@"Return");
+        return;
+    }
     if ([[NSApplication sharedApplication] isRunningUnitTests]) {
         completionHandler(nil, nil);
         return;
@@ -87,7 +153,7 @@ static BOOL gWaitingForFullScreen;
 
     DLog(@"Restore window with identifier %@", identifier);
     if ([[[NSBundle mainBundle] bundleIdentifier] containsString:@"applescript"]) {
-        // Disable window restoration for iTerm2ForApplescriptTesting
+        // Disable window restoration for iTerm2ForAppleScriptTesting
         DLog(@"Abort because bundle ID contains applescript");
         completionHandler(nil, nil);
         return;
@@ -97,11 +163,11 @@ static BOOL gWaitingForFullScreen;
         completionHandler(nil, nil);
         return;
     }
+    [[[iTermApplication sharedApplication] delegate] willRestoreWindow];
 
     if ([iTermPreferences boolForKey:kPreferenceKeyOpenArrangementAtStartup]) {
         DLog(@"Abort because opening arrangement at startup");
-        NSDictionary *arrangement =
-            [state decodeObjectForKey:kTerminalWindowStateRestorationWindowArrangementKey];
+        NSDictionary *arrangement = state.arrangement;
         if (arrangement) {
             [PseudoTerminal registerSessionsInArrangement:arrangement];
         }
@@ -116,12 +182,8 @@ static BOOL gWaitingForFullScreen;
     if (!queuedBlocks) {
         DLog(@"This is the first run of PseudoTerminalRestorer");
         queuedBlocks = [[NSMutableArray alloc] init];
-        [[NSNotificationCenter defaultCenter] addObserver:[PseudoTerminalRestorer class]
-                                                 selector:@selector(runQueuedBlocks)
-                                                     name:kApplicationDidFinishLaunchingNotification
-                                                   object:nil];
     }
-    NSDictionary *arrangement = [state decodeObjectForKey:kTerminalWindowStateRestorationWindowArrangementKey];
+    NSDictionary *arrangement = [state.arrangement retain];
     if (arrangement) {
         DLog(@"Have an arrangement");
         VoidBlock theBlock = ^{
@@ -129,6 +191,7 @@ static BOOL gWaitingForFullScreen;
             DLog(@"Creating term");
             PseudoTerminal *term = [PseudoTerminal bareTerminalWithArrangement:arrangement
                                                       forceOpeningHotKeyWindow:NO];
+            [arrangement autorelease];
             DLog(@"Create a new terminal %@", term);
             if (!term) {
                 DLog(@"Failed to create term");
@@ -148,32 +211,36 @@ static BOOL gWaitingForFullScreen;
                 case WINDOW_TYPE_TOP_PARTIAL:
                 case WINDOW_TYPE_BOTTOM:
                 case WINDOW_TYPE_BOTTOM_PARTIAL:
+                case WINDOW_TYPE_MAXIMIZED:
+                case WINDOW_TYPE_COMPACT_MAXIMIZED:
                     DLog(@"Canonicalizing window frame");
                     [term performSelector:@selector(canonicalizeWindowFrame)
                                withObject:nil
                                afterDelay:0];
                     break;
-                    
+
                 case WINDOW_TYPE_LEFT:
                 case WINDOW_TYPE_RIGHT:
                 case WINDOW_TYPE_NORMAL:
                 case WINDOW_TYPE_LEFT_PARTIAL:
                 case WINDOW_TYPE_NO_TITLE_BAR:
+                case WINDOW_TYPE_COMPACT:
                 case WINDOW_TYPE_RIGHT_PARTIAL:
                 case WINDOW_TYPE_LION_FULL_SCREEN:
+                case WINDOW_TYPE_ACCESSORY:
                     break;
             }
 
             DLog(@"Invoking completion handler");
-            if (![self useElCapitanFullScreenLogic] || !term.togglingLionFullScreen) {
-                // In 10.10 or earlier, or 10.11 and a nonfullscreen window.
+            if (!term.togglingLionFullScreen) {
+                DLog(@"In 10.10 or earlier, or 10.11 and a nonfullscreen window");
                 term.restoringWindow = YES;
                 completionHandler([term window], nil);
                 term.restoringWindow = NO;
                 DLog(@"Registering terminal window");
                 [[iTermController sharedInstance] addTerminalWindow:term];
             } else {
-                // 10.11 and this is a fullscreen window.
+                DLog(@"10.11 and this is a fullscreen window.");
                 // Keep any more blocks from running until this window finishes entering fullscreen.
                 gWaitingForFullScreen = YES;
                 DLog(@"Set gWaitingForFullScreen=YES and set callback on %@", term);
