@@ -36,15 +36,48 @@
     BOOL _needsClear;
 }
 
-- (void)drawRect:(NSRect)rect {
-    if (_useMetal) {
-        if (_needsClear) {
-            [[NSColor clearColor] set];
-            NSRectFill(rect);
-            _needsClear = NO;
+- (instancetype)initWithFrame:(NSRect)frameRect {
+    self = [super initWithFrame:frameRect];
+    if (self) {
+        if (@available(macOS 10.14, *)) {
+            [[NSNotificationCenter defaultCenter] addObserver:self
+                                                     selector:@selector(scrollViewDidScroll:)
+                                                         name:NSViewBoundsDidChangeNotification
+                                                       object:nil];
         }
+    }
+    return self;
+}
+
+- (void)dealloc {
+    [[NSNotificationCenter defaultCenter] removeObserver:self];
+    [super dealloc];
+}
+
+// This is a hack to fix an apparent bug in macOS 10.14 beta 3. I would like to remove it when it's no longer needed.
+// https://openradar.appspot.com/radar?id=6090021505335296
+// rdar://42228044
+- (void)scrollViewDidScroll:(NSNotification *)notification {
+    if (notification.object != self.superview) {
         return;
     }
+    [self setNeedsDisplay:YES];
+}
+
+- (void)drawRect:(NSRect)bogusRect {
+    // The passed-in rect tends to be 0x0 but respecting it leaves visible
+    // parts undrawn. Some day macOS 10.0's features will work correctly but
+    // I'm not holding my breath.
+    NSRect rect = self.enclosingScrollView.documentVisibleRect;
+    rect.size.height = [iTermAdvancedSettingsModel terminalVMargin];
+    if (_useMetal) {
+        if (@available(macOS 10.14, *)) {
+            return;
+        }
+        if (!_needsClear) {
+            return;
+        }
+    }
     [child_.delegate textViewDrawBackgroundImageInView:self
                                               viewRect:rect
                                 blendDefaultBackground:YES];
