@@ -26,6 +26,7 @@
 #import "iTermShellPromptTrigger.h"
 #import "MarkTrigger.h"
 #import "NSColor+iTerm.h"
+#import "NSObject+iTerm.h"
 #import "PasswordTrigger.h"
 #import "ProfileModel.h"
 #import "ScriptTrigger.h"
@@ -43,8 +44,8 @@ static NSString *const kiTermTriggerControllerPasteboardType =
 
 static NSString *const kRegexColumnIdentifier = @"kRegexColumnIdentifier";
 static NSString *const kParameterColumnIdentifier = @"kParameterColumnIdentifier";
-static NSString *const kTextColorWellIdentifier = @"kTextColorWellIdentifier";
-static NSString *const kBackgroundColorWellIdentifier = @"kBackgroundColorWellIdentifier";
+NSString *const kTextColorWellIdentifier = @"kTextColorWellIdentifier";
+NSString *const kBackgroundColorWellIdentifier = @"kBackgroundColorWellIdentifier";
 
 // This is a color well that continues to work after it's removed from the view
 // hierarchy. NSTableView likes to randomly remove its views, so a regular
@@ -68,7 +69,7 @@ static NSString *const kBackgroundColorWellIdentifier = @"kBackgroundColorWellId
 
 @end
 
-@interface TriggerController() <NSTextFieldDelegate>
+@interface TriggerController() <iTermTriggerParameterController>
 // Keeps the color well whose popover is currently open from getting
 // deallocated. It may get removed from the view hierarchy but we need it to
 // continue existing so we can get the color out of it.
@@ -94,7 +95,7 @@ static NSString *const kBackgroundColorWellIdentifier = @"kBackgroundColorWellId
     self = [super init];
     if (self) {
         NSMutableArray *triggers = [NSMutableArray array];
-        for (Class class in [self triggerClasses]) {
+        for (Class class in [self.class triggerClasses]) {
             [triggers addObject:[[class alloc] init]];
         }
         _triggers = triggers;
@@ -103,7 +104,7 @@ static NSString *const kBackgroundColorWellIdentifier = @"kBackgroundColorWellId
     return self;
 }
 
-- (NSArray *)triggerClasses {
++ (NSArray<Class> *)triggerClasses {
     NSArray *allClasses = @[ [AlertTrigger class],
                              [AnnotateTrigger class],
                              [BellTrigger class],
@@ -143,24 +144,28 @@ static NSString *const kBackgroundColorWellIdentifier = @"kBackgroundColorWellId
 }
 
 - (int)numberOfTriggers {
-    return [[self triggerClasses] count];
+    return [[self.class triggerClasses] count];
 }
 
 - (int)indexOfAction:(NSString *)action {
     int n = [self numberOfTriggers];
-    NSArray *classes = [self triggerClasses];
+    NSArray *classes = [self.class triggerClasses];
     for (int i = 0; i < n; i++) {
         NSString *className = NSStringFromClass(classes[i]);
         if ([className isEqualToString:action]) {
             return i;
         }
+        Class theClass = NSClassFromString(className);
+        if ([[theClass synonyms] containsObject:action]) {
+            return i;
+        }
     }
     return -1;
 }
 
 // Index in triggerClasses of an object of class "c"
 - (NSInteger)indexOfTriggerClass:(Class)c {
-    NSArray *classes = [self triggerClasses];
+    NSArray *classes = [self.class triggerClasses];
     for (int i = 0; i < classes.count; i++) {
         if (classes[i] == c) {
             return i;
@@ -243,10 +248,10 @@ static NSString *const kBackgroundColorWellIdentifier = @"kBackgroundColorWellId
 - (void)setGuid:(NSString *)guid {
     _guid = [guid copy];
     [_tableView reloadData];
-    _interpolatedStringParameters.state = [iTermProfilePreferences boolForKey:KEY_TRIGGERS_USE_INTERPOLATED_STRINGS inProfile:[self bookmark]] ? NSOnState : NSOffState;
+    _interpolatedStringParameters.state = [iTermProfilePreferences boolForKey:KEY_TRIGGERS_USE_INTERPOLATED_STRINGS inProfile:[self bookmark]] ? NSControlStateValueOn : NSControlStateValueOff;
 }
 
-- (NSTextField *)labelWithString:(NSString *)string origin:(NSPoint)origin {
++ (NSTextField *)labelWithString:(NSString *)string origin:(NSPoint)origin {
     NSTextField *textField = [[NSTextField alloc] initWithFrame:NSMakeRect(origin.x,
                                                                            origin.y,
                                                                            0,
@@ -279,7 +284,9 @@ static NSString *const kBackgroundColorWellIdentifier = @"kBackgroundColorWellId
         [indexes addObject:@(idx)];
     }];
 
-    NSData *data = [NSKeyedArchiver archivedDataWithRootObject:indexes];
+    NSData *data = [NSKeyedArchiver archivedDataWithRootObject:indexes
+                                         requiringSecureCoding:NO
+                                                         error:nil];
     [pasteboard declareTypes:@[ kiTermTriggerControllerPasteboardType ] owner:self];
     [pasteboard setData:data forType:kiTermTriggerControllerPasteboardType];
     return YES;
@@ -312,7 +319,14 @@ static NSString *const kBackgroundColorWellIdentifier = @"kBackgroundColorWellId
     dropOperation:(NSTableViewDropOperation)operation {
     NSPasteboard *pasteboard = [info draggingPasteboard];
     NSData *rowData = [pasteboard dataForType:kiTermTriggerControllerPasteboardType];
-    NSArray *indexes = [NSKeyedUnarchiver unarchiveObjectWithData:rowData];
+    NSError *error = nil;
+    NSArray *indexes = [NSKeyedUnarchiver unarchivedObjectOfClass:[NSArray class]
+                                                         fromData:rowData
+                                                            error:&error];
+    if (error) {
+        DLog(@"Drop failed: %@", error);
+        return NO;
+    }
 
     // This code assumes you can only select one trigger at a time.
     int sourceRow = [indexes[0] intValue];
@@ -337,6 +351,108 @@ static NSString *const kBackgroundColorWellIdentifier = @"kBackgroundColorWellId
     return NO;
 }
 
++ (NSView *)viewForParameterForTrigger:(Trigger *)trigger
+                                  size:(CGSize)size
+                                 value:(id)value
+                              receiver:(id<iTermTriggerParameterController>)receiver
+                   interpolatedStrings:(BOOL)interpolatedStrings
+                           wellFactory:(CPKColorWell *(^ NS_NOESCAPE)(NSRect, NSColor *))wellFactory {
+    if (![trigger takesParameter]) {
+        return [[NSView alloc] initWithFrame:NSZeroRect];
+    }
+    if ([trigger paramIsTwoColorWells]) {
+        NSView *container = [[NSView alloc] initWithFrame:NSMakeRect(0,
+                                                                     0,
+                                                                     size.width,
+                                                                     size.height)];
+        CGFloat x = 4;
+        NSTextField *label = [self labelWithString:@"Text:" origin:NSMakePoint(x, 0)];
+        [container addSubview:label];
+        x += label.frame.size.width;
+        const CGFloat kWellWidth = 30;
+        CPKColorWell *well = wellFactory(NSMakeRect(x,
+                                                    0,
+                                                    kWellWidth,
+                                                    size.height),
+                                         trigger.textColor);
+        well.identifier = kTextColorWellIdentifier;
+
+        [container addSubview:well];
+
+        x += 10 + kWellWidth;
+        label = [self labelWithString:@"Background:" origin:NSMakePoint(x, 0)];
+        [container addSubview:label];
+        x += label.frame.size.width;
+
+        well = wellFactory(NSMakeRect(x,
+                                      0,
+                                      kWellWidth,
+                                      size.height),
+                           trigger.backgroundColor);
+        [container addSubview:well];
+        well.identifier = kBackgroundColorWellIdentifier;
+        return container;
+    }
+    if ([trigger paramIsPopupButton]) {
+        NSPopUpButton *popUpButton = [[NSPopUpButton alloc] init];
+        [popUpButton setTitle:@""];
+        popUpButton.bordered = NO;
+
+        NSMenu *theMenu = popUpButton.menu;
+        BOOL isFirst = YES;
+        for (NSDictionary *items in [trigger groupedMenuItemsForPopupButton]) {
+            if (!isFirst) {
+                [theMenu addItem:[NSMenuItem separatorItem]];
+            }
+            isFirst = NO;
+            for (id object in [trigger objectsSortedByValueInDict:items]) {
+                NSString *theTitle = [items objectForKey:object];
+                if (theTitle) {
+                    NSMenuItem *anItem = [[NSMenuItem alloc] initWithTitle:theTitle
+                                                                    action:nil
+                                                             keyEquivalent:@""];
+                    [theMenu addItem:anItem];
+                }
+            }
+        }
+
+        id param = value;
+        if (!param) {
+            // Force popup buttons to have the first item selected by default
+            [popUpButton selectItemAtIndex:trigger.defaultIndex];
+        } else {
+            [popUpButton selectItemAtIndex:[trigger indexForObject:param]];
+        }
+        popUpButton.target = receiver;
+        popUpButton.action = @selector(parameterPopUpButtonDidChange:);
+
+        return popUpButton;
+    }
+
+
+    // If not a popup button, then text by default.
+    iTermFocusReportingTextField *textField =
+    [[iTermFocusReportingTextField alloc] initWithFrame:NSMakeRect(0,
+                                                                   0,
+                                                                   size.width,
+                                                                   size.height)];
+    textField.font = [NSFont systemFontOfSize:[NSFont systemFontSize]];
+    textField.stringValue = value ?: @"";
+    textField.editable = YES;
+    textField.selectable = YES;
+    textField.bordered = NO;
+    textField.drawsBackground = NO;
+    if ([textField respondsToSelector:@selector(setPlaceholderString:)]) {
+        textField.placeholderString = [trigger triggerOptionalParameterPlaceholderWithInterpolation:interpolatedStrings];
+    }
+    id<iTermFocusReportingTextFieldDelegate> delegate =
+        [trigger newParameterDelegateWithPassthrough:receiver]  ?: receiver;
+    textField.delegate = delegate;
+    textField.identifier = kParameterColumnIdentifier;
+
+    return textField;
+}
+
 - (NSView *)tableView:(NSTableView *)tableView
    viewForTableColumn:(NSTableColumn *)tableColumn
                   row:(NSInteger)row {
@@ -374,9 +490,9 @@ static NSString *const kBackgroundColorWellIdentifier = @"kBackgroundColorWellId
     } else if (tableColumn == _partialLineColumn) {
         NSButton *checkbox = [[NSButton alloc] initWithFrame:NSZeroRect];
         [checkbox sizeToFit];
-        [checkbox setButtonType:NSSwitchButton];
+        [checkbox setButtonType:NSButtonTypeSwitch];
         checkbox.title = @"";
-        checkbox.state = [triggerDictionary[kTriggerPartialLineKey] boolValue] ? NSOnState : NSOffState;
+        checkbox.state = [triggerDictionary[kTriggerPartialLineKey] boolValue] ? NSControlStateValueOn : NSControlStateValueOff;
         checkbox.target = self;
         checkbox.action = @selector(instantDidChange:);
         return checkbox;
@@ -384,131 +500,43 @@ static NSString *const kBackgroundColorWellIdentifier = @"kBackgroundColorWellId
         NSArray *triggerDicts = [self triggerDictionariesForCurrentProfile];
         Trigger *trigger = [self triggerWithAction:triggerDicts[row][kTriggerActionKey]];
         trigger.param = triggerDicts[row][kTriggerParameterKey];
-        if ([trigger takesParameter]) {
-            if ([trigger paramIsTwoColorWells]) {
-                NSView *container = [[NSView alloc] initWithFrame:NSMakeRect(0,
-                                                                             0,
-                                                                             tableColumn.width,
-                                                                             _tableView.rowHeight)];
-                CGFloat x = 4;
-                NSTextField *label = [self labelWithString:@"Text:" origin:NSMakePoint(x, 0)];
-                [container addSubview:label];
-                x += label.frame.size.width;
-                const CGFloat kWellWidth = 30;
-                iTermColorWell *well =
-                    [[iTermColorWell alloc] initWithFrame:NSMakeRect(x,
-                                                                     0,
-                                                                     kWellWidth,
-                                                                     _tableView.rowHeight)];
-                well.noColorAllowed = YES;
-                well.continuous = NO;
-                well.tag = row;
-                x += kWellWidth;
-                well.color = trigger.textColor;
-                [container addSubview:well];
-                well.target = self;
-                well.action = @selector(colorWellDidChange:);
-                well.identifier = kTextColorWellIdentifier;
-                __weak __typeof(self) weakSelf = self;
-                __weak __typeof(well) weakWell = well;
-                well.willOpenPopover = ^() {
-                    if (weakWell) {
-                        weakSelf.activeWell = weakWell;
-                    }
-                };
-                well.willClosePopover = ^() {
-                    if (self.activeWell == well) {
-                        self.activeWell = nil;
-                    }
-                };
-                x += 10;
-                label = [self labelWithString:@"Background:" origin:NSMakePoint(x, 0)];
-                [container addSubview:label];
-                x += label.frame.size.width;
-                well = [[iTermColorWell alloc] initWithFrame:NSMakeRect(x,
-                                                                        0,
-                                                                        kWellWidth,
-                                                                        _tableView.rowHeight)];
-                well.noColorAllowed = YES;
-                well.continuous = NO;
-                well.color = trigger.backgroundColor;
-                well.tag = row;
-                [container addSubview:well];
-                well.target = self;
-                well.action = @selector(colorWellDidChange:);
-                well.identifier = kBackgroundColorWellIdentifier;
-                weakWell = well;
-                well.willOpenPopover = ^() {
-                    if (weakWell) {
-                        weakSelf.activeWell = weakWell;
-                    }
-                };
-                well.willClosePopover = ^() {
-                    if (self.activeWell == well) {
-                        self.activeWell = nil;
-                    }
-                };
-
-                return container;
-            } else if ([trigger paramIsPopupButton]) {
-                NSPopUpButton *popUpButton = [[NSPopUpButton alloc] init];
-                [popUpButton setTitle:@""];
-                popUpButton.bordered = NO;
-
-                NSMenu *theMenu = popUpButton.menu;
-                BOOL isFirst = YES;
-                for (NSDictionary *items in [trigger groupedMenuItemsForPopupButton]) {
-                    if (!isFirst) {
-                        [theMenu addItem:[NSMenuItem separatorItem]];
-                    }
-                    isFirst = NO;
-                    for (id object in [trigger objectsSortedByValueInDict:items]) {
-                        NSString *theTitle = [items objectForKey:object];
-                        if (theTitle) {
-                            NSMenuItem *anItem = [[NSMenuItem alloc] initWithTitle:theTitle
-                                                                            action:nil
-                                                                     keyEquivalent:@""];
-                            [theMenu addItem:anItem];
-                        }
-                    }
+        NSView *result = [self.class viewForParameterForTrigger:trigger
+                                                           size:NSMakeSize(tableColumn.width, _tableView.rowHeight)
+                                                          value:triggerDictionary[kTriggerParameterKey]
+                                                       receiver:self
+                                            interpolatedStrings:_interpolatedStringParameters.state == NSControlStateValueOn
+                                                    wellFactory:
+                          ^iTermColorWell *(NSRect frame,
+                                            NSColor *color) {
+            iTermColorWell *well = [[iTermColorWell alloc] initWithFrame:frame];
+            well.noColorAllowed = YES;
+            well.continuous = NO;
+            well.tag = row;
+            well.color = color;
+            well.target = self;
+            well.action = @selector(colorWellDidChange:);
+            __weak __typeof(self) weakSelf = self;
+            __weak __typeof(well) weakWell = well;
+            well.willOpenPopover = ^() {
+                if (weakWell) {
+                    weakSelf.activeWell = weakWell;
                 }
-
-                id param = triggerDictionary[kTriggerParameterKey];
-                if (!param) {
-                    // Force popup buttons to have the first item selected by default
-                    [popUpButton selectItemAtIndex:trigger.defaultIndex];
-                } else {
-                    [popUpButton selectItemAtIndex:[trigger indexForObject:param]];
+            };
+            well.willClosePopover = ^() {
+                if (self.activeWell == well) {
+                    self.activeWell = nil;
                 }
-                popUpButton.target = self;
-                popUpButton.action = @selector(parameterPopUpButtonDidChange:);
-
-                return popUpButton;
-            } else {
-                // If not a popup button, then text by default.
-                iTermFocusReportingTextField *textField =
-                    [[iTermFocusReportingTextField alloc] initWithFrame:NSMakeRect(0,
-                                                                                   0,
-                                                                                   tableColumn.width,
-                                                                                   self.tableView.rowHeight)];
-                textField.font = [NSFont systemFontOfSize:[NSFont systemFontSize]];
-                textField.stringValue = triggerDictionary[kTriggerParameterKey] ?: @"";
-                textField.editable = YES;
-                textField.selectable = YES;
-                textField.bordered = NO;
-                textField.drawsBackground = NO;
-                _parameterDelegate = [trigger newParameterDelegateWithPassthrough:self];
-                textField.delegate = _parameterDelegate ?: self;
-                if ([textField respondsToSelector:@selector(setPlaceholderString:)]) {
-                    textField.placeholderString = [trigger triggerOptionalParameterPlaceholderWithInterpolation:_interpolatedStringParameters.state == NSOnState];
-                }
-                textField.identifier = kParameterColumnIdentifier;
-
-                return textField;
+            };
+            return well;
+        }];
+        NSTextField *textField = [NSTextField castFrom:result];
+        if (textField) {
+            if (textField.delegate != self) {
+                _parameterDelegate = textField.delegate;
             }
-        } else {
-            return [[NSView alloc] initWithFrame:NSZeroRect];
         }
+
+        return result;
     }
     return nil;
 }
@@ -547,8 +575,8 @@ static NSString *const kBackgroundColorWellIdentifier = @"kBackgroundColorWellId
 }
 
 - (IBAction)toggleUseInterpolatedStrings:(id)sender {
-    _interpolatedStringParameters.state = (![iTermProfilePreferences boolForKey:KEY_TRIGGERS_USE_INTERPOLATED_STRINGS inProfile:[self bookmark]]) ? NSOnState : NSOffState;
-    [self.delegate triggerSetUseInterpolatedStrings:_interpolatedStringParameters.state == NSOnState];
+    _interpolatedStringParameters.state = (![iTermProfilePreferences boolForKey:KEY_TRIGGERS_USE_INTERPOLATED_STRINGS inProfile:[self bookmark]]) ? NSControlStateValueOn : NSControlStateValueOff;
+    [self.delegate triggerSetUseInterpolatedStrings:_interpolatedStringParameters.state == NSControlStateValueOn];
     [_tableView reloadData];
 }
 
@@ -600,7 +628,7 @@ static NSString *const kBackgroundColorWellIdentifier = @"kBackgroundColorWellId
 }
 
 - (void)instantDidChange:(NSButton *)checkbox {
-    NSNumber *newValue = checkbox.state == NSOnState ? @(YES) : @(NO);
+    NSNumber *newValue = checkbox.state == NSControlStateValueOn ? @(YES) : @(NO);
     NSInteger row = [_tableView rowForView:checkbox];
 
     // If a text field is editing, make it save its contents before we get the trigger dictionary.
@@ -634,8 +662,8 @@ static NSString *const kBackgroundColorWellIdentifier = @"kBackgroundColorWellId
     Trigger *triggerObj = [self triggerWithAction:triggerDictionary[kTriggerActionKey]];
     if ([triggerObj paramIsPopupButton]) {
         triggerDictionary[kTriggerParameterKey] = [triggerObj defaultPopupParameterObject];
-    } else if ([triggerObj triggerOptionalDefaultParameterValueWithInterpolation:_interpolatedStringParameters.state == NSOnState]) {
-        triggerDictionary[kTriggerParameterKey] = [triggerObj triggerOptionalDefaultParameterValueWithInterpolation:_interpolatedStringParameters.state == NSOnState];
+    } else if ([triggerObj triggerOptionalDefaultParameterValueWithInterpolation:_interpolatedStringParameters.state == NSControlStateValueOn]) {
+        triggerDictionary[kTriggerParameterKey] = [triggerObj triggerOptionalDefaultParameterValueWithInterpolation:_interpolatedStringParameters.state == NSControlStateValueOn];
     }
     [self setTriggerDictionary:triggerDictionary forRow:rowIndex reloadData:YES];
 }
