@@ -10,16 +10,20 @@
 
 #import "Api.pbobjc.h"
 #import "DebugLogging.h"
+#import "iTermAdvancedSettingsModel.h"
 #import "iTermHTTPConnection.h"
 #import "iTermLSOF.h"
 #import "iTermWebSocketConnection.h"
 #import "iTermWebSocketFrame.h"
 #import "iTermSocket.h"
-#import "iTermIPV4Address.h"
-#import "iTermSocketIPV4Address.h"
+#import "iTermSocketAddress.h"
 #import "NSArray+iTerm.h"
+#import "NSFileManager+iTerm.h"
 #import "NSObject+iTerm.h"
+
 #import <objc/runtime.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 
 #import <Cocoa/Cocoa.h>
 
@@ -125,7 +129,7 @@ NSString *const iTermAPIServerConnectionClosed = @"iTermAPIServerConnectionClose
 @end
 
 @implementation iTermAPIServer {
-    iTermSocket *_socket;
+    iTermSocket *_unixSocket;
     NSMutableDictionary<id, iTermWebSocketConnection *> *_connections;  // _queue
     dispatch_queue_t _executionQueue;
     NSMutableArray<iTermHTTPConnection *> *_pendingConnections;  // _queue
@@ -141,39 +145,60 @@ NSString *const iTermAPIServerConnectionClosed = @"iTermAPIServerConnectionClose
     return instance;
 }
 
++ (NSString *)folderForUnixSocket {
+    return [[[NSFileManager defaultManager] applicationSupportDirectory] stringByAppendingPathComponent:@"private"];
+}
+
++ (NSString *)unixSocketPath {
+    return [[self folderForUnixSocket] stringByAppendingPathComponent:@"socket"];
+}
+
 - (instancetype)init {
     self = [super init];
     if (self) {
         _connections = [[NSMutableDictionary alloc] init];
-        _socket = [iTermSocket tcpIPV4Socket];
-        if (!_socket) {
-            XLog(@"Failed to create socket");
+        _unixSocket = [iTermSocket unixDomainSocket];
+        if (!_unixSocket) {
+            XLog(@"Failed to create unix socket");
             return nil;
         }
         _pendingConnections = [NSMutableArray array];
         _queue = dispatch_queue_create("com.iterm2.apisockets", NULL);
         _executionQueue = dispatch_queue_create("com.iterm2.apiexec", DISPATCH_QUEUE_SERIAL);
 
-        [_socket setReuseAddr:YES];
-        iTermIPV4Address *loopback = [[iTermIPV4Address alloc] initWithLoopback];
-        iTermSocketAddress *socketAddress = [iTermSocketAddress socketAddressWithIPV4Address:loopback
-                                                                                        port:1912];
-        if (![_socket bindToAddress:socketAddress]) {
-            XLog(@"Failed to bind");
-            return nil;
-        }
-
-        BOOL ok = [_socket listenWithBacklog:5 accept:^(int fd, iTermSocketAddress *clientAddress) {
-            [self didAcceptConnectionOnFileDescriptor:fd fromAddress:clientAddress retries:1];
-        }];
-        if (!ok) {
-            XLog(@"Failed to listen");
+        if (![self listenOnUnixSocket]) {
             return nil;
         }
     }
     return self;
 }
 
+- (BOOL)listenOnUnixSocket {
+    iTermSocketAddress *socketAddress = nil;
+    NSString *path = [iTermAPIServer unixSocketPath];
+    [[NSFileManager defaultManager] createDirectoryAtPath:[iTermAPIServer folderForUnixSocket]
+                              withIntermediateDirectories:YES
+                                               attributes:@{ NSFilePosixPermissions: @(S_IRWXU) }
+                                                    error:nil];
+    socketAddress = [iTermSocketAddress socketAddressWithPath:path];
+    unlink(path.UTF8String);
+    if (![_unixSocket bindToAddress:socketAddress]) {
+        XLog(@"Failed to bind");
+        return NO;
+    }
+    chmod([iTermAPIServer unixSocketPath].UTF8String, (S_IRUSR | S_IWUSR));
+
+    BOOL ok = [_unixSocket listenWithBacklog:5 accept:^(int fd, iTermSocketAddress *clientAddress, NSNumber *euid) {
+        [self didAcceptConnectionOnFileDescriptor:fd fromAddress:clientAddress euid:euid retries:1];
+    }];
+    if (!ok) {
+        XLog(@"Failed to listen");
+        return NO;
+    }
+
+    return YES;
+}
+
 - (void)postAPINotification:(ITMNotification *)notification toConnectionKey:(NSString *)connectionKey {
     dispatch_async(_queue, ^{
         iTermWebSocketConnection *webSocketConnection = self->_connections[connectionKey];
@@ -189,8 +214,8 @@ NSString *const iTermAPIServerConnectionClosed = @"iTermAPIServerConnectionClose
 
 - (void)stop {
     self.delegate = nil;
-    [_socket close];
-    _socket = nil;
+    [_unixSocket close];
+    _unixSocket = nil;
     dispatch_sync(_queue, ^{
         [self->_pendingConnections enumerateObjectsUsingBlock:^(iTermHTTPConnection * _Nonnull connection, NSUInteger idx, BOOL * _Nonnull stop) {
             [connection threadSafeClose];
@@ -217,11 +242,16 @@ NSString *const iTermAPIServerConnectionClosed = @"iTermAPIServerConnectionClose
 
 }
 
-- (void)didAcceptConnectionOnFileDescriptor:(int)fd fromAddress:(iTermSocketAddress *)address retries:(NSInteger)retries {
+- (void)didAcceptConnectionOnFileDescriptor:(int)fd
+                                fromAddress:(iTermSocketAddress *)address
+                                       euid:(NSNumber *)euid
+                                    retries:(NSInteger)retries {
     DLog(@"Accepted connection");
     dispatch_queue_t queue = _queue;
     dispatch_async(queue, ^{
-        iTermHTTPConnection *connection = [[iTermHTTPConnection alloc] initWithFileDescriptor:fd clientAddress:address];
+        iTermHTTPConnection *connection = [[iTermHTTPConnection alloc] initWithFileDescriptor:fd
+                                                                                clientAddress:address
+                                                                                         euid:euid];
         [self->_pendingConnections addObject:connection];
         [self reallyDidAcceptConnection:connection retries:retries];
     });
@@ -230,47 +260,34 @@ NSString *const iTermAPIServerConnectionClosed = @"iTermAPIServerConnectionClose
 // run on _queue
 - (void)reallyDidAcceptConnection:(iTermHTTPConnection *)connection
                           retries:(NSInteger)retries {
-    dispatch_queue_t queue = _queue;
-    [iTermLSOF getProcessIDsWithConnectionFromAddress:connection.clientAddress
-                                                queue:queue
-                                           completion:^(NSArray<NSNumber *> *pids) {
-        if (!pids.count) {
-            if (retries > 0) {
-                // There seems to be a race where the kernel doesn't always
-                // report that a process ID has a file descriptor matching this
-                // address. Wait a bit and try again. Issue 8684.
-                dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.25 * NSEC_PER_SEC)), queue, ^{
-                    [self reallyDidAcceptConnection:connection
-                                            retries:retries - 1];
-                });
-                return;
-            }
-            XLog(@"Reject connection from unidentifiable process with address %@", connection.clientAddress);
-            dispatch_async(dispatch_get_main_queue(), ^{
-                [[NSNotificationCenter defaultCenter] postNotificationName:iTermAPIServerConnectionRejected
-                                                                    object:nil
-                                                                  userInfo:@{ @"reason": [NSString stringWithFormat:@"Could not get process ID for connection from port %@", @(connection.clientAddress.port)] }];
-            });
+    DLog(@"reallyDidAcceptConnection with retries=%@", @(retries));
+    if (!connection.euid || connection.euid.unsignedIntValue != geteuid()) {
+        DLog(@"Deny bad euid %@ != mine of %@", connection.euid, @(geteuid()));
+        dispatch_async(connection.queue, ^{
+            NSString *reason = [NSString stringWithFormat:@"Peer's euid of %@ not equal to my euid of %@",
+                                connection.euid, @(geteuid())];
+            [[NSNotificationCenter defaultCenter] postNotificationName:iTermAPIServerConnectionRejected
+                                                                object:nil
+                                                              userInfo:@{ @"reason": reason }];
+            [connection unauthorized];
+        });
+        return;
+    }
+
+    [self startRequestOnConnection:connection pids:@[] completion:^(BOOL ok, NSString *reason) {
+        [self->_pendingConnections removeObject:connection];
+        if (!ok) {
+            XLog(@"Reject unix domain socket connection: %@", reason);
             dispatch_async(connection.queue, ^{
                 [connection unauthorized];
             });
-            return;
         }
-
-        [self startRequestOnConnection:connection pids:pids completion:^(BOOL ok, NSString *reason) {
-            [self->_pendingConnections removeObject:connection];
-            if (!ok) {
-                XLog(@"Reject unauthenticated process (pids %@): %@", pids, reason);
-                dispatch_async(connection.queue, ^{
-                    [connection unauthorized];
-                });
-            }
-        }];
     }];
 }
 
 // _queue
 - (void)startRequestOnConnection:(iTermHTTPConnection *)connection pids:(NSArray<NSNumber *> *)pids completion:(void (^)(BOOL, NSString *))completion {
+    DLog(@"startRequest for pids %@", pids);
     dispatch_async(connection.queue, ^{
         NSURLRequest *request = [connection readRequest];
         dispatch_async(self->_queue, ^{
@@ -319,8 +336,12 @@ NSString *const iTermAPIServerConnectionClosed = @"iTermAPIServerConnectionClose
         dispatch_async(dispatch_get_main_queue(), ^{
             NSString *reason = nil;
             NSString *displayName = nil;
+            const BOOL disableAuthUI = request.allHTTPHeaderFields[@"x-iterm2-disable-auth-ui"] != nil;
+            assert(connection.clientAddress.addressFamily == AF_UNIX);
             const BOOL ok = [self.delegate apiServerAuthorizeProcesses:pids
                                                          preauthorized:webSocketConnection.preauthorized
+                                                         disableAuthUI:disableAuthUI
+                                                          advisoryName:webSocketConnection.advisoryName
                                                                 reason:&reason
                                                            displayName:&displayName];
             if (ok) {
@@ -343,11 +364,19 @@ NSString *const iTermAPIServerConnectionClosed = @"iTermAPIServerConnectionClose
                     }];
                 });
             } else {
-                [[NSNotificationCenter defaultCenter] postNotificationName:iTermAPIServerConnectionRejected
-                                                                    object:request.allHTTPHeaderFields[@"x-iterm2-key"]
-                                                                  userInfo:@{ @"reason": reason ?: @"Unknown reason",
-                                                                              @"job": displayName ?: [NSNull null],
-                                                                              @"pids": pids }];
+                if (disableAuthUI) {
+                    [[NSNotificationCenter defaultCenter] postNotificationName:iTermAPIServerConnectionRejected
+                                                                        object:nil
+                                                                      userInfo:@{ @"reason": reason ?: @"Unknown reason",
+                                                                                  @"job": displayName ?: [NSNull null],
+                                                                                  @"pids": pids }];
+                } else {
+                    [[NSNotificationCenter defaultCenter] postNotificationName:iTermAPIServerConnectionRejected
+                                                                        object:request.allHTTPHeaderFields[@"x-iterm2-key"]
+                                                                      userInfo:@{ @"reason": reason ?: @"Unknown reason",
+                                                                                  @"job": displayName ?: [NSNull null],
+                                                                                  @"pids": pids }];
+                }
                 dispatch_async(connection.queue, ^{
                     [connection unauthorized];
                 });
@@ -390,7 +419,7 @@ NSString *const iTermAPIServerConnectionClosed = @"iTermAPIServerConnectionClose
 // Runs on execution queue
 - (void)dispatchRequestWhileNotInTransaction:(ITMClientOriginatedMessage *)request
                                   connection:(iTermWebSocketConnection *)webSocketConnection {
-    ITUpgradedNSAssert(!self.transaction, @"Already in a transaction");
+    ITAssertWithMessage(!self.transaction, @"Already in a transaction");
 
     __weak __typeof(self) weakSelf = self;
     if (request.submessageOneOfCase == ITMClientOriginatedMessage_Submessage_OneOfCase_TransactionRequest) {
@@ -521,6 +550,19 @@ NSString *const iTermAPIServerConnectionClosed = @"iTermAPIServerConnectionClose
     }];
 }
 
+- (void)handleListPromptsRequest:(ITMClientOriginatedMessage *)request connection:(iTermWebSocketConnection *)webSocketConnection {
+    ITMServerOriginatedMessage *response = [self newResponseForRequest:request];
+
+    __block BOOL handled = NO;
+    __weak __typeof(self) weakSelf = self;
+    [_delegate apiServerListPrompts:request.listPromptsRequest handler:^(ITMListPromptsResponse *listPromptsResponse) {
+        assert(!handled);
+        handled = YES;
+        response.listPromptsResponse = listPromptsResponse;
+        [weakSelf finishHandlingRequestWithResponse:response onConnection:webSocketConnection];
+    }];
+}
+
 - (void)handleNotificationRequest:(ITMClientOriginatedMessage *)request connection:(iTermWebSocketConnection *)webSocketConnection {
     ITMServerOriginatedMessage *response = [self newResponseForRequest:request];
 
@@ -980,6 +1022,10 @@ NSString *const iTermAPIServerConnectionClosed = @"iTermAPIServerConnectionClose
             [self handleGetPromptRequest:request connection:webSocketConnection];
             break;
 
+        case ITMClientOriginatedMessage_Submessage_OneOfCase_ListPromptsRequest:
+            [self handleListPromptsRequest:request connection:webSocketConnection];
+            break;
+
         case ITMClientOriginatedMessage_Submessage_OneOfCase_NotificationRequest:
             [self handleNotificationRequest:request connection:webSocketConnection];
             break;
