@@ -63,6 +63,11 @@ NSString *const iTermImageDidLoad = @"iTermImageDidLoad";
     return self;
 }
 
+- (NSString *)description {
+    return [NSString stringWithFormat:@"<%@: %p code=%@ size=%@ uniqueIdentifier=%@ filename=%@ broken=%@>",
+            self.class, self, @(self.code), NSStringFromSize(self.size), self.uniqueIdentifier, self.filename, @(self.broken)];
+}
+
 - (NSString *)uniqueIdentifier {
     if (!_uniqueIdentifier) {
         _uniqueIdentifier = [[[NSUUID UUID] UUIDString] copy];
@@ -92,23 +97,27 @@ NSString *const iTermImageDidLoad = @"iTermImageDidLoad";
         }
         return;
     }
-    
+
     [_dictionary release];
     _dictionary = nil;
-    
+
     DLog(@"Queueing load of %@", self.uniqueIdentifier);
     void (^block)(void) = ^{
         // This is a slow operation that blocks for a long time.
-        iTermImage *image = [iTermImage imageWithCompressedData:_data];
+        iTermImage *image = [[iTermImage imageWithCompressedData:_data] retain];
         dispatch_sync(dispatch_get_main_queue(), ^{
+            [image autorelease];
             [_queuedBlock release];
             _queuedBlock = nil;
+            [_animatedImage autorelease];
             _animatedImage = [[iTermAnimatedImageInfo alloc] initWithImage:image];
             if (!_animatedImage) {
                 _image = [image retain];
             }
-            DLog(@"Loaded %@", self.uniqueIdentifier);
-            [[NSNotificationCenter defaultCenter] postNotificationName:iTermImageDidLoad object:self];
+            if (_image || _animatedImage) {
+                DLog(@"Loaded %@", self.uniqueIdentifier);
+                [[NSNotificationCenter defaultCenter] postNotificationName:iTermImageDidLoad object:self];
+            }
         });
     };
     _queuedBlock = [block copy];
@@ -139,28 +148,28 @@ NSString *const iTermImageDidLoad = @"iTermImageDidLoad";
 }
 
 - (void)saveToFile:(NSString *)filename {
-    NSBitmapImageFileType fileType = NSPNGFileType;
+    NSBitmapImageFileType fileType = NSBitmapImageFileTypePNG;
     if ([filename hasSuffix:@".bmp"]) {
-        fileType = NSBMPFileType;
+        fileType = NSBitmapImageFileTypeBMP;
     } else if ([filename hasSuffix:@".gif"]) {
-        fileType = NSGIFFileType;
+        fileType = NSBitmapImageFileTypeGIF;
     } else if ([filename hasSuffix:@".jp2"]) {
-        fileType = NSJPEG2000FileType;
+        fileType = NSBitmapImageFileTypeJPEG2000;
     } else if ([filename hasSuffix:@".jpg"] || [filename hasSuffix:@".jpeg"]) {
-        fileType = NSJPEGFileType;
+        fileType = NSBitmapImageFileTypeJPEG;
     } else if ([filename hasSuffix:@".png"]) {
-        fileType = NSPNGFileType;
+        fileType = NSBitmapImageFileTypePNG;
     } else if ([filename hasSuffix:@".tiff"]) {
-        fileType = NSTIFFFileType;
+        fileType = NSBitmapImageFileTypeTIFF;
     }
 
     NSData *data = nil;
-    NSDictionary *universalTypeToCocoaMap = @{ (NSString *)kUTTypeBMP: @(NSBMPFileType),
-                                               (NSString *)kUTTypeGIF: @(NSGIFFileType),
-                                               (NSString *)kUTTypeJPEG2000: @(NSJPEG2000FileType),
-                                               (NSString *)kUTTypeJPEG: @(NSJPEGFileType),
-                                               (NSString *)kUTTypePNG: @(NSPNGFileType),
-                                               (NSString *)kUTTypeTIFF: @(NSTIFFFileType) };
+    NSDictionary *universalTypeToCocoaMap = @{ (NSString *)kUTTypeBMP: @(NSBitmapImageFileTypeBMP),
+                                               (NSString *)kUTTypeGIF: @(NSBitmapImageFileTypeGIF),
+                                               (NSString *)kUTTypeJPEG2000: @(NSBitmapImageFileTypeJPEG2000),
+                                               (NSString *)kUTTypeJPEG: @(NSBitmapImageFileTypeJPEG),
+                                               (NSString *)kUTTypePNG: @(NSBitmapImageFileTypePNG),
+                                               (NSString *)kUTTypeTIFF: @(NSBitmapImageFileTypeTIFF) };
     NSString *imageType = self.imageType;
     if (self.broken) {
         data = self.data;
@@ -242,8 +251,22 @@ NSString *const iTermImageDidLoad = @"iTermImageDidLoad";
     return (self.image || self.animatedImage);
 
 }
+static NSSize iTermImageInfoGetSizeForRegionPreservingAspectRatio(const NSSize region,
+                                                                  NSSize imageSize) {
+    double imageAR = imageSize.width / imageSize.height;
+    double canvasAR = region.width / region.height;
+    if (imageAR > canvasAR) {
+        // Image is wider than canvas, add letterboxes on top and bottom.
+        return NSMakeSize(region.width, region.width / imageAR);
+    } else {
+        // Image is taller than canvas, add pillarboxes on sides.
+        return NSMakeSize(region.height * imageAR, region.height);
+    }
+}
+
 - (NSImage *)imageWithCellSize:(CGSize)cellSize timestamp:(NSTimeInterval)timestamp {
     if (!self.ready) {
+        DLog(@"%@ not ready", self.uniqueIdentifier);
         return nil;
     }
     if (!_embeddedImages) {
@@ -263,18 +286,10 @@ NSString *const iTermImageDidLoad = @"iTermImageDidLoad";
         } else {
             theImage = [self.image.images firstObject];
         }
-        if (!_preserveAspectRatio) {
-            size = region;
+        if (_preserveAspectRatio) {
+            size = iTermImageInfoGetSizeForRegionPreservingAspectRatio(region, theImage.size);
         } else {
-            double imageAR = theImage.size.width / theImage.size.height;
-            double canvasAR = region.width / region.height;
-            if (imageAR > canvasAR) {
-                // image is wider than canvas, add black bars on top and bottom
-                size = NSMakeSize(region.width, region.width / imageAR);
-            } else {
-                // image is taller than canvas, add black bars on sides
-                size = NSMakeSize(region.height * imageAR, region.height);
-            }
+            size = region;
         }
         [canvas setSize:region];
         [canvas lockFocus];
@@ -283,10 +298,11 @@ NSString *const iTermImageDidLoad = @"iTermImageDidLoad";
         inset.bottom *= cellSize.height;
         inset.left *= cellSize.width;
         inset.right *= cellSize.width;
-        [theImage drawInRect:NSMakeRect((region.width - size.width) / 2 + inset.left,
-                                        (region.height - size.height) / 2 + inset.bottom,
-                                        MAX(0, size.width - inset.left - inset.right),
-                                        MAX(0, size.height - inset.top - inset.bottom))];
+        const NSRect destinationRect = NSMakeRect((region.width - size.width) / 2 + inset.left,
+                                                  (region.height - size.height) / 2 + inset.bottom,
+                                                  MAX(0, size.width - inset.left - inset.right),
+                                                  MAX(0, size.height - inset.top - inset.bottom));
+        [theImage drawInRect:destinationRect];
         [canvas unlockFocus];
 
         self.embeddedImages[@(frame)] = canvas;
@@ -294,6 +310,36 @@ NSString *const iTermImageDidLoad = @"iTermImageDidLoad";
     return _embeddedImages[@(frame)];
 }
 
++ (NSEdgeInsets)fractionalInsetsForPreservedAspectRatioWithDesiredSize:(NSSize)desiredSize
+                                                          forImageSize:(NSSize)imageSize
+                                                              cellSize:(NSSize)cellSize
+                                                         numberOfCells:(NSSize)numberOfCells {
+    const NSSize region = NSMakeSize(cellSize.width * numberOfCells.width,
+                                     cellSize.height * numberOfCells.height);
+    const NSSize size = iTermImageInfoGetSizeForRegionPreservingAspectRatio(region, imageSize);
+
+    // Given the following equalities, inferred from how the destinationRect is computed when
+    // creating the embedded image in -imageWithCellSize:timestamp: above:
+    //
+    // left = (region.width - size.width) / 2 + insets.left
+    // bottom = (region.height - size.height) / 2 + insets.bottom
+    // width = size.width - insets.left - insets.right
+    // height = size.height - insets.top - insets.bottom
+    //
+    // Set left=0, bottom=0, width=desiredSize.width, height=desiredSize.height.
+    // Solve for insets. Here's what you get:
+    const CGFloat left = (region.width - size.width) / -2.0;
+    const CGFloat bottom = (region.height - size.height) / -2.0;
+    const CGFloat right = size.width - left - desiredSize.width;
+    const CGFloat top = size.height - bottom - desiredSize.height;
+
+    // To top-align, swap the bottom and top insets.
+    return NSEdgeInsetsMake(bottom / cellSize.height,
+                            left / cellSize.width,
+                            top / cellSize.height,
+                            right / cellSize.width);
+}
+
 - (NSString *)nameForNewSavedTempFile {
     NSString *name = nil;
     if (_filename.pathExtension.length) {
@@ -303,7 +349,7 @@ NSString *const iTermImageDidLoad = @"iTermImageDidLoad";
         name = [[NSWorkspace sharedWorkspace] temporaryFileNameWithPrefix:@"iTerm2."
                                                                    suffix:suffix];
     } else {
-        // Empty extension case. Try to guess the exetnsion.
+        // Empty extension case. Try to guess the extension.
         NSString *extension = [NSImage extensionForUniformType:self.imageType];
         if (extension) {
             extension = [@"." stringByAppendingString:extension];
