@@ -13,6 +13,7 @@
 #import "iTermPasteHelper.h"
 #import "iTermPasteSpecialViewController.h"
 #import "iTermPasteSpecialWindowController.h"
+#import "iTermPasteViewManager.h"
 #import "iTermPreferences.h"
 #import "iTermWarning.h"
 #import "NSData+iTerm.h"
@@ -25,49 +26,55 @@
 
 const int kNumberOfSpacesPerTabCancel = -2;
 const int kNumberOfSpacesPerTabNoConversion = -1;
+const int kNumberOfSpacesPerTabOpenAdvancedPaste = -3;
 
-@interface iTermPasteHelper () <PasteViewControllerDelegate>
+// Was 1024, but this seemed to cause a lot of problems. Let's try this.
+const NSInteger iTermQuickPasteBytesPerCallDefaultValue = 768;
+
+@interface iTermPasteHelper () <iTermPasteViewManagerDelegate>
 @end
 
 @implementation iTermPasteHelper {
     NSMutableArray *_eventQueue;
-    PasteViewController *_pasteViewController;
     PasteContext *_pasteContext;
 
     // Paste from the head of this string from a timer until it's empty.
     NSMutableString *_buffer;
     NSTimer *_timer;
-
+    iTermPasteViewManager *_pasteViewManager;
 }
 
 + (NSMutableCharacterSet *)unsafeControlCodeSet {
-    NSMutableCharacterSet *controlSet = [[[NSMutableCharacterSet alloc] init] autorelease];
+    NSMutableCharacterSet *controlSet = [[NSMutableCharacterSet alloc] init];
     [controlSet addCharactersInRange:NSMakeRange(0, 32)];
     [controlSet removeCharactersInRange:NSMakeRange(9, 2)];  // Tab and line feed
     [controlSet removeCharactersInRange:NSMakeRange(12, 2)];  // Form feed and carriage return
     return controlSet;
 }
 
++ (BOOL)promptToConvertTabsToSpacesWhenPasting {
+    return ![iTermWarning identifierIsSilenced:@"AboutToPasteTabsWithCancel"];
+}
+
++ (void)togglePromptToConvertTabsToSpacesWhenPasting {
+    if (![self promptToConvertTabsToSpacesWhenPasting]) {
+        [iTermWarning unsilenceIdentifier:@"AboutToPasteTabsWithCancel"];
+    } else {
+        [iTermWarning setIdentifier:@"AboutToPasteTabsWithCancel" permanentSelection:kiTermWarningSelection0];
+    }
+}
+
 - (instancetype)init {
     self = [super init];
     if (self) {
         _eventQueue = [[NSMutableArray alloc] init];
         _buffer = [[NSMutableString alloc] init];
+        _pasteViewManager = [[iTermPasteViewManager alloc] init];
+        _pasteViewManager.delegate = self;
     }
     return self;
 }
 
-- (void)dealloc {
-    [_eventQueue release];
-    [_pasteViewController release];
-    [_pasteContext release];
-    [_buffer release];
-    if (_timer) {
-        [_timer invalidate];
-    }
-    [super dealloc];
-}
-
 - (void)showPasteOptionsInWindow:(NSWindow *)window bracketingEnabled:(BOOL)bracketingEnabled {
     int chunkSize = [iTermPreferences intForKey:kPreferenceKeyPasteSpecialChunkSize];
     NSTimeInterval chunkDelay = [iTermPreferences floatForKey:kPreferenceKeyPasteSpecialChunkDelay];
@@ -78,7 +85,9 @@ const int kNumberOfSpacesPerTabNoConversion = -1;
                                                   encoding:[_delegate pasteHelperEncoding]
                                           canWaitForPrompt:[_delegate pasteHelperCanWaitForPrompt]
                                            isAtShellPrompt:![_delegate pasteHelperShouldWaitForPrompt]
+                                        forceEscapeSymbols:NO
                                                 completion:^(PasteEvent *event) {
+                                                    event.suppressMultilinePasteWarning = YES;
                                                     [self tryToPasteEvent:event];
                                                     [iTermPreferences setInt:event.defaultChunkSize
                                                                       forKey:kPreferenceKeyPasteSpecialChunkSize];
@@ -88,14 +97,23 @@ const int kNumberOfSpacesPerTabNoConversion = -1;
 }
 
 - (void)abort {
+    [self abortAndCancelPendingEvents:YES];
+}
+
+- (void)abortAndCancelPendingEvents:(BOOL)cancel {
     if (_timer) {
         [_timer invalidate];
         _timer = nil;
-        [_eventQueue removeAllObjects];
+        if (cancel) {
+            [_eventQueue removeAllObjects];
+        }
     }
-    [_buffer release];
     _buffer = [[NSMutableString alloc] init];
     [self hidePasteIndicator];
+    _pasteContext = nil;
+    if (!cancel) {
+        [self dequeueEvents];
+    }
 }
 
 - (BOOL)isPasting {
@@ -155,7 +173,7 @@ const int kNumberOfSpacesPerTabNoConversion = -1;
     }
 
     if (flags & kPasteFlagsEscapeSpecialCharacters) {
-        // Put backslash before anything the shell might interpret.
+        // Escape/quote anything the shell might interpret..
         if (pasteEvent.tabTransform != kTabTransformEscapeWithCtrlV) {
             theString = [theString stringWithEscapedShellCharactersIncludingNewlines:NO];
         } else {
@@ -181,7 +199,7 @@ const int kNumberOfSpacesPerTabNoConversion = -1;
         theString = [temp stringWithBase64EncodingWithLineBreak:@"\r"];
     }
 
-    pasteEvent.string = theString;
+    [pasteEvent setModifiedString:theString];
 }
 
 - (void)pasteString:(NSString *)theString stringConfig:(NSString *)jsonConfig {
@@ -199,13 +217,13 @@ const int kNumberOfSpacesPerTabNoConversion = -1;
     [self pasteString:theString slowly:slowly escapeShellChars:escapeShellChars isUpload:isUpload tabTransform:tabTransform spacesPerTab:spacesPerTab progress:nil];
 }
 
-- (void)pasteString:(NSString *)theString
-             slowly:(BOOL)slowly
-   escapeShellChars:(BOOL)escapeShellChars
-           isUpload:(BOOL)isUpload
-       tabTransform:(iTermTabTransformTags)tabTransform
-       spacesPerTab:(int)spacesPerTab
-           progress:(void (^)(NSInteger))progress {
+- (PasteEvent *)pasteEventWithString:(NSString *)theString
+                              slowly:(BOOL)slowly
+                    escapeShellChars:(BOOL)escapeShellChars
+                            isUpload:(BOOL)isUpload
+                        tabTransform:(iTermTabTransformTags)tabTransform
+                        spacesPerTab:(int)spacesPerTab
+                            progress:(void (^)(NSInteger))progress {
     NSUInteger bracketFlag = [_delegate pasteHelperShouldBracket] ? kPasteFlagsBracket : 0;
     NSUInteger flags = (kPasteFlagsSanitizingNewlines |
                         kPasteFlagsRemovingUnsafeControlCodes |
@@ -223,7 +241,7 @@ const int kNumberOfSpacesPerTabNoConversion = -1;
         chunkKey = @"SlowPasteBytesPerCall";
         delayKey = @"SlowPasteDelayBetweenCalls";
     } else {
-        defaultChunkSize = 1024;
+        defaultChunkSize = iTermQuickPasteBytesPerCallDefaultValue;
         defaultDelay = 0.01;
         chunkKey = @"QuickPasteBytesPerCall";
         delayKey = @"QuickPasteDelayBetweenCalls";
@@ -238,8 +256,26 @@ const int kNumberOfSpacesPerTabNoConversion = -1;
                                             spacesPerTab:spacesPerTab
                                                    regex:nil
                                             substitution:nil];
+    event.slow = slowly;
     event.progress = progress;
     event.isUpload = isUpload;
+    return event;
+}
+
+- (void)pasteString:(NSString *)theString
+             slowly:(BOOL)slowly
+   escapeShellChars:(BOOL)escapeShellChars
+           isUpload:(BOOL)isUpload
+       tabTransform:(iTermTabTransformTags)tabTransform
+       spacesPerTab:(int)spacesPerTab
+           progress:(void (^)(NSInteger))progress {
+    PasteEvent *event = [self pasteEventWithString:theString
+                                            slowly:slowly
+                                  escapeShellChars:escapeShellChars
+                                          isUpload:isUpload
+                                      tabTransform:tabTransform
+                                      spacesPerTab:spacesPerTab
+                                          progress:progress];
     [self tryToPasteEvent:event];
 }
 
@@ -248,11 +284,11 @@ const int kNumberOfSpacesPerTabNoConversion = -1;
     DLog(@"-[iTermPasteHelper pasteString:flags:");
     DLog(@"length=%@, flags=%@", @(pasteEvent.string.length), @(pasteEvent.flags));
     if ([pasteEvent.string length] == 0) {
-        DLog(@"Tried to paste 0-byte string. Beep.");
+        DLog(@"Beep: Tried to paste 0-byte string. Beep.");
         NSBeep();
         return;
     }
-    if (!(pasteEvent.flags & kPasteFlagsCommands)) {
+    if (!(pasteEvent.flags & (kPasteFlagsCommands | kPasteFlagsDisableWarnings))) {
         if (![self maybeWarnAboutMultiLinePaste:pasteEvent]) {
             DLog(@"Multiline paste declined.");
             return;
@@ -285,41 +321,35 @@ const int kNumberOfSpacesPerTabNoConversion = -1;
 
     if (pasteEvent.flags & kPasteFlagsBracket) {
         DLog(@"Bracketing string to paste.");
-        NSString *startBracket = [NSString stringWithFormat:@"%c[200~", 27];
-        NSString *endBracket = [NSString stringWithFormat:@"%c[201~", 27];
-        NSArray *components = @[ startBracket, pasteEvent.string, endBracket ];
-        pasteEvent.string = [components componentsJoinedByString:@""];
+        [pasteEvent addPasteBracketing];
     }
 
     DLog(@"String to paste now has length %@", @(pasteEvent.string.length));
     if ([pasteEvent.string length] == 0) {
-        DLog(@"Tried to paste 0-byte string (became 0 length after removing controls). Beep.");
+        DLog(@"Beep: Tried to paste 0-byte string (became 0 length after removing controls). Beep.");
         NSBeep();
         return;
     }
 
-    [_buffer appendString:pasteEvent.string];
-    [self pasteWithBytePerCallPrefKey:pasteEvent.chunkKey
-                         defaultValue:pasteEvent.defaultChunkSize
-             delayBetweenCallsPrefKey:pasteEvent.delayKey
-                         defaultValue:pasteEvent.defaultDelay
-                       blockAtNewline:!!(pasteEvent.flags & kPasteFlagsCommands)
-                             isUpload:pasteEvent.isUpload
-                             progress:pasteEvent.progress];
+    [self pasteLiteralEventUnconditionallyImmediately:pasteEvent];
 }
 
 // Outputs 16 bytes every 125ms so that clients that don't buffer input can handle pasting large buffers.
 // Override the constants by setting defaults SlowPasteBytesPerCall and SlowPasteDelayBetweenCalls
 - (void)pasteSlowly:(NSString *)theString {
     DLog(@"pasteSlowly length=%@", @(theString.length));
-    [_buffer appendString:theString];
-    [self pasteWithBytePerCallPrefKey:@"SlowPasteBytesPerCall"
-                         defaultValue:16
-             delayBetweenCallsPrefKey:@"SlowPasteDelayBetweenCalls"
-                         defaultValue:0.125
-                       blockAtNewline:NO
-                             isUpload:NO
-                             progress:nil];
+    PasteEvent *pasteEvent = [PasteEvent pasteEventWithString:theString
+                                                        flags:0
+                                             defaultChunkSize:16
+                                                     chunkKey:@"SlowPasteBytesPerCall"
+                                                 defaultDelay:0.125
+                                                     delayKey:@"SlowPasteDelayBetweenCalls"
+                                                tabTransform:kTabTransformNone
+                                                 spacesPerTab:4
+                                                        regex:nil
+                                                 substitution:nil];
+
+    [self pasteLiteralEventUnconditionallyImmediately:pasteEvent];
 }
 
 - (void)pasteNormally:(NSString *)aString {
@@ -327,20 +357,24 @@ const int kNumberOfSpacesPerTabNoConversion = -1;
     // This is the "normal" way of pasting. It's fast but tends not to
     // outrun a shell's ability to read from its buffer. Why this crazy
     // thing? See bug 1031.
-    [_buffer appendString:aString];
-    [self pasteWithBytePerCallPrefKey:@"QuickPasteBytesPerCall"
-                         defaultValue:1024
-             delayBetweenCallsPrefKey:@"QuickPasteDelayBetweenCalls"
-                         defaultValue:0.01
-                       blockAtNewline:NO
-                             isUpload:NO
-                             progress:nil];
+    PasteEvent *pasteEvent = [PasteEvent pasteEventWithString:aString
+                                                        flags:0
+                                             defaultChunkSize:iTermQuickPasteBytesPerCallDefaultValue
+                                                     chunkKey:@"QuickPasteBytesPerCall"
+                                                 defaultDelay:0.01
+                                                     delayKey:@"QuickPasteDelayBetweenCalls"
+                                                tabTransform:kTabTransformNone
+                                                 spacesPerTab:4
+                                                        regex:nil
+                                                 substitution:nil];
+
+    [self pasteLiteralEventUnconditionallyImmediately:pasteEvent];
 }
 
 - (NSInteger)normalChunkSize {
     NSNumber *n = [[NSUserDefaults standardUserDefaults] objectForKey:@"QuickPasteBytesPerCall"];
     if (!n) {
-        return 1024;
+        return iTermQuickPasteBytesPerCallDefaultValue;
     } else {
         return [n integerValue];
     }
@@ -358,7 +392,7 @@ const int kNumberOfSpacesPerTabNoConversion = -1;
 - (void)dequeueEvents {
     DLog(@"Dequeueing paste events...");
     while (_eventQueue.count) {
-        NSEvent *event = [[[_eventQueue firstObject] retain] autorelease];
+        NSEvent *event = [_eventQueue firstObject];
         [_eventQueue removeObjectAtIndex:0];
         if ([event isKindOfClass:[PasteEvent class]]) {
             DLog(@"Found a queued paste event");
@@ -379,31 +413,20 @@ const int kNumberOfSpacesPerTabNoConversion = -1;
     [_eventQueue addObject:event];
 }
 
-- (void)showPasteIndicatorInView:(NSView *)view {
-    _pasteViewController = [[PasteViewController alloc] initWithContext:_pasteContext
-                                                                 length:_buffer.length];
-    _pasteViewController.delegate = self;
-    _pasteViewController.view.frame = NSMakeRect(20,
-                                                 view.frame.size.height - _pasteViewController.view.frame.size.height,
-                                                 _pasteViewController.view.frame.size.width,
-                                                 _pasteViewController.view.frame.size.height);
-    [view addSubview:_pasteViewController.view];
-    [_pasteViewController updateFrame];
-    _pasteViewIsVisible = YES;
-    [self.delegate pasteHelperPasteViewVisibilityDidChange];
+- (void)showPasteIndicatorInView:(NSView *)view
+         statusBarViewController:(iTermStatusBarViewController *)statusBarViewController {
+    _pasteViewManager.pasteContext = _pasteContext;
+    _pasteViewManager.bufferLength = _buffer.length;
+    [_pasteViewManager startWithViewForDropdown:view
+                        statusBarViewController:statusBarViewController];
 }
 
 - (void)hidePasteIndicator {
-    [_pasteViewController closeWithCompletion:^{
-        [self.delegate pasteHelperPasteViewVisibilityDidChange];
-    }];
-    [_pasteViewController release];
-    _pasteViewController = nil;
-    _pasteViewIsVisible = NO;
+    [_pasteViewManager didStop];
 }
 
 - (void)updatePasteIndicator {
-    [_pasteViewController setRemainingLength:_buffer.length];
+    [_pasteViewManager setRemainingLength:_buffer.length];
 }
 
 - (void)pasteNextChunkAndScheduleTimer {
@@ -448,7 +471,6 @@ const int kNumberOfSpacesPerTabNoConversion = -1;
         DLog(@"Done pasting");
         _timer = nil;
         [self hidePasteIndicator];
-        [_pasteContext release];
         _pasteContext = nil;
         [self dequeueEvents];
     }
@@ -474,30 +496,24 @@ const int kNumberOfSpacesPerTabNoConversion = -1;
     }
 }
 
-- (void)pasteWithBytePerCallPrefKey:(NSString*)bytesPerCallKey
-                       defaultValue:(int)bytesPerCallDefault
-           delayBetweenCallsPrefKey:(NSString*)delayBetweenCallsKey
-                       defaultValue:(float)delayBetweenCallsDefault
-                     blockAtNewline:(BOOL)blockAtNewline
-                           isUpload:(BOOL)isUpload
-                           progress:(void (^)(NSInteger))progress {
-    [_pasteContext release];
-    _pasteContext = [[PasteContext alloc] initWithBytesPerCallPrefKey:bytesPerCallKey
-                                                         defaultValue:bytesPerCallDefault
-                                             delayBetweenCallsPrefKey:delayBetweenCallsKey
-                                                         defaultValue:delayBetweenCallsDefault];
-    _pasteContext.blockAtNewline = blockAtNewline;
-    _pasteContext.isUpload = isUpload;
-    _pasteContext.progress = progress;
+- (void)temporaryRightStatusBarComponentDidBecomeAvailable {
+    [_pasteViewManager temporaryRightStatusBarComponentDidBecomeAvailable];
+}
+
+- (void)pasteLiteralEventUnconditionallyImmediately:(PasteEvent *)pasteEvent {
+    [_buffer appendString:pasteEvent.string];
+
+    _pasteContext = [[PasteContext alloc] initWithPasteEvent:pasteEvent];
     const int kPasteBytesPerSecond = 10000;  // This is a wild-ass guess.
     const NSTimeInterval sumOfDelays =
         _pasteContext.delayBetweenCalls * _buffer.length / _pasteContext.bytesPerCall;
     const NSTimeInterval timeSpentWriting = _buffer.length / kPasteBytesPerSecond;
     const NSTimeInterval kMinEstimatedPasteTimeToShowIndicator = 3;
-    if (!isUpload) {
+    if (!pasteEvent.isUpload) {
         if ((sumOfDelays + timeSpentWriting > kMinEstimatedPasteTimeToShowIndicator) ||
-            blockAtNewline) {
-            [self showPasteIndicatorInView:[_delegate pasteHelperViewForIndicator]];
+            _pasteContext.blockAtNewline) {
+            [self showPasteIndicatorInView:[_delegate pasteHelperViewForIndicator]
+                   statusBarViewController:[_delegate pasteHelperStatusBarViewController]];
         }
     }
 
@@ -510,8 +526,42 @@ const int kNumberOfSpacesPerTabNoConversion = -1;
     [self pasteNextChunkAndScheduleTimer];
 }
 
+- (BOOL)isWaitingForPrompt {
+    return _pasteContext.isBlocked;
+}
+
 // This may modify pasteEvent.string.
 - (BOOL)maybeWarnAboutMultiLinePaste:(PasteEvent *)pasteEvent {
+    const int limit = [iTermAdvancedSettingsModel alwaysWarnBeforePastingOverSize];
+    if (limit >= 0) {
+        if (pasteEvent.string.length > limit) {
+            NSNumberFormatter *numberFormatter = [[NSNumberFormatter alloc] init];
+            numberFormatter.numberStyle = NSNumberFormatterDecimalStyle;
+            const iTermWarningSelection selection =
+            [iTermWarning showWarningWithTitle:[NSString stringWithFormat:@"OK to paste %@ characters?", [numberFormatter stringFromNumber:@(pasteEvent.string.length)]]
+                                       actions:@[ @"OK", @"Cancel", @"Advanced…" ]
+                                     accessory:nil
+                                    identifier:@"NoSyncPasteOverCharacterLimitWarning"
+                                   silenceable:kiTermWarningTypePersistent
+                                       heading:@"Paste Limit Exceeded"
+                                        window:self.delegate.pasteHelperViewForIndicator.window];
+            switch (selection) {
+                case kiTermWarningSelection0:
+                    break;
+                case kiTermWarningSelection1:
+                    return NO;
+                case kiTermWarningSelection2:
+                    [self showAdvancedPasteWithFlags:0];
+                    return NO;
+                default:
+                    break;
+            }
+        }
+    }
+    if (pasteEvent.flags & kPasteFlagsRemovingNewlines) {
+        return YES;
+    }
+
     NSCharacterSet *newlineCharacterSet =
         [NSCharacterSet characterSetWithCharactersInString:@"\r\n"];
     NSRange rangeOfFirstNewline = [pasteEvent.string rangeOfCharacterFromSet:newlineCharacterSet];
@@ -527,10 +577,13 @@ const int kNumberOfSpacesPerTabNoConversion = -1;
         !atShellPrompt) {
         return YES;
     }
+    if (pasteEvent.suppressMultilinePasteWarning) {
+        return YES;
+    }
     NSArray *lines = [pasteEvent.string componentsSeparatedByRegex:@"(?:\r\n)|(?:\r)|(?:\n)"];
     NSString *theTitle;
     NSMutableArray<iTermWarningAction *> *actions = [NSMutableArray array];
-    
+
     __block BOOL result = YES;
     iTermWarningAction *cancel = [iTermWarningAction warningActionWithLabel:@"Cancel"
                                                                       block:^(iTermWarningSelection selection) { result = NO; }];
@@ -539,12 +592,10 @@ const int kNumberOfSpacesPerTabNoConversion = -1;
     iTermWarningAction *pasteWithoutNewline =
         [iTermWarningAction warningActionWithLabel:@"Paste Without Newline"
                                              block:^(iTermWarningSelection selection) {
-                                                 NSCharacterSet *newlines = [NSCharacterSet newlineCharacterSet];
-                                                 pasteEvent.string =
-                                                     [pasteEvent.string stringByTrimmingTrailingCharactersFromCharacterSet:newlines];
-                                                 result = YES;
-                                             }];
-    
+            [pasteEvent trimNewlines];
+            result = YES;
+        }];
+
     [actions addObject:paste];
     [actions addObject:cancel];
     NSString *identifier = [iTermAdvancedSettingsModel noSyncDoNotWarnBeforeMultilinePasteUserDefaultsKey];
@@ -574,28 +625,61 @@ const int kNumberOfSpacesPerTabNoConversion = -1;
     }
     // Issue 5115
     [iTermWarning unsilenceIdentifier:identifier ifSelectionEquals:[actions indexOfObjectIdenticalTo:cancel]];
-    
-    iTermWarning *warning = [[[iTermWarning alloc] init] autorelease];
+    [actions addObject:[iTermWarningAction warningActionWithLabel:@"Advanced…" block:^(iTermWarningSelection selection) {
+        PTYSessionPasteFlags flags = 0;
+        if (pasteEvent.slow) {
+            flags |= kPTYSessionPasteSlowly;
+            // The other two flags do not appear to be used
+        }
+        [self showAdvancedPasteWithFlags:flags];
+        result = NO;
+    }]];
+    iTermWarning *warning = [[iTermWarning alloc] init];
     warning.title = theTitle;
     warning.warningActions = actions;
     warning.identifier = identifier;
     warning.warningType = kiTermWarningTypePermanentlySilenceable;
     warning.cancelLabel = @"Cancel";
+    warning.window = [[self.delegate pasteHelperViewForIndicator] window];
     [warning runModal];
     return result;
 }
 
+- (void)showAdvancedPasteWithFlags:(PTYSessionPasteFlags)flags {
+    PasteEvent *temporaryEvent = [self pasteEventWithString:@""
+                                                     slowly:!!(flags & kPTYSessionPasteSlowly)
+                                           escapeShellChars:!!(flags & kPTYSessionPasteEscapingSpecialCharacters)
+                                                   isUpload:NO
+                                               tabTransform:kTabTransformNone
+                                               spacesPerTab:4
+                                                   progress:nil];
+    PasteContext *temporaryContext = [[PasteContext alloc] initWithPasteEvent:temporaryEvent];
+    [iTermPasteSpecialWindowController showAsPanelInWindow:self.delegate.pasteHelperViewForIndicator.window
+                                                 chunkSize:temporaryContext.bytesPerCall
+                                        delayBetweenChunks:temporaryContext.delayBetweenCalls
+                                         bracketingEnabled:[self.delegate pasteHelperShouldBracket]
+                                                  encoding:[_delegate pasteHelperEncoding]
+                                          canWaitForPrompt:[_delegate pasteHelperCanWaitForPrompt]
+                                           isAtShellPrompt:![_delegate pasteHelperShouldWaitForPrompt]
+                                        forceEscapeSymbols:!!(flags & kPTYSessionPasteEscapingSpecialCharacters)
+                                                completion:^(PasteEvent *event) {
+                                                    event.suppressMultilinePasteWarning = YES;
+                                                    [self tryToPasteEvent:event];
+                                                }];
+}
+
 - (int)numberOfSpacesToConvertTabsTo:(NSString *)source {
     if ([source rangeOfString:@"\t"].location != NSNotFound) {
         iTermNumberOfSpacesAccessoryViewController *accessoryController =
-            [[[iTermNumberOfSpacesAccessoryViewController alloc] init] autorelease];
+            [[iTermNumberOfSpacesAccessoryViewController alloc] init];
 
         iTermWarningSelection selection =
             [iTermWarning showWarningWithTitle:@"You're about to paste a string with tabs."
-                                       actions:@[ @"Paste with tabs", @"Cancel", @"Convert tabs to spaces" ]
+                                       actions:@[ @"OK", @"Cancel", @"Convert tabs to spaces", @"Advanced…" ]
                                      accessory:accessoryController.view
                                     identifier:@"AboutToPasteTabsWithCancel"
-                                   silenceable:kiTermWarningTypePermanentlySilenceable];
+                                   silenceable:kiTermWarningTypePermanentlySilenceable
+                                        window:self.delegate.pasteHelperViewForIndicator.window];
         switch (selection) {
             case kiTermWarningSelection0:  // Paste with tabs
                 return kNumberOfSpacesPerTabNoConversion;
@@ -604,6 +688,8 @@ const int kNumberOfSpacesPerTabNoConversion = -1;
             case kiTermWarningSelection2:  // Convert to spaces
                 [accessoryController saveToUserDefaults];
                 return accessoryController.numberOfSpaces;
+            case kiTermWarningSelection3:  // Advanced
+                return kNumberOfSpacesPerTabOpenAdvancedPaste;
             default:
                 return kNumberOfSpacesPerTabNoConversion;
         }
@@ -611,17 +697,22 @@ const int kNumberOfSpacesPerTabNoConversion = -1;
     return kNumberOfSpacesPerTabNoConversion;
 }
 
-#pragma mark - PasteViewControllerDelegate
+- (BOOL)dropDownPasteViewIsVisible {
+    return _pasteViewManager.dropDownPasteViewIsVisible;
+}
 
-- (void)pasteViewControllerDidCancel {
-    [self hidePasteIndicator];
-    [_timer invalidate];
-    _timer = nil;
-    [_buffer release];
-    _buffer = [[NSMutableString alloc] init];
-    [_pasteContext release];
-    _pasteContext = nil;
-    [self dequeueEvents];
+#pragma mark - iTermPasteViewManagerDelegate
+
+- (void)pasteViewManagerDropDownPasteViewVisibilityDidChange {
+    [self.delegate pasteHelperPasteViewVisibilityDidChange];
+}
+
+- (void)pasteViewManagerUserDidCancel {
+    [self abortAndCancelPendingEvents:NO];
+}
+
+- (iTermVariableScope *)pasteViewManagerScope {
+    return [self.delegate pasteHelperScope];
 }
 
 #pragma mark - Testing
