@@ -188,7 +188,13 @@ NS_ASSUME_NONNULL_BEGIN
 }
 
 - (NSArray<iTermScriptItem *> *)scriptItems {
-    iTermScriptItem *root = [[iTermScriptItem alloc] initFolderWithPath:[[NSFileManager defaultManager] scriptsPath] parent:nil];
+    NSFileManager *fm = [NSFileManager defaultManager];
+    NSString *path = [fm scriptsPath];
+    if ([fm fileExistsAtPath:path]) {
+        [fm applicationSupportDirectoryWithoutSpaces];  // create link if needed
+        path = [fm scriptsPathWithoutSpaces];
+    }
+    iTermScriptItem *root = [[iTermScriptItem alloc] initFolderWithPath:path parent:nil];
     [self populateScriptItem:root];
     return root.children;
 }
@@ -713,7 +719,7 @@ NS_ASSUME_NONNULL_BEGIN
     NSPopUpButton *popUpButton = [[NSPopUpButton alloc] initWithFrame:NSMakeRect(10, 0, 50, 50)];
 
     NSArray<NSString *> *components = @[ @"iterm2env", @"versions" ];
-    NSString *path = [[NSFileManager defaultManager] applicationSupportDirectory];
+    NSString *path = [[NSFileManager defaultManager] applicationSupportDirectoryWithoutSpaces];
     for (NSString *part in components) {
         path = [path stringByAppendingPathComponent:part];
     }
@@ -925,7 +931,7 @@ NS_ASSUME_NONNULL_BEGIN
 }
 
 - (NSString *)folderForFullEnvironmentSavePanelURL:(NSURL *)url {
-    NSString *noSpacesScriptsRoot = [[NSFileManager defaultManager] scriptsPath];
+    NSString *noSpacesScriptsRoot = [[NSFileManager defaultManager] scriptsPathWithoutSpaces];
     NSString *scriptsRoot = [[[NSURL fileURLWithPath:noSpacesScriptsRoot] URLByResolvingSymlinksInPath] path];
     NSString *selectedPath = [url URLByResolvingSymlinksInPath].path;
     NSString *relative = [self relativePathFrom:scriptsRoot
@@ -946,10 +952,10 @@ NS_ASSUME_NONNULL_BEGIN
         NSString *name = url.path.lastPathComponent;
         // For a path like foo/bar this returns foo/bar/bar/bar.py
         // So the hierarchy looks like
-        // ~/Library/Application Support/iTerm2/Scripts/foo/bar/setup.cfg
-        // ~/Library/Application Support/iTerm2/Scripts/foo/bar/iterm2env
-        // ~/Library/Application Support/iTerm2/Scripts/foo/bar/bar/
-        // ~/Library/Application Support/iTerm2/Scripts/foo/bar/bar/bar.py
+        // ~/Library/ApplicationSupport/iTerm2/Scripts/foo/bar/setup.cfg
+        // ~/Library/ApplicationSupport/iTerm2/Scripts/foo/bar/iterm2env
+        // ~/Library/ApplicationSupport/iTerm2/Scripts/foo/bar/bar/
+        // ~/Library/ApplicationSupport/iTerm2/Scripts/foo/bar/bar/bar.py
         return [[folder stringByAppendingPathComponent:name] stringByAppendingPathComponent:[url.path.lastPathComponent stringByAppendingPathExtension:@"py"]];
     } else {
         return url.path;
