@@ -18,6 +18,7 @@
 #import "iTermRecentDirectoryMO+Additions.h"
 #import "NSArray+iTerm.h"
 #import "NSDictionary+iTerm.h"
+#import "NSStringITerm.h"
 #import "iTermCommandHistoryEntryMO.h"
 #import "PreferencePanel.h"
 #import "VT100RemoteHost.h"
@@ -49,13 +50,13 @@ static const NSTimeInterval kMaxTimeToRememberDirectories = 60 * 60 * 24 * 90;
 
 @implementation iTermShellHistoryController {
     NSMutableDictionary<NSString *, iTermHostRecordMO *> *_records;
-    
+
     // Keys are remote host keys, "user@hostname".
     NSMutableDictionary<NSString *, NSMutableArray<iTermCommandHistoryCommandUseMO *> *> *_expandedCache;
     NSManagedObjectContext *_managedObjectContext;
     iTermDirectoryTree *_tree;
 
-    // Prevents notifications from being posted durinig initialization since that causes deadlock in
+    // Prevents notifications from being posted during initialization since that causes deadlock in
     // -sharedInstance.
     BOOL _initializing;
 
@@ -100,7 +101,6 @@ static const NSTimeInterval kMaxTimeToRememberDirectories = 60 * 60 * 24 * 90;
     _expandedCache = [[NSMutableDictionary alloc] init];
     _tree = [[iTermDirectoryTree alloc] init];
 
-    [self migrateFromPlistToCoreData];
     [self removeOldData];
     [self loadObjectGraph];
 
@@ -164,7 +164,7 @@ static const NSTimeInterval kMaxTimeToRememberDirectories = 60 * 60 * 24 * 90;
 // vacuum a database after changing the setting to in-memory. It doesn't make sense to vacuum RAM,
 // after all.
 - (BOOL)initializeCoreDataWithRetry:(BOOL)retry vacuum:(BOOL)vacuum {
-    NSURL *modelURL = [[NSBundle mainBundle] URLForResource:@"Model" withExtension:@"momd"];
+    NSURL *modelURL = [[NSBundle bundleForClass:self.class] URLForResource:@"Model" withExtension:@"momd"];
     assert(modelURL);
 
     NSManagedObjectModel *managedObjectModel =
@@ -177,7 +177,7 @@ static const NSTimeInterval kMaxTimeToRememberDirectories = 60 * 60 * 24 * 90;
     NSPersistentStoreCoordinator *persistentStoreCoordinator =
         [[[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:managedObjectModel] autorelease];
     assert(persistentStoreCoordinator);
-    
+
     _managedObjectContext =
         [[NSManagedObjectContext alloc] initWithConcurrencyType:NSMainQueueConcurrencyType];
     assert(_managedObjectContext);
@@ -220,7 +220,7 @@ static const NSTimeInterval kMaxTimeToRememberDirectories = 60 * 60 * 24 * 90;
             NSLog(@"Giving up");
             return NO;
         }
-        
+
         NSLog(@"Deleting the presumably corrupt file and trying again");
         NSError *removeError = nil;
         [self deleteDatabase];
@@ -228,13 +228,13 @@ static const NSTimeInterval kMaxTimeToRememberDirectories = 60 * 60 * 24 * 90;
             NSLog(@"Failed to delete corrupt database: %@", removeError);
             return NO;
         }
-        
+
         NSLog(@"Trying again...");
         [_managedObjectContext release];
         _managedObjectContext = nil;
         return [self initializeCoreDataWithRetry:NO vacuum:vacuum];
     }
-    
+
     if (self.shouldSaveToDisk) {
         [self makeDatabaseReadableOnlyByUser];
     }
@@ -243,7 +243,12 @@ static const NSTimeInterval kMaxTimeToRememberDirectories = 60 * 60 * 24 * 90;
 
 - (void)makeDatabaseReadableOnlyByUser {
     NSString *basePath = [self pathForFileNamed:nil];
-    for (NSString *filename in [[NSFileManager defaultManager] enumeratorAtPath:basePath]) {
+    NSDirectoryEnumerator<NSURL *> *enumerator = [[NSFileManager defaultManager] enumeratorAtURL:[NSURL fileURLWithPath:basePath]
+                                                                      includingPropertiesForKeys:nil
+                                                                                         options:NSDirectoryEnumerationSkipsSubdirectoryDescendants
+                                                                                    errorHandler:nil];
+    for (NSURL *fileURL in enumerator) {
+        NSString *filename = fileURL.path;
         if (![filename hasPrefix:self.databaseFilenamePrefix]) {
             DLog(@"Skip setting permissions on file %@ that isn't part of the shell history db", filename);
             continue;
@@ -255,13 +260,13 @@ static const NSTimeInterval kMaxTimeToRememberDirectories = 60 * 60 * 24 * 90;
             XLog(@"Failed to get attributes of %@: %@", fullPath, error);
             continue;
         }
-        
+
         NSNumber *permissionsNumber = attributes[NSFilePosixPermissions];
         if (!permissionsNumber) {
             XLog(@"Couldn't get permissions of file %@. Attributes are: %@", fullPath, attributes);
             continue;
         }
-        
+
         short posixPermissions = [permissionsNumber shortValue];
         short fixedPosixPermissions = (posixPermissions & 0700);  // Remove other and group permissions.
         if (fixedPosixPermissions != posixPermissions) {
@@ -280,9 +285,11 @@ static const NSTimeInterval kMaxTimeToRememberDirectories = 60 * 60 * 24 * 90;
     NSString *path = [[self pathToDatabase] stringByDeletingLastPathComponent];
     NSDirectoryEnumerator<NSString *> *enumerator =
         [[NSFileManager defaultManager] enumeratorAtPath:path];
+    [enumerator skipDescendants];
     BOOL foundAny = NO;
     BOOL anyErrors = NO;
     for (NSString *filename in enumerator) {
+        [enumerator skipDescendants];
         if ([filename hasPrefix:[self databaseFilenamePrefix]]) {
             NSError *error = nil;
             [[NSFileManager defaultManager] removeItemAtPath:[path stringByAppendingPathComponent:filename]
@@ -349,99 +356,6 @@ static const NSTimeInterval kMaxTimeToRememberDirectories = 60 * 60 * 24 * 90;
     [self loadObjectGraph];
 }
 
-#pragma mark - Migration
-
-// Returns YES if a migration was attempted.
-- (BOOL)migrateFromPlistToCoreData {
-    BOOL migrateCommands = [self shouldMigrateCommands];
-    BOOL migrateDirectories = [self shouldMigrateDirectories];
-    if (migrateCommands || migrateDirectories) {
-        NSMutableDictionary *records = [NSMutableDictionary dictionary];
-        [self loadObjectGraphIntoDictionary:records];
-        if (migrateCommands) {
-            [self migrateCommandHistoryFromPlistToCoreData:records];
-        }
-        if (migrateDirectories) {
-            [self migrateDirectoriesFromPlistToCoreData:records];
-        }
-    }
-    return migrateCommands || migrateDirectories;
-}
-
-- (BOOL)shouldMigrateDirectories {
-    NSString *path = [self pathToDeprecatedDirectoriesPlist];
-    return [[NSFileManager defaultManager] fileExistsAtPath:path isDirectory:nil];
-}
-
-- (BOOL)shouldMigrateCommands {
-    NSString *path = [self pathToDeprecatedCommandHistoryPlist];
-    return [[NSFileManager defaultManager] fileExistsAtPath:path isDirectory:nil];
-}
-
-- (BOOL)migrateDirectoriesFromPlistToCoreData:(NSMutableDictionary *)records {
-    NSString *path = [self pathToDeprecatedDirectoriesPlist];
-    NSDictionary *archive = [NSKeyedUnarchiver unarchiveObjectWithFile:path];
-    for (NSString *host in archive) {
-        NSArray *parts = [host componentsSeparatedByString:@"@"];
-        if (parts.count != 2) {
-            continue;
-        }
-        iTermHostRecordMO *hostRecord = records[host];
-        if (!hostRecord) {
-            hostRecord = [iTermHostRecordMO hostRecordInContext:_managedObjectContext];
-            records[host] = hostRecord;
-        }
-        hostRecord.username = parts[0];
-        hostRecord.hostname = parts[1];
-        for (NSDictionary *dict in archive[host]) {
-            iTermRecentDirectoryMO *directory =
-                [iTermRecentDirectoryMO entryWithDictionary:dict
-                                                  inContext:_managedObjectContext];
-            [hostRecord addDirectoriesObject:directory];
-            directory.remoteHost = hostRecord;
-        }
-    }
-    NSError *error = nil;
-    if (![_managedObjectContext save:&error]) {
-        NSLog(@"Failed to migrate directory history: %@", error);
-    } else {
-        [[NSFileManager defaultManager] removeItemAtPath:path error:NULL];
-    }
-    return YES;
-}
-
-- (BOOL)migrateCommandHistoryFromPlistToCoreData:(NSMutableDictionary *)records {
-    NSString *path = [self pathToDeprecatedCommandHistoryPlist];
-    NSDictionary *archive = [NSKeyedUnarchiver unarchiveObjectWithFile:path];
-    for (NSString *host in archive) {
-        NSArray *parts = [host componentsSeparatedByString:@"@"];
-        if (parts.count != 2) {
-            continue;
-        }
-        iTermHostRecordMO *hostRecord = records[host];
-        if (!hostRecord) {
-            hostRecord = [iTermHostRecordMO hostRecordInContext:_managedObjectContext];
-            records[host] = hostRecord;
-        }
-        hostRecord.username = parts[0];
-        hostRecord.hostname = parts[1];
-        for (NSDictionary *commandDict in archive[host]) {
-            iTermCommandHistoryEntryMO *managedObject =
-                [iTermCommandHistoryEntryMO commandHistoryEntryFromDeprecatedDictionary:commandDict
-                                                                              inContext:_managedObjectContext];
-            managedObject.remoteHost = hostRecord;
-            [hostRecord addEntriesObject:managedObject];
-        }
-    }
-    NSError *error = nil;
-    if (![_managedObjectContext save:&error]) {
-        NSLog(@"Failed to migrate command history: %@", error);
-    } else {
-        [[NSFileManager defaultManager] removeItemAtPath:self.pathToDeprecatedCommandHistoryPlist error:NULL];
-    }
-    return YES;
-}
-
 #pragma mark - APIs
 
 + (void)showInformationalMessage {
@@ -548,7 +462,7 @@ static const NSTimeInterval kMaxTimeToRememberDirectories = 60 * 60 * 24 * 90;
        inDirectory:(NSString *)directory
           withMark:(VT100ScreenMark *)mark {
     [[NSUserDefaults standardUserDefaults] setBool:YES forKey:kCommandHistoryHasEverBeenUsed];
-    
+
     iTermHostRecordMO *hostRecord = [self recordForHost:host];
     if (!hostRecord) {
         hostRecord = [iTermHostRecordMO hostRecordInContext:_managedObjectContext];
@@ -556,7 +470,7 @@ static const NSTimeInterval kMaxTimeToRememberDirectories = 60 * 60 * 24 * 90;
         hostRecord.username = host.username;
         [self setRecord:hostRecord forHost:host];
     }
-    
+
     iTermCommandHistoryEntryMO *theEntry = nil;
     for (iTermCommandHistoryEntryMO *entry in hostRecord.entries) {
         if ([entry.command isEqualToString:command]) {
@@ -564,16 +478,16 @@ static const NSTimeInterval kMaxTimeToRememberDirectories = 60 * 60 * 24 * 90;
             break;
         }
     }
-    
+
     if (!theEntry) {
         theEntry = [iTermCommandHistoryEntryMO commandHistoryEntryInContext:_managedObjectContext];
         theEntry.command = command;
         [hostRecord addEntriesObject:theEntry];
     }
-    
+
     theEntry.numberOfUses = @(theEntry.numberOfUses.integerValue + 1);
     theEntry.timeOfLastUse = @([self now]);
-    
+
     iTermCommandHistoryCommandUseMO *commandUse =
     [iTermCommandHistoryCommandUseMO commandHistoryCommandUseInContext:_managedObjectContext];
     commandUse.time = theEntry.timeOfLastUse;
@@ -581,7 +495,7 @@ static const NSTimeInterval kMaxTimeToRememberDirectories = 60 * 60 * 24 * 90;
     commandUse.directory = directory;
     commandUse.command = theEntry.command;
     [theEntry addUsesObject:commandUse];
-    
+
     NSString *key = host.key ?: @"";
     if (_expandedCache[key]) {
         [_expandedCache[key] addObject:commandUse];
@@ -615,7 +529,7 @@ static const NSTimeInterval kMaxTimeToRememberDirectories = 60 * 60 * 24 * 90;
     NSMutableArray<iTermCommandHistoryEntryMO *> *result = [NSMutableArray array];
     iTermHostRecordMO *hostRecord = [self recordForHost:host];
     for (iTermCommandHistoryEntryMO *entry in hostRecord.entries) {
-        if (emptyPartialCommand || [entry.command hasPrefix:partialCommand]) {
+        if (emptyPartialCommand || [entry.command caseInsensitiveHasPrefix:partialCommand]) {
             // The FinalTerm algorithm doesn't require |partialCommand| to be a prefix of the
             // history entry, but based on how our autocomplete works, it makes sense to only
             // accept prefixes. Their scoring algorithm is implemented in case this should change.
