@@ -9,8 +9,12 @@
 #import <Cocoa/Cocoa.h>
 
 @interface iTermSizeRememberingView : NSView
-@property(nonatomic, readonly) NSSize originalSize;
+@property(nonatomic) NSSize originalSize;
 
 - (void)resetToOriginalSize;
 
 @end
+
+@interface iTermPrefsProfilesGeneralView : iTermSizeRememberingView
+@end
+
